/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.concurrent.Semaphore;

import org.apache.log4j.Logger;

import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.Commander;
import critiquer.activities.parameters.CommanderParameter;
import critiquer.activities.parameters.DescriptionParameter;
import critiquer.activities.parameters.NameParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.constraints.ConstraintFactory;
import critiquer.constraints.TimeParameterOrdering;
import critiquer.constraints.VisualConstraints;
import critiquer.gui.NameChangeListener;
import critiquer.misc.ReferenceHolder;



/**
 * 
 * Base class for all types of activities in a plan.
 * 
 * @author olale
 *
 */
public abstract class Activity implements Serializable {

	/**
     * 
     * Interface for recognizing state properties of activities during
     * simulation. This is only supposed to be implemented by enums in the
     * specific activity types
     * 
     * @author olale
     * 
     */
	public interface Properties {

	}

	private interface Updater {

		public void update(ActivityUpdateListener l);
	}

	private transient Logger											 logger;

	private static final long								  serialVersionUID		  = 1L;

	private ArrayList<ActivityParameter>					   params;

	private Activity										   parent					= null;

	private ArrayList<Activity>								children				  = new ArrayList<Activity>();

	private transient static ArrayList<ActivityUpdateListener> staticUpdateListeners	 = null;

	public String											  type					  = "Default mission";

	private ArrayList<ActivityUpdateListener>				  myUpdateListeners;

	private HashSet<TimeParameterOrdering>					 temporalConstraints	   = new HashSet<TimeParameterOrdering>();

	private static ArrayList<ActivityConsistencyCheck>		 staticConsistencyCheckers = new ArrayList<ActivityConsistencyCheck>();

	private TimeParameterOrdering							  afterParentStart;

	private TimeParameterOrdering							  beforeParentEnd;

	private static Semaphore								   pendingUpdatesLock		= new Semaphore(1);

	private static int										 pendingUpdates			= 0;

	private transient Semaphore										  atomicUpdateLock		  = new Semaphore(1);

	private TimeParameterOrdering							  internalTimeConstraint;

	/**
     * Acquire the semaphore for pending updates, if there is one available.
     * 
     */
	private void addPendingUpdate() {
		pendingUpdatesLock.tryAcquire();
		pendingUpdates++;
	}

	private void removePendingUpdate() {
		assert (pendingUpdates > 0);
		pendingUpdates--;
		if (pendingUpdates == 0) {
			pendingUpdatesLock.release();
		}
	}

	public int getPendingupdates() {
		return pendingUpdates;
	}

	public void acquirePendingUpdatesLock() {
		try {
			pendingUpdatesLock.acquire();
		} catch (InterruptedException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		}
	}

	public void releasePendingUpdatesLock() {
		pendingUpdatesLock.release();
	}

	/**
     * Each subclass of Activity should have a static counter of how many
     * activities have been created, so that we may create unique new names for
     * newly created activities.
     * 
     * @return the number of objects of this class that have been created.
     */
	public abstract int getNumActivities();

	public Activity getRootActivity() {
		if (parent == null) {
			return this;
		} else {
			return parent.getRootActivity();
		}
	}

	/**
     * Creates a dummy placeholder activity, not to be used in tasks
     */
	public Activity() {
		Logger.getLogger(getClass()).error("Dummy activity created");
	}

	public Activity(Activity parent) {
		init(parent);
	}

	public Activity(String type, Activity parent) {
		this.type = type;
		init(parent);
	}

	private void init(Activity parent) {
		ReferenceHolder.allActivities.add(this);
		logger = Logger.getLogger(getClass());
		addDefaultParams();
		internalTimeConstraint = ConstraintFactory.connectStartToEnd(this);
		initParentRelationship(parent);
		signalActivityCreated();
	}

	/**
     * 
     * Called when an activity is added to an overarching activity.
     * 
     * @param parent
     */
	public void initParentRelationship(Activity parent) {
		this.parent = parent;
		if (parent != null) {
			afterParentStart = ConstraintFactory.connectStartToStart(parent, this);
			VisualConstraints.addConstraint(afterParentStart);
			beforeParentEnd = ConstraintFactory.connectEndToEnd(parent, this);
			VisualConstraints.addConstraint(beforeParentEnd);
			Commander commander = ActivityUtils.getCommander(parent);
			if (commander != null) {
				commander.addActivityToAgentMappings(this);
			}
			parent.addChild(this);
		}
		signalActivityUpdated();
	}

	/**
     * This method indicates whether or not an activity can be assigned an agent
     * or if it should inherit agents from parent activities. It is determined
     * by whether or not there is a commander assigned to the activity.
     * 
     */
	public boolean isAgentAssignable() {
		CommanderParameter param = (CommanderParameter) ActivityUtils.getParamByClassAndName(this,
				CommanderParameter.class,
				"commander");
		return (param != null && param.isEnabled());
	}

	/**
     * Decouple this agent from its parent
     * 
     */
	public void endParentRelationship() {

		if (parent != null) {
			parent.removeChild(this);
			Commander commander = ActivityUtils.getCommander(parent);
			if (commander != null) {
				commander.removeActivityFromAgentMappings(this);
			}
			this.parent = null;
		}
		afterParentStart.disableConstraint();
		VisualConstraints.removeConstraint(afterParentStart);
		beforeParentEnd.disableConstraint();
		VisualConstraints.removeConstraint(beforeParentEnd);
		signalActivityUpdated();
	}

	public static void addActivityUpdateListener(ActivityUpdateListener ul) {
		if (!Activity.getStaticUpdateListeners().contains(ul)) {
			Activity.getStaticUpdateListeners().add(ul);
		} else {
			Logger.getLogger(Activity.class).info("Note: Trying to att update listener " + ul + " twice");
		}
		/**
         * Ensure that modifying listeners are notified before passive ones
         */
		Collections.sort(getStaticUpdateListeners());
	}

	/**
     * @return Returns the params.
     */
	public ArrayList<ActivityParameter> getParams() {
		if (params == null) {
			params = new ArrayList<ActivityParameter>();
		}
		return params;
	}

	public <T> void addParam(ActivityParameter<T> p) {
		getParams().add(p);
		signalParamAdded(p);
	}

	public static <T> boolean isConsistent(Activity a, Parameter<T> p, T newValue) {
		boolean consistent = true;
		for (ActivityConsistencyCheck check : staticConsistencyCheckers) {
			if (!check.paramValueCheck(a, p, newValue)) {
				consistent = false;
				break;
			}
		}
		return consistent;
	}

	/**
     * 
     * TODO: We probably need to remove synchronization requirements from this
     * method or drop the semaphore in ActivityParameter..
     * 
     * @param u
     */
	private synchronized void notifyAllActivityListeners(Updater u) {
		/**
         * We are processing the updater, so reduce the "pending" count
         */
		removePendingUpdate();
		/**
         * First notify all generic listeners, then those specific to this
         * instance
         */
		for (ActivityUpdateListener listener : Activity.getStaticUpdateListeners()) {
			u.update(listener);
		}

		for (ActivityUpdateListener listener : getMyUpdateListeners()) {
			u.update(listener);
		}
	}

	/**
     * create a new activity parameter nobody should be interested in per se and
     * fire a paramChanged event with that parameter
     */
	public void signalParamChanged() {
		ActivityParameter<String> dummyParam = new ActivityParameter<String>("dummy name", this) {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			@Override
			public String toString() {
				return null;
			}

		};
		dummyParam.setValue("dummy value");
	}

	private void signalActivityCreated() {
		addPendingUpdate();
		notifyAllActivityListeners(new Updater() {

			public void update(ActivityUpdateListener l) {
				l.activityCreated(Activity.this);
			}

		});

	}

	/**
     * 
     * When the activity is updated in some way, but not through parameter
     * values, this method is called
     * 
     */

	public void signalActivityUpdated() {
		addPendingUpdate();
		notifyAllActivityListeners(new Updater() {

			public void update(ActivityUpdateListener l) {
				l.activityUpdated(Activity.this);
			}

		});

	}

	/**
     * Notify activity listeners that this activity has been removed from the
     * plan
     * 
     */
	public void signalActivityRemoved() {
		addPendingUpdate();
		notifyAllActivityListeners(new Updater() {

			public void update(ActivityUpdateListener l) {
				l.activityRemoved(Activity.this);
			}

		});

	}

	private <T> void signalParamAdded(final Parameter<T> p) {
		addPendingUpdate();
		notifyAllActivityListeners(new Updater() {

			public void update(ActivityUpdateListener l) {
				l.paramAdded(Activity.this, p);
			}

		});

	}

	public <T> void signalParamChanged(final Parameter<T> p) {
		addPendingUpdate();
		notifyAllActivityListeners(new Updater() {

			public void update(ActivityUpdateListener l) {
				l.paramChanged(Activity.this, p);
			}

		});

	}

	/**
     * Adds an activity update listener
     * 
     * @param l
     */
	public void addPrivateUpdateListener(ActivityUpdateListener l) {
		myUpdateListeners.add(l);
		Collections.sort(myUpdateListeners);
	}

	public void removePrivateUpdateListener(ActivityUpdateListener l) {
		getMyUpdateListeners().remove(l);
	}

	public String toString() {
		return ActivityUtils.getNameParameter(this).toString();
	}

	public boolean hasParent() {
		return getParent() != null;
	}

	/**
     * @return Returns the parent.
     */
	public Activity getParent() {
		return parent;
	}

	/**
     * @param parent
     *            The parent to set.
     */
	public void setParent(Activity parent) {
		if (this.parent != null) {
			endParentRelationship();
		}
		initParentRelationship(parent);
	}

	/**
     * Add name, description, start and end time for all activities
     */
	public void addDefaultParams() {
		addParam(new NameParameter(this));
		addParam(new DescriptionParameter(this));
		ActivityUtils.addCommanderParameter(this);
		ActivityUtils.addMaxTimeParameter(this);

		// Add time parameters
		TimeParameter start = new TimeParameter("start time", TimeParameter.Type.START, this);
		addParam(start);
		TimeParameter end = new TimeParameter("end time", TimeParameter.Type.END, this);
		Calendar endTime = (Calendar) start.getValue().clone();
		endTime.add(Calendar.HOUR_OF_DAY, 1);
		end.setValue(endTime, true);
		addParam(end);
	}

	public void addChild(Activity child) {
		getChildren().add(child);
		signalActivityUpdated();
	}

	public void removeChild(Activity currentActivity) {
		getChildren().remove(currentActivity);
		signalActivityUpdated();
	}

	/**
     * @return Returns the children.
     */
	public ArrayList<Activity> getChildren() {
		if (children == null) {
			children = new ArrayList<Activity>();
		}
		return children;
	}

	public void addAll(ArrayList<Activity> c) {
		getChildren().addAll(c);
		signalActivityUpdated();
	}

	public void removeChildren() {
		getChildren().clear();
		signalActivityUpdated();
	}

	public Date getStartTime() {
		TimeParameter startParam = ActivityUtils.getStartTimeParameter(this);
		if (startParam != null) {
			return startParam.getValue().getTime();
		}
		return null;
	}

	public Date getEndTime() {
		TimeParameter endParam = ActivityUtils.getEndTimeParameter(this);
		if (endParam != null) {
			return endParam.getValue().getTime();
		}
		return null;

	}

	public static void addNameChangeListener(final NameChangeListener listener) {
		AbstractParamChangedListener nameChangeListener = new AbstractParamChangedListener() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			public <T> void paramChanged(Activity activity, Parameter<T> p) {
				if (p instanceof NameParameter) {
					listener.nameChanged();
				}

			}

		};
		Activity.getStaticUpdateListeners().add(nameChangeListener);
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
	@Override
	public boolean equals(Object o) {
		return o != null && o instanceof Activity && toString().equals(o.toString());
	}

	/**
     * @return Returns the staticUpdateListeners.
     */
	public static ArrayList<ActivityUpdateListener> getStaticUpdateListeners() {
		if (Activity.staticUpdateListeners == null) {
			Activity.staticUpdateListeners = new ArrayList<ActivityUpdateListener>();
		}
		return Activity.staticUpdateListeners;
	}

	/**
     * @return Returns the staticUpdateListeners.
     */
	ArrayList<ActivityUpdateListener> getMyUpdateListeners() {
		if (myUpdateListeners == null) {
			myUpdateListeners = new ArrayList<ActivityUpdateListener>();
		}
		return myUpdateListeners;
	}

	public int indexOfChild(Activity activity) {
		return getChildren().indexOf(activity);
	}

	/**
     * 
     * Calculates the effects of performing a mission with respect to some
     * state. This is supposed to be used during simulation.
     * 
     * @param
     */
	public abstract long simulate() throws IncompleteActivityException;

	/**
     * Test if this activity can be simulated. This usually entails checking
     * that some structural constraints hold. It is assumed that this method is
     * checked before any simulation or time calculation takes place.
     * 
     * @return
     */
	public abstract boolean canBeSimulated();

	public long getMaxTime() {
		return ActivityUtils.getMaxTimeParameter(this).getValue();
	}

	/**
     * <p>
     * This function is called when an activity is deserialized.
     * </p>
     */
	public void postLoad() {
		signalActivityCreated();
	}

	/**
     * @return Returns the temporalConstraints.
     */
	public HashSet<TimeParameterOrdering> getTemporalConstraints() {
		return temporalConstraints;
	}

	public void preSave() {

	}

	/**
     * <p>
     * Remove the temporal ordering between the current activity and a2
     * </p>
     * <p>
     * TODO: This could be an instance method so that additional ordering
     * information could be added by subclasses, if location parameters need to
     * be syncronized for instance
     * </p>
     * 
     * @param laterActivity
     *            the activity that was ordered after <code>this</code>
     * 
     * @see critiquer.activities.Activity#order(critiquer.activities.Activity)
     *      for more information.
     * 
     */
	public void removeOrder(Activity laterActivity) {
		TimeParameterOrdering toBeRemoved = null;
		for (TimeParameterOrdering ordering : ReferenceHolder.temporalConstraints) {
			if (ordering.getT1().getActivity().equals(this) && ordering.getT2().getActivity().equals(laterActivity)) {
				toBeRemoved = ordering;
				break;
			}
		}

		if (toBeRemoved != null) {
			toBeRemoved.disableConstraint();
			ReferenceHolder.temporalConstraints.remove(toBeRemoved);
			VisualConstraints.removeConstraint(toBeRemoved);
		}
		signalActivityUpdated();
		laterActivity.signalActivityUpdated();
	}

	/**
     * <p>
     * Introduce an ordering between two activities by ordering the end time
     * parameter of the current activity before the start time parameter of a2.
     * </p>
     * TODO: Possibly add other constraints that relate to ordering two
     * consecutive actions
     * 
     * 
     * @param laterActivity
     *            The activity that should come after <code>this</code>
     */
	public void order(Activity laterActivity) {
		TimeParameterOrdering order = ConstraintFactory.connectEndToStart(this, laterActivity);
		if (!ReferenceHolder.temporalConstraints.contains(order)) {
			ReferenceHolder.temporalConstraints.add(order);
			VisualConstraints.addConstraint(order);
		}
		signalActivityUpdated();
		laterActivity.signalActivityUpdated();
	}

	/**
     * Can this activity be a child of parent?
     */
	public boolean canBeChildOf(Activity parent) {
		boolean ok = true;
		for (ActivityConsistencyCheck consistencyChecker : staticConsistencyCheckers) {
			ok = consistencyChecker.childOfCheck(parent, this);
			if (!ok) {
				break;
			}
		}
		return ok;
	}

	/**
     * Can this activity come before activity in a mission?
     */
	public boolean canComeBefore(Activity activity) {
		boolean ok = true;
		for (ActivityConsistencyCheck consistencyChecker : staticConsistencyCheckers) {
			ok = consistencyChecker.orderingCheck(this, activity);
			if (!ok) {
				break;
			}
		}
		return ok;
	}

	/**
     * @return Returns the staticConsistencyCheckers.
     */
	public static ArrayList<ActivityConsistencyCheck> getStaticConsistencyCheckers() {
		return staticConsistencyCheckers;
	}

	/**
     * @param checker
     *            The ActivityConsistencyChecker to add.
     */
	public static void addStaticConsistencyChecker(ActivityConsistencyCheck checker) {
		Activity.staticConsistencyCheckers.add(checker);

	}

	/**
     * @return Returns the atomicUpdateLock.
     */
	public Semaphore getAtomicUpdateLock() {
		return atomicUpdateLock;
	}

	
}
