/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Vector;

import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.AgentParameter;
import critiquer.activities.parameters.Commander;
import critiquer.activities.parameters.CommanderParameter;
import critiquer.activities.parameters.NameParameter;
import critiquer.activities.parameters.NumberSelectionParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.constraints.TimeParameterOrdering;
import critiquer.misc.Predicate;
import critiquer.misc.ReferenceHolder;
import critiquer.misc.Utils;
import critiquer.resources.Agent;



public class ActivityUtils implements Serializable {

	/**
     * 
     */
	private static final String COMMANDER		= "commander";

	/**
     * 
     */
	private static final String MAX_TIME		 = "max time";

	/**
     * 
     */
	private static final long   serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	public static <T extends ActivityParameter> ArrayList<T> filterParams(Activity a, final Class<T> c) {
		ArrayList<T> l = new ArrayList<T>();
		for (ActivityParameter p : a.getParams()) {
			if (c.isInstance(p)) {
				l.add((T) p);
			}
		}
		return l;
	}

	public static boolean agentInActivity(Agent agent, Activity activity) {
		return getAgentsInActivity(activity).contains(agent);
	}

	public static HashSet<Agent> getAgentsInActivity(Activity a) {
		HashSet<Agent> agents = new HashSet<Agent>();
		AgentParameter agentParam = getMainAgentParameter(a);
		if (agentParam != null) {
			HashSet<Agent> agentsInActivity = agentParam.getCommander().getAgentsInActivity(a);
			if (agentsInActivity!=null) {
				agents.addAll(agentsInActivity);
				
			}
		}
		return agents;
	}

	public static NameParameter getNameParameter(Activity a) {
		ArrayList<NameParameter> l = filterParams(a, NameParameter.class);
		return l.get(0);
	}

	/**
     * Retrieves the first time interval parameter
     */

	public static TimeParameter getStartTimeParameter(Activity a) {
		return Utils.find(filterParams(a, TimeParameter.class), new Predicate<TimeParameter>() {

			public boolean check(TimeParameter obj) {
				return obj.getType() == TimeParameter.Type.START;
			}

		});

	}

	public static TimeParameter getEndTimeParameter(Activity a) {
		return Utils.find(filterParams(a, TimeParameter.class), new Predicate<TimeParameter>() {

			public boolean check(TimeParameter obj) {
				return obj.getType() == TimeParameter.Type.END;
			}

		});
	}

	public static Commander getCommander(Activity activity) {
		if (activity.isAgentAssignable()) {
			/**
             * There is a commander associated with this activity
             */
			CommanderParameter commander = getCommanderParameter(activity);
			return commander.getValue();
		} else if (activity.hasParent()) {
			return getCommander(activity.getParent());
		} else {
			return null;
		}
	}

	public static AgentParameter getMainAgentParameter(Activity a) {
		Commander commander = getCommander(a);
		return (commander != null ? commander.getAgentParameter() : null);
	}

	public static ActivityParameter getParamByName(Activity a, final String name) {
		ArrayList<ActivityParameter> params = a.getParams();
		return Utils.find(params, new Predicate<ActivityParameter>() {

			public boolean check(ActivityParameter param) {
				return param.getName().equals(name);
			}

		});
	}

	public static <T extends ActivityParameter> T getParamByClassAndName(Activity a, Class<T> c, final String name) {
		ArrayList<T> paramsByClass = ActivityUtils.filterParams(a, c);
		return Utils.find(paramsByClass, new Predicate<T>() {

			public boolean check(T param) {
				return param.getName().equals(name);
			}

		});
	}

	/**
     * Returns the current distance between the values of the start time
     * parameter and the end time parameter
     * 
     * @param activity
     * @return a time difference in minutes
     */
	public static Long getCurrentTimeForActivity(Activity activity) {
		Long returnValue = new Long(0);
		TimeParameter end = getEndTimeParameter(activity);
		TimeParameter start = getStartTimeParameter(activity);
		if (!(end == null || start == null)) {
			returnValue = end.getValue().getTimeInMillis() - start.getValue().getTimeInMillis();

		}
		assert returnValue > 0;
		return returnValue / 60000;
	}

	public static void addMaxTimeParameter(Activity activity) {

		Vector<Integer> numbers = new Vector<Integer>();
		for (int i = 0; i < 300; i += 10) {
			numbers.add(i);
		}
		NumberSelectionParameter maxTimeParam = new NumberSelectionParameter(activity, MAX_TIME, numbers, 60);
		activity.addParam(maxTimeParam);

	}

	public static NumberSelectionParameter getMaxTimeParameter(Activity activity) {
		return getParamByClassAndName(activity, NumberSelectionParameter.class, MAX_TIME);
	}

	public static void addCommanderParameter(Activity activity) {
		activity.addParam(new CommanderParameter(COMMANDER, activity));
	}

	public static CommanderParameter getCommanderParameter(Activity a) {
		return getParamByClassAndName(a, CommanderParameter.class, COMMANDER);
	}

	/**
     * Set the missionTimeChanged flag in the activity and signal a new
     * paramChanged event to all interested listeners whenever parameter
     * <code>param</code> changes value.
     * 
     * @param param -
     *            The parameter which can change the mission time
     */
	public static AbstractParamChangedListener createMissionTimeUpdater(final ActivityParameter param) {
		return new AbstractParamChangedListener() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			@Override
			public <T> void paramChanged(Activity activity, Parameter<T> p) {
				if (p == param) {
					activity.signalParamChanged();
					
				}

			}

		};
	}

	@SuppressWarnings("unchecked")
	public static void removeConstraintsForActivity(Activity currentActivity) {
		ArrayList<TimeParameterOrdering> temporalConstraints = (ArrayList<TimeParameterOrdering>) ReferenceHolder.temporalConstraints.clone();
		for (TimeParameterOrdering ordering : temporalConstraints) {
			if (ordering.getT1().getActivity().equals(currentActivity) || ordering.getT2()
																				  .getActivity()
																				  .equals(currentActivity)) {
				ReferenceHolder.temporalConstraints.remove(ordering);
			}
		}
	}

	/**
     * Checks if two activities overlap in time
     * 
     * @return true iff there is an overlap in time
     */

	public static boolean overlap(Activity a1, Activity a2) {
		Calendar start1 = getStartTimeParameter(a1).getValue();
		Calendar start2 = getStartTimeParameter(a2).getValue();
		Calendar end1 = getEndTimeParameter(a1).getValue();
		Calendar end2 = getEndTimeParameter(a2).getValue();
		Calendar lastStart = (start1.after(start2) ? start1 : start2);
		Calendar firstEnd = (end1.before(end2) ? end1 : end2);
		return lastStart.before(firstEnd);

	}

	public static Activity getActivityByName(final Comparable name) {
		if (!(name instanceof String)) {
			return null;
		}
		return Utils.find(ReferenceHolder.allActivities, new Predicate<Activity>() {

			public boolean check(Activity activity) {
				return activity.toString().equals(name);
			}

		});
	}

}
