/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.io.Serializable;
import java.util.concurrent.Semaphore;

import critiquer.activities.Activity;


/**
 * <p>
 * This class describes the parameters that describe an activity
 * </p>
 * <p>
 * When creating a plan, we start by choosing the type of activity which in turn
 * gives us a template hierarchy of activities that we can start to manipulate
 * </p>
 * <p>
 * This structure contains textual information on missions and possibly time
 * dependencies.
 * </p>
 * <p>
 * Next, resources are assigned to missions in the resource view by dragging
 * resources to missions
 * </p>
 * <p>
 * When this is done, time intervals for each mission can be calculated in the
 * time view, according to one of several preconfigured settings (launch
 * missions immediately after one another, set certain time slacks)
 * </p>
 * <p>
 * Therefore, an activity parameter has a view associated with it where one can
 * configure the parameter
 * </p>
 */
public abstract class ActivityParameter<T> implements Parameter<T>, Serializable {

	protected T		 value	   = null;

	protected T		 oldValue	= null;

	/**
     * If the user has ever set the value of this parameter, this will be true,
     * otherwise false.
     */
	protected boolean   hasBeenSet  = false;

	private Activity	activity;

	protected Semaphore valueUpdate = new Semaphore(1, true);

	protected String	name;

	/**
     * @param activity
     */
	public ActivityParameter(String name, Activity activity) {
		this.activity = activity;
		this.name = name;
	}

	public abstract String toString();

	/**
     * @return Returns the value.
     */
	public T getValue() {
		return value;
	}

	public boolean hasValue() {
		return value != null && hasBeenSet;
	}

	/**
     * This method can be used directly by subclasses if the parameter value
     * reference remaing the same but there has been changes to the value so
     * listeners should be notified
     * 
     * Also, we create a new thread to do the updating here since this may be
     * called during the process of updating a parameter.
     * 
     * 
     */

	protected void valueChanged() {
		new Thread("\"" + activity + "\":\"" + name + "\" update thread") {

			@Override
			public void run() {
				activity.signalParamChanged(ActivityParameter.this);

			}

		}.start();

	}

	/**
     * 
     * Update the value of this parameter iff the value is consistent for this
     * parameter The updating is performed with a binary semaphore set so that
     * there can be no more than one thread updating and signalling activities
     * of a value update
     * 
     * @param value
     *            The value to set. Sets a value and notifies all listeners
     * @return true if the change was consistent, false otherwise.
     */
	public boolean setValue(T value) {
		return setValue(value, false);
	}

	/**
     * 
     * This may be used when we don't want to notify activities of the change.
     * This should primarily be used in the constructor of the parameter when
     * some default value is set.
     * 
     * @param value
     * @param silent
     * @return
     */
	public boolean setValue(T value, boolean silent) {
		boolean consistent = false;
		consistent = Activity.isConsistent(getActivity(), this, value);
		if (consistent) {
			oldValue = this.value;
			this.value = value;
			hasBeenSet = true;
			if (!silent) {
				valueChanged();
			}
		}
		return consistent;
	}

	/**
     * @return Returns the name.
     */
	public String getName() {
		return name;
	}

	/**
     * @param name
     *            The name to set.
     */
	void setName(String name) {
		this.name = name;
	}

	/**
     * @return Returns the activity.
     */
	public Activity getActivity() {
		return activity;
	}

	/**
     * @return Returns the oldValue.
     */
	public T getOldValue() {
		return oldValue;
	}

}
