/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.JPanel;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.gui.AgentLabel;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;


/**
 * 
 * <p>
 * One or several agents that are supposed to perform some action are collected
 * by an AgentParameter. The AgentParameter is also capable of producing a
 * graphical panel presenting those agents. The AgentParameter is connected to a
 * Commander, which in turn is connected to one or several activities through
 * CommanderParameters of the activities. Therefore this is not an
 * ActivityParameter but only a Parameter.
 * </p>
 * 
 * <p>
 * This parameter is special in that it doesn't contain a value in itself but
 * relies on a mapping between activities and agents that the commander has.
 * </p>
 * 
 * The reason for this is that ...
 * 
 */

public class AgentParameter implements AgentChangedListener, Parameter<HashSet<Agent>>, Serializable {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	private transient JPanel  pane;

	private Commander		 commander;

	public JPanel getAgentPanel() {
		if (pane == null) {
			pane = new JPanel(new GridLayout(0,3));
			for (Agent agent : getAllAgents()) {
				AgentLabel label = agent.getLabel(this);
				pane.add(label);
			}
		}
		return pane;
	}
	/**
	 * 
	 * @param agent
	 * @return a point at the center of the component that the agent is represented by
	 */
	
	public Point getAgentLocation(Agent agent) {
		Component c = null;
		Point center = new Point();
		try {
			Vector<Agent> allAgents = getAllAgents();
			c = getAgentPanel().getComponent(allAgents.indexOf(agent));
			Rectangle rect = c.getBounds();
			center.x = (int) rect.getCenterX();
			center.y = (int) rect.getCenterY();

		} catch (Exception e) {

			Logger.getLogger(getClass()).error("No label for agent " + agent);
			e.printStackTrace();
		}
		return center;
	}

	/**
     * 
     * <p>We are not supposed to use this method for the agent parameter since the
     * Commander keeps track of all agent-activity mappings.
     * </p> 
     * 
     * <p>AgentParameter objects are not connected to a single activity either so
     * it makes no sense asking for the value of this parameter
     * </p> 
     * 
     * <p>To obtain a list of the agents taking part of an activity, use
     * 
     * @link critiquer.activities.ActivityUtils#getAgentsInActivity(Activity)
     *       instead.
     * </p>
     * 
     * @see critiquer.activities.parameters.ActivityParameter#getValue()
     */
	public HashSet<Agent> getValue() {
		throw new UnsupportedOperationException("AgentParameter.getValue() is not supported");
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.parameters.ActivityParameter#getValue()
     */
	public boolean hasValue() {
		return !commander.getSubordinates().isEmpty();
	}

	/**
     * 
     * Check if it is ok to add this agent to the list of agents and do so if
     * possible.
     * 
     * @param a
     * @return true iff the agent can be selected.
     * 
     */

	@SuppressWarnings("unchecked")
	public boolean selectAgent(Agent a) {
		HashSet<Agent> agents = (HashSet<Agent>) commander.getSubordinates().clone();
		agents.add(a);
		boolean valueOk = Activity.isConsistent(null, this, agents);
		if (valueOk) {
			// Let the activity know when the agent is changed

			a.addAgentChangedListener(this);
			agentChanged(a);
			commander.agentAdded(a);
		}
		return valueOk;
	}

	/**
     * 
     * Check with the consistency checkers in Activity if it is OK to remove
     * this agent and remove it in such case.
     * 
     * @param agent
     */
	@SuppressWarnings("unchecked")
	public boolean deSelectAgent(Agent agent, Activity activity) {
		HashSet<Agent> agents = (HashSet<Agent>) commander.getSubordinates().clone();
		agents.remove(agent);
		boolean valueOk = Activity.isConsistent(activity, this, agents);
		if (valueOk) {

			agent.removeAgentChangedListener(this);
			agentChanged(agent);
			commander.agentRemoved(activity, agent);
		}
		return valueOk;
	}

	public AgentParameter(String string, Commander commander) {
		this.commander = commander;
	}

	public String toString() {
		return "Select agents";
	}

	private Vector<Agent> getAllAgents() {
		return ReferenceHolder.organizationModel.getAllAgents();
	}

	/**
     * Whenever parameters that describe the agent change, propagate that
     * information to the activity containing this agent
     */
	public void agentChanged(Agent agent) {
		commander.notifyActivities();
	}

	/**
     * @return Returns the commander.
     */
	public Commander getCommander() {
		return commander;
	}

}
