/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Stack;

import critiquer.activities.Activity;
import critiquer.resources.Agent;


/**
 * 
 * A commander has a list of activities for which this commander is responsible.
 * Whenever choices are made in the CommanderParameter, Commander objects are
 * notified that they should update their information on which activities they
 * are responsible for.
 * 
 * Agents are added to commanding activities, but commanders may choose not to
 * use agents for specific tasks. Therefore it is possible to remove agents on a
 * per-activity basis.
 * 
 * To enable a more fine-grained control over which activities are associated
 * with which agents, agents can be removed from individual sub-activities that
 * this commander controls,
 * 
 * @author olale
 * 
 */
public class Commander implements Serializable {

	/**
     * 
     */
	private static final long				 serialVersionUID = 1L;

	private String							name;

	private AgentParameter					agentParameter;

	/**
	 */
	private HashMap<Activity, HashSet<Agent>> agentMappings	= new HashMap<Activity, HashSet<Agent>>();

	private ArrayList<Activity>			   activities;

	public void agentAdded(Agent a) {
		ArrayList<Activity> allActivities = getAllActivities();
		for (Activity activity : allActivities) {
			addMapping(activity, a);
		}
	}

	public void addMapping(Activity activity, Agent agent) {
		if (agentMappings.containsKey(activity)) {
			agentMappings.get(activity).add(agent);
		} else {
			HashSet<Agent> map = new HashSet<Agent>();
			map.add(agent);
			agentMappings.put(activity, map);
		}
	}

	public HashSet<Agent> getAgentsInActivity(Activity activity) {
		return agentMappings.get(activity);
	}

	public void agentRemoved(Activity activity, Agent agent) {
		if (agentMappings.containsKey(activity)) {
			agentMappings.get(activity).remove(agent);
		}
	}

	public Commander(String name) {
		this.name = name;
		this.activities = new ArrayList<Activity>();
		agentParameter = new AgentParameter("main agent", this);
	}

	/**
     * Add an activity directly controlled by this commander
     * 
     * @param a
     */
	public void addActivity(Activity a) {
		activities.add(a);
		addActivityToAgentMappings(a);
	}

	@SuppressWarnings("unchecked")
	public void addActivityToAgentMappings(Activity a) {
		ArrayList<Activity> newActivities = getActivitiesFrom(a);
		for (Activity activity : newActivities) {
			agentMappings.put(activity, (HashSet<Agent>) getSubordinates().clone());
		}
	}

	/**
     * Used when an activity that had this commander get a different one (or
     * none at all)
     * 
     * @param a
     */
	public void removeActivity(Activity a) {
		activities.remove(a);
		removeActivityFromAgentMappings(a);
	}

	/**
     * 
     * Used when an activity is no longer among the indirectly controlled
     * activities under this commander. This happens when inheritance has changed.
     * 
     * @param a The activity which has changed inheritance.
     */
	public void removeActivityFromAgentMappings(Activity a) {
		ArrayList<Activity> activitiesToRemove = getActivitiesFrom(a);
		for (Activity activity : activitiesToRemove) {
			agentMappings.remove(activity);
		}
	}

	@Override
	public String toString() {
		return getName();
	}

	/**
     * @return Returns the name.
     */
	public String getName() {
		return name;
	}

	/**
     * @return Returns the agents that this commanders controls.
     */
	public HashSet<Agent> getSubordinates() {
		HashSet<Agent> myAgents = new HashSet<Agent>();
		Collection<HashSet<Agent>> agentSets = agentMappings.values();
		for (HashSet<Agent> set : agentSets) {
			myAgents.addAll(set);
		}

		return myAgents;
	}

	/**
     * @return Returns the agentParameter.
     */
	public AgentParameter getAgentParameter() {
		return agentParameter;
	}

	/**
     * get the activities that have this commander as the value of their
     * CommanderParameter.
     * 
     * @return Returns the activities.
     */
	public ArrayList<Activity> getActivities() {
		return activities;
	}

	/**
     * Recurse through the activity forest and return a list of all activities.
     * 
     * @return
     */
	public ArrayList<Activity> getAllActivities() {
		Stack<Activity> queue = new Stack<Activity>();
		queue.addAll(activities);
		return recursivelyListActivities(queue);
	}

	/**
     * Recurse through the activity tree starting at rootActivity and return a
     * list of all activities.
     * 
     * @return
     */

	public ArrayList<Activity> getActivitiesFrom(Activity rootActivity) {
		Stack<Activity> queue = new Stack<Activity>();
		queue.push(rootActivity);
		return recursivelyListActivities(queue);
	}

	private ArrayList<Activity> recursivelyListActivities(Stack<Activity> queue) {
		ArrayList<Activity> allActivities = new ArrayList<Activity>();
		while (!queue.isEmpty()) {
			Activity activity = queue.pop();
			allActivities.add(activity);
			queue.addAll(activity.getChildren());
		}

		return allActivities;
	}

	/**
     * There is no special updating to do other than related to time
     * 
     * We must remember to update the dependent activities as well however
     * 
     */
	public void notifyActivities() {
		for (Activity activity : getAllActivities()) {
			activity.signalParamChanged(agentParameter);
		}
	}

}
