/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.awt.Rectangle;
import java.io.Serializable;
import java.util.ArrayList;

import critiquer.misc.ReferenceHolder;


/**
 * A geographical location is modelled using this class. There may be one or
 * more location parameters that have this Location as their value. They are all
 * notified when the location changes value.
 * 
 * @author olale
 * 
 */
public class Location implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private Rectangle					area		   = null;

	private ArrayList<LocationParameter> locationParams = new ArrayList<LocationParameter>();

	public Location(LocationParameter param) {
		locationParams.add(param);
	}

	public void addParam(LocationParameter param) {
		locationParams.add(param);
	}

	public void removeParam(LocationParameter param) {
		locationParams.remove(param);
	}

	/**
     * @return Returns the location parameters that this location is connected
     *         to.
     */
	public ArrayList<LocationParameter> getLocationParams() {
		return locationParams;
	}

	/**
     * Distance in meters between two locations. This takes the scale of the
     * geographical view into account.
     * 
     * @param loc
     *            the location to calculate distance to.
     * @return a distance in meters
     */
	public double distanceTo(Location loc) {
		int dx = loc.getArea().x - area.x;
		int dy = loc.getArea().y - area.y;
		return Math.sqrt(dx * dx + dy * dy) * ReferenceHolder.simulationView.getScale();
	}

	/**
     * @return Returns the area.
     */
	public Rectangle getArea() {
		return area;
	}

	/**
     * @param area
     *            The area to set.
     */
	public void setArea(Rectangle area) {
		this.area = area;
		for (LocationParameter parameter : locationParams) {
			parameter.hasBeenSet = true;
			parameter.valueChanged();
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
	@Override
	public boolean equals(Object obj) {
		boolean eq = false;
		if (obj instanceof Location) {
			Location loc = (Location) obj;
			eq = loc.toString().equals(toString());
		}
		return eq;
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
	@Override
	public String toString() {
		StringBuffer s = new StringBuffer();
		ArrayList<LocationParameter> locationParameters = locationParams;
		boolean first = true;
		for (LocationParameter parameter : locationParameters) {
			s.append((first ?"":"\nand ") + parameter.getName() + " of " + parameter.getActivity());
			if (first) {
				first = false;
			}
		}
		return s.toString();
	}

	/**
     * 
     * Ensure that locations with the same name have the same hash value. This
     * should not really be necessary though, since there should be no need for
     * two different locations with the same name.
     * 
     * @see java.lang.Object#hashCode()
     */
	@Override
	public int hashCode() {
		return toString().hashCode();
	}

}
