/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.communication;

import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Set;

import javax.swing.JOptionPane;

import jxl.peermi.DiscoveryListener;
import jxl.peermi.PeerManager;
import jxl.peermi.PeerRegistry;
import jxl.peermi.PeerRemote;
import jxl.peermi.PeerRemoteObject;

import org.apache.log4j.Logger;

import critiquer.gui.ComPlan;
import critiquer.misc.Utils;

/**
 * 
 * Manage peer planners
 * 
 * @author olale
 * 
 */
public class Communicator {

	public static HashMap<Class<? extends RemotePlanListener>, RemotePlanListener> remotePlannerConstraints = new HashMap<Class<? extends RemotePlanListener>, RemotePlanListener>();

	static {
		// Just add classes for now, initialize later
		remotePlannerConstraints.put(RemoteAgentMutexConstraint.class, null);
	}

	private transient static Logger												logger				   = Logger.getLogger(Communicator.class);

	private final class Initializer implements Runnable {

		/**
		 * 
		 * Initialize plan proxy, look up all peers and add the remote plan
		 * listeners to them
		 * 
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			try {
				if (System.getProperty("java.rmi.server.hostname") == null) {
					System.setProperty("java.rmi.server.hostname", InetAddress.getLocalHost().getHostAddress());
				}
				logger.debug("RMI Server hostaddress: " + System.getProperty("java.rmi.server.hostname"));

				localRegistry = PeerRegistry.getLocalInstance();
				registerObject(planProxy);
				final String planProxyName = PlanProxy.class.getName();
				localRegistry.addDiscoveryListener(planProxyName, new DiscoveryListener() {

					public void discovery(PeerRemote remote) {
						if (remote instanceof RemotePlan) {
							RemotePlan remotePlan = (RemotePlan) remote;
							String message = null;
							try {
								message = "New planner " + remotePlan.getName() + " found on network";
								registerAllListeners(remotePlan);
							} catch (RemoteException exception) {
								message = exception.getMessage();
							}
							logger.debug(message);
							JOptionPane.showMessageDialog(ComPlan.getPlanningTool(), message);
						}
					}

				});
				initRemoteConstraints();
				remotePlanners = localRegistry.requestPeerManager(planProxyName, RemotePlan.class);
				registerAllListeners();
			} catch (Exception e) {
				logger.error(e);
			}

		}

		/**
		 * Register all plan listeners with all planners
		 * 
		 */
		private void registerAllListeners() {
			Collection<RemotePlanListener> remoteConstraints = remotePlannerConstraints.values();
			for (final RemotePlanListener listener : remoteConstraints) {
				registerListener(listener);
			}
		}

		/**
		 * 
		 * Register all listeners with a specific remote planner
		 * 
		 * @param planner
		 *            the planner to register listeners with
		 */
		private void registerAllListeners(RemotePlan planner) {
			Collection<RemotePlanListener> remoteConstraints = remotePlannerConstraints.values();
			for (final RemotePlanListener listener : remoteConstraints) {
				registerListener(listener, planner);
			}
		}

		/**
		 * Register a listener with all remote planners
		 * 
		 * @param listener
		 */
		private void registerListener(final RemotePlanListener listener) {
			for (final RemotePlan planner : remotePlanners) {
				registerListener(listener, planner);
			}
		}

		/**
		 * 
		 * Register a listener with a specific remote planner
		 * 
		 * @param listener
		 * @param planner
		 */
		private void registerListener(final RemotePlanListener listener, final RemotePlan planner) {
			new Thread() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Thread#run()
				 */
				@Override
				public void run() {
					try {
						planner.registerListener(listener);
					} catch (RemoteException exception) {
						logger.error(exception);
					}
				}

			}.start();
		}

	}

	private PeerRegistry			localRegistry;

	private PeerManager<RemotePlan> remotePlanners;

	private PlanProxy			   planProxy;

	/**
     * 
     */
	public Communicator(String name) {
		super();
		try {
			planProxy = new PlanProxy(name);
			new Thread(null, new Initializer(), "communication initializer").start();
		} catch (RemoteException exception) {
			Utils.logException(logger, exception);
		}
	}

	/**
	 * @return Returns the remotePlanners.
	 */
	public PeerManager<RemotePlan> getRemotePlanners() {
		return remotePlanners;
	}

	/**
	 * @return Returns the planProxy.
	 */
	public PlanProxy getPlanProxy() {
		return planProxy;
	}

	/**
	 * Register objects that should be published remotely
	 */
	public void registerObject(PeerRemote obj) {
		if (obj instanceof PeerRemoteObject) {
			try {
				String name = obj.getClass().getName();// + " @ " +
													   // InetAddress.getLocalHost().getHostName();
				localRegistry.bind((PeerRemoteObject) obj, name);
			} catch (RemoteException exception) {
				logger.error(exception);
			}
		}
	}

	private void initRemoteConstraints() throws InstantiationException, IllegalAccessException {
		/**
		 * Initialize all remote plan listeners.
		 * 
		 * TODO: Only do this if necessary
		 */

		Set<Class<? extends RemotePlanListener>> listenerClasses = remotePlannerConstraints.keySet();
		for (Class<? extends RemotePlanListener> listenerClass : listenerClasses) {
			RemotePlanListener remoteListener = listenerClass.newInstance();
			registerObject(remoteListener);
			remotePlannerConstraints.put(listenerClass, remoteListener);
		}
	}

}
