/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.communication;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.concurrent.Semaphore;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.parameters.Parameter;
import critiquer.misc.Utils;


import jxl.peermi.PeerRemoteObject;

/**
 * @author olale
 * 
 */
public class PlanProxy extends PeerRemoteObject implements RemotePlan, ActivityUpdateListener {

	private final class RemoteUpdater implements Runnable {
		/**
         * 
         * Notifies remote listeners watching for updates regularly, if there
         * are any. We cache updates so that listeners are not told about
         * updates more than the updateFrequency stipulates.
         * 
         * @see java.lang.Thread#run()
         */

		public void run() {
			Date currentTime = new Date();
			if (currentTime.getTime() - lastAccess.getTime() > (updateFrequency * 1000) && !updatedActivities.isEmpty()) {
				try {
					planListenerUpdateLock.acquire();
					activityLock.acquire();
					for (RemotePlanListener listener : remotePlanListeners) {
						for (RemoteActivity activity : updatedActivities) {
							try {
								listener.activityUpdated(activity);
							} catch (RemoteException exception) {
								Utils.logException(Logger.getLogger(getClass()), exception);
							}
						}
					}
					updatedActivities.clear();
					activityLock.release();
					planListenerUpdateLock.release();
					lastAccess = currentTime;
				} catch (InterruptedException exception1) {
					// TODO Auto-generated catch block
					exception1.printStackTrace();
				}
			}
		}
	}

	/**
     * 
     */
	private static final long			 serialVersionUID	= 1L;

	private Date						  lastAccess		  = new Date();

	private int						   updateFrequency	 = 5;
	private Semaphore planListenerUpdateLock = new Semaphore(1);
	private Semaphore activityLock = new Semaphore(1);
	private ArrayList<RemotePlanListener> remotePlanListeners = new ArrayList<RemotePlanListener>();

	/**
     * A set to keep track of which activities have changed properties.
     */
	private HashSet<RemoteActivity>			 updatedActivities   = new HashSet<RemoteActivity>();

	private Runnable					  updateThread		= new RemoteUpdater();

	private String name;

	/**
     * @throws RemoteException
     */
	public PlanProxy(String name) throws RemoteException {
		super();
		this.name=name;
		init();
	}
	

	private void init() {
		Activity.addActivityUpdateListener(this);
	}

	/**
     * 
     * Register remote listeners
     * 
     * @see critiquer.communication.RemotePlan#registerListener(critiquer.communication.RemotePlanListener)
     */
	public void registerListener(RemotePlanListener l) throws RemoteException {
		try {
			planListenerUpdateLock.acquire();
			remotePlanListeners.add(l);
			planListenerUpdateLock.release();
		} catch (InterruptedException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		}
	}


	/**
     * Add information to the update cache and dispatch to the update thread.
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityUpdated(critiquer.activities.Activity)
     */
	public void activityUpdated(Activity activity) {
		cacheUpdateAndDispatch(activity);
	}

	private void cacheUpdateAndDispatch(Activity activity) {
		try {
			activityLock.acquire();
			updatedActivities.add(new ActivityProxy(activity,this));
			activityLock.release();
			new Thread(updateThread).start();
		} catch (RemoteException exception) {
			Utils.logException(Logger.getLogger(getClass()), exception);
		} catch (InterruptedException exception) {
			Utils.logException(Logger.getLogger(getClass()), exception);
		}
	}
	/**
     * 
     * Add information to the update cache and dispatch to the update thread.
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityRemoved(critiquer.activities.Activity)
     */
	public void activityRemoved(Activity activity) {
		// Can we really wait with dispatch when this activity has been removed from the list of activities?
		cacheUpdateAndDispatch(activity);		
	}

	/**
     * 
     * Add information to the update cache and dispatch to the update thread.
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityCreated(critiquer.activities.Activity)
     */
	public void activityCreated(Activity activity) {
		cacheUpdateAndDispatch(activity);
	}

	/**
     * Ignored for remote plans
     * 
     * @see critiquer.activities.ActivityUpdateListener#paramAdded(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter)
     */
	public <T> void paramAdded(Activity activity, Parameter<T> p) {
		
	}

	/**
	 * 
	 * Add information to the update cache and dispatch to the update thread.
     *
     * 
     * @see critiquer.activities.ActivityUpdateListener#paramChanged(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter)
     */
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		cacheUpdateAndDispatch(activity);
	}

	/**
	 * 
	 * This needs to come late in order for all modifications to have taken place
     * 
     * @see java.lang.Comparable#compareTo(T)
     */
	public int compareTo(ActivityUpdateListener o) {
		return 1;
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemotePlan#getName()
	 */
	public String getName() throws RemoteException {
		return name;
	}

}
