/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.communication;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.HashSet;

import jxl.peermi.PeerRemoteObject;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.constraints.AgentMutex;
import critiquer.constraints.ConstraintFactory;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;

/**
 * @author olale
 *
 */
public class RemoteAgentMutexConstraint extends PeerRemoteObject implements RemotePlanListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private transient Logger logger = Logger.getLogger(getClass());
	private AgentMutex delegator;
	
	/**
	 * @throws RemoteException
	 */
	public RemoteAgentMutexConstraint() throws RemoteException {
		super();
		
	}

	private void initDelegator() {
		delegator = AgentMutex.getSingleton();
		/**
		 * If there was no local delegator available, create one
		 */
		if (delegator==null) {
			ConstraintFactory.createSingletonConstraint(AgentMutex.class,false);
			initDelegator();
		}
	}

	/**
	 * @param port
	 * @throws RemoteException
	 */
	public RemoteAgentMutexConstraint(int port) throws RemoteException {
		super(port);
		
	}

	/**
	 * Check if the activity passed as the first parameter overlaps any of the other parameters in time
	 * @param remoteActivity
	 * @param activities
	 * @return
	 * @throws RemoteException 
	 */
	private void checkActivityOverlaps(RemoteActivity remoteActivity) throws RemoteException {
		String information = "";
		AgentMutex agentMutex = getDelegator();
		for (Activity localActivity : ReferenceHolder.allActivities) {
			if (overlaps(remoteActivity, localActivity)) {
				logger.debug("Found overlapping activities: "+remoteActivity.getName()+" and "+localActivity);
				HashSet<RemoteAgent> agentsInActivity = remoteActivity.getAgents();
				HashSet<Agent> agentsInOtherActivity = ActivityUtils.getAgentsInActivity(localActivity);
				HashSet<Agent> agentsInBoth = new HashSet<Agent>();
				for (Agent agent : agentsInOtherActivity) {
					boolean match=false;
					for (RemoteAgent remoteAgent : agentsInActivity) {
						if (remoteAgent.getName().equals(agent.toString())) {
							match=true;
							logger.debug(" ... with the same agent: "+agent);
							break;
						}
					}
					if (match) {
						agentsInBoth.add(agent);
					}
				}
				if (!agentsInBoth.isEmpty()) {
					information = "Caution: ";
					for (Agent agent : agentsInBoth) {

						information += "\n" + agent
						+ " is now used by "
						+ remoteActivity.getPlanner().getName()
						+ " for activity \""+remoteActivity.getName()
						+ "\" and you have planned it for "
						+ localActivity
						+ " at the same time";
					}
					agentMutex.getViolationActivities().put(localActivity,true);
					agentMutex.getErrorMessages().put(localActivity, information);
					
				}
			} 
				
		}
		
		agentMutex.setConsistent(!agentMutex.getViolationActivities().containsValue(true));
	}

	
	/**
	 * @param activity
	 * @param otherActivity
	 * @return
	 * @throws RemoteException 
	 */
	private boolean overlaps(RemoteActivity activity, Activity otherActivity) throws RemoteException {
		Date start1 = activity.getStartTime();
		Date end1 = activity.getEndTime();
		Date start2 = otherActivity.getStartTime();
		Date end2 = otherActivity.getEndTime();
		Date lastStart = (start1.after(start2) ? start1 : start2);
		Date firstEnd = (end1.before(end2) ? end1 : end2);
		return lastStart.before(firstEnd);
		
	}

	/**
	 * Dispatch to the local AgentMutex critic, if one exists
	 * 
	 * @see critiquer.communication.RemotePlanListener#activityUpdated(critiquer.activities.Activity)
	 */
	public void activityUpdated(RemoteActivity activity) throws RemoteException {
		checkActivityOverlaps(activity);
	}

	/**
	 * @return Returns the delegator.
	 */
	public AgentMutex getDelegator() {
		if (delegator==null) {
			initDelegator();
		}
		return delegator;
	}

}
