/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Graphics2D;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.ToplevelActivity;
import critiquer.activities.parameters.Parameter;
import critiquer.gui.TaskView;
import critiquer.gui.View;
import critiquer.misc.Pair;
import critiquer.misc.UnaryFunction;
import critiquer.misc.Utils;



/**
 * 
 * <p>
 * If an activity needs to have some special properties in order to be
 * considered completely specified, we can use this constraint to notify the
 * user. For instance, we may require that there should be a top-level activity
 * for each mission and therefore, all new activities except top-level
 * activities should have a parent.
 * </p>
 * 
 * This critic can be considered to be a meta-level critic since it collects
 * information from other critics. This information can be displayed in textual
 * form, but more importantly it can be used by activities
 * 
 * @author olale
 * 
 */
public class ActivityStructureCritic extends StandardConstraint implements TextCritic, ConstraintStatusListener {

	private static ActivityStructureCritic   singleton;

	/**
     * 
     */
	private static final long				serialVersionUID = 1L;

	private HashMap<String, StructureCritic> structureCritics = new HashMap<String, StructureCritic>();

	/**
     * 
     */
	public ActivityStructureCritic() {
		super();
		singleton = this;
		init();
	}

	/**
     * Add some basic structure critics, such as that all activities except the
     * top level one should have parents.
     * 
     */
	private void init() {
		StructureCritic noParentCritic = new StructureCritic() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			/**
             * 
             * Applicable to all except the top level critic
             * 
             * @see critiquer.constraints.StructureCritic#applicableForActivity(critiquer.activities.Activity)
             */
			@Override
			public boolean applicableForActivity(Activity activity) {
				return !ToplevelActivity.class.isAssignableFrom(activity.getClass());
			}

			@Override
			public String getDescription() {
				return "Shows incompletely specified activities";
			}

			@Override
			public String getName() {
				return "no-parent critic";
			}

			@Override
			public Pair<Boolean, String> check(Activity activity) {
				Pair<Boolean, String> checkVal = new Pair<Boolean, String>(true, "");
				checkVal.a = activity.hasParent();
				if (!checkVal.a) {
					checkVal.b = activity + " should have a parent activity";
				}
				return checkVal;
			}

		};
		addStructureCritic(noParentCritic);
	}

	/**
     * 
     * Add a critic to the map of structure critics
     * 
     */
	public void addStructureCritic(StructureCritic critic) {
		if (structureCritics.containsKey(critic.getName())) {
			warn("Warning: Trying to add structure critic with name " + critic.getName() + " which is already present");
		}
		structureCritics.put(critic.getName(), critic);
		StandardConstraint.addStatusListener(this);
	}

	public StructureCritic getStructureCritic(String name) {
		return structureCritics.get(name);
	}

	/**
     * Summarize the information collected from all the individual structure
     * critics.
     * 
     */
	public String getText() {
		String s = "";
		Collection<StructureCritic> critics = structureCritics.values();
		for (StructureCritic critic : critics) {
			HashMap<Activity, String> errorMessages = critic.getErrorMessages();
			if (!errorMessages.isEmpty()) {
				s += "\nInformation from \"" + critic.getName() + "\"(" + critic.getDescription() + "):\n\n";
				for (String message : errorMessages.values()) {
					s += message + "\n";
				}
			}
		}
		return s;
	}

	public boolean isConsistent(Activity activity) {
		boolean activityConsistent = true;
		Collection<StructureCritic> critics = structureCritics.values();
		for (StructureCritic critic : critics) {
			activityConsistent = critic.isConsistent(activity);
			if (!activityConsistent) {
				violationActivities.put(activity, activityConsistent);
				break;
			}
		}
		return activityConsistent;
	}

	/**
     * 
     * If one of the structure critics returns false, our status should be
     * "inconsistent", otherwise consistent. This is handled by listening to
     * when the structure critics change status and not here however.
     * 
     * @see critiquer.activities.AbstractParamChangedListener#paramChanged(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter)
     *      
     *      @see critiquer.constraints.ActivityStructureCritic#violationStatusChanged(StandardConstraint)
     *      
     */
	@Override
	public <T> void paramChanged(Activity activity, Parameter<T> p) {

	}

	/**
     * Returns a description
     * 
     * @see critiquer.constraints.SingletonConstraint#getDescription()
     */
	public String getDescription() {
		return "Checks whether or not activities are properly specified according to structure and form";
	}

	/**
     * Present some graphical information
     * 
     * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View,
     *      java.awt.Graphics2D)
     */
	public void viewUpdated(View v, Graphics2D g2) {
		// TODO Auto-generated method stub

	}

	/**
     * Set this critic visible in the graph view only
     * 
     * @see critiquer.constraints.StandardConstraint#initApplicableViews()
     */
	@Override
	protected void initApplicableViews() {
		applicableViews.add(TaskView.class);
	}

	/**
     * @return Returns the singleton.
     */
	public static ActivityStructureCritic getSingleton() {
		return singleton;
	}

	/**
     * 
     * Make sure the structure critics are processed before the
     * ActivityStructureCritic
     * 
     * @see java.lang.Comparable#compareTo(T)
     */
	public int compareTo(ActivityUpdateListener l) {
		int comp = 0;
		if (l.getClass() == StructureCritic.class) {
			comp = 1;
		}
		return comp;
	}

	/**
     * 
     * Listen to when structure critics change active status. The active status
     * is only supposed to indicate when a critic should show information
     * graphically, so this should not be applicable to the text-only structure
     * critics
     * 
     * @see critiquer.constraints.ConstraintStatusListener#activeStatusChanged(critiquer.constraints.StandardConstraint)
     */
	public void activeStatusChanged(StandardConstraint c) {

	}

	private boolean conjunction(List<Boolean> truthValues) {
		boolean b = true;
		for (Boolean bool : truthValues) {
			b &= bool;
		}
		return b;
	}

	/**
     * 
     * We are only interested in structure critics here, so we need to make sure
     * we are not listening to other critics. We collect the information from
     * all structure critics and set our consistency status depending on that,
     * since it may be the case that this critic is inconsistent, but all
     * structure critics have actually become consistent.
     * 
     * @see critiquer.constraints.ConstraintStatusListener#violationStatusChanged(critiquer.constraints.StandardConstraint)
     */
	public void violationStatusChanged(StandardConstraint c) {
		if (c instanceof StructureCritic) {
			Collection<StructureCritic> critics = structureCritics.values();
			setConsistent(conjunction(Utils.map(new UnaryFunction<StructureCritic, Boolean>() {

				public Boolean perform(StructureCritic critic) {
					return critic.isConsistent();
				}
			}, 
			critics)));
		}
	}

}
