/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import critiquer.activities.Activity;
import critiquer.activities.ActivityConsistencyCheck;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.AgentParameter;
import critiquer.activities.parameters.Commander;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.gui.View;
import critiquer.misc.Pair;
import critiquer.resources.Agent;


import sun.security.action.GetLongAction;

/**
 * 
 * Checks whether or not an agent can be added to a commander, depending on if
 * there are any other activities the agent is engaged in.
 * 
 * @author olale
 * 
 */
public class AgentMutex extends StructureCritic implements TextCritic {

	private static AgentMutex singleton = null;

	/**
	 * 
     * Create a new object unless there an AgentMutex has already been created.
     * 
     */
	public AgentMutex() {
		super();
		/**
         * Ensure that there is only one instance created through
         * ConstraintFactory.createSingletonVisualConstraint
         */
		assert (singleton == null);
		singleton = this;
	}

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	/**
	 * Check if the activity passed as the first parameter overlaps any of the other parameters in time
	 * @param activity
	 * @param activities
	 * @return
	 */
	private Pair<Boolean, String> checkActivityOverlaps(Activity activity, ArrayList<Activity> activities) {
		assert (!activities.contains(activity));
		String information = "";
		for (Activity otherActivity : activities) {
			if (ActivityUtils.overlap(activity, otherActivity)) {
				HashSet<Agent> agentsInActivity = ActivityUtils.getAgentsInActivity(activity);
				agentsInActivity.retainAll(ActivityUtils.getAgentsInActivity(otherActivity));
				information = "";
				for (Agent agent : agentsInActivity) {
					
					information += "\n"+agent + " is already committed to activity "
					+ otherActivity
					+ " at the same time so it cannot perform "
					+ activity;
				}
				errorMessages.put(otherActivity,information);
				debug(information);
				return new Pair<Boolean, String>(false,information);
			} 
				
		}
		return OK;
	}

	


	/**
     * The activity parameter is null here since we are called from an
     * AgentParameter which is not directly connected to an activity. However,
     * we can use the <code>Commander</code> of the
     * <code>AgentParameter</code> <i>p</i> to retrieve a list of activities
     * for which the current selection of agents is about to be selected.
     * If there is an overlap in time between any of them and the other
     * activities to which these agents are already assigned, the check fails.
     * 
     * @param a
     *            Activity that fired this event or null if this was a parameter
     *            not directly connected to an activity. It is of little
     *            interest here since we use the activities available through
     *            the <code>Commander</code>
     * @param p
     *            The parameter that caused this event.
     * @param newValue
     *            The new value that we are supposed to check.
     * @return
     */
	
	public Pair<Boolean, String> check(Activity a) {
		AgentParameter agentParam = ActivityUtils.getMainAgentParameter(a);
		Pair<Boolean, String> checkValue;
		if (agentParam!=null) {
			checkValue = checkAgentOverlaps(ActivityUtils.getAgentsInActivity(a), agentParam);
		} else {
			checkValue = OK;
		}
		return checkValue;
	}


	private <T> Pair<Boolean, String> checkAgentOverlaps(T newValue, AgentParameter agentParam) {
		Commander myCommander = agentParam.getCommander();
		ArrayList<Activity> myActivities = (ArrayList<Activity>) myCommander.getAllActivities().clone();
		HashSet<Agent> agents = (HashSet<Agent>) newValue;
		ArrayList<Activity> otherActivities = new ArrayList<Activity>();
		for (Agent agent : agents) {
			 otherActivities.addAll(agent.getActivitiesForAgent());
		}
		otherActivities.removeAll(myActivities);
		Pair<Boolean, String> overlapsAny  = OK;
		while (!myActivities.isEmpty() && !otherActivities.isEmpty()) {
			Activity activity = myActivities.remove(0);
			overlapsAny= checkActivityOverlaps(activity, otherActivities);
			if (!overlapsAny.a) {
				violationActivities.put(activity,true);
				break;
			} else {
				violationActivities.put(activity,false);
			}
		}
		return overlapsAny;
	}

	/**
     * Describe the critic
     * 
     * @see critiquer.constraints.SingletonConstraint#getDescription()
     */
	public String getDescription() {
		return "Make sure agents cannot be employed by more than one commander at the same time";
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View,
     *      java.awt.Graphics2D)
     */
	public void viewUpdated(View v, Graphics2D g2) {
		// TODO Auto-generated method stub

	}

	/**
	 * Returns the textual information that should be presented by this critic
	 * @see critiquer.constraints.TextCritic#getText()
	 */
	public String getText() {
		StringBuffer str = new StringBuffer();
		Collection<String> messages = errorMessages.values();
		for (String string : messages) {
			str.append(string);
		}
		return str.toString();
	}


	/**
	 * @return Returns the singleton.
	 */
	public static AgentMutex getSingleton() {
		return singleton;
	}

	/**
	 * 
	 * Returns the name of the critic
	 * 
	 * @see critiquer.constraints.StructureCritic#getName()
	 */
	@Override
	public String getName() {
		
		return "agent mutex critic";
	}

}
