/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.util.HashMap;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.TimeParameter;



public class ConstraintFactory {

	private static HashMap<Class, Object> singletonConstraints = new HashMap<Class, Object>();

	public static VisualConstraint getVisualConstraint(Class visualConstraintClass) {
		Object obj = singletonConstraints.get(visualConstraintClass);
		if (obj instanceof VisualConstraint) {
			return (VisualConstraint) obj;
		} else {
			return null;
		}
	}

	public static SingletonConstraint getSingletonConstraint(Class visualConstraintClass) {
		Object obj = singletonConstraints.get(visualConstraintClass);
		if (obj instanceof SingletonConstraint) {
			return (SingletonConstraint) obj;
		} else {
			return null;
		}
	}

	public static TimeParameterOrdering createLaterThanConstraint(TimeParameter endtimeA1,
																  TimeParameter starttimeA2,
																  TimeParameterOrdering.Type type,
																  ConstraintPolicy policy) {
		TimeParameterOrdering paramOrder = new TimeParameterOrdering(endtimeA1, starttimeA2,type);
		paramOrder.setPolicy(policy);
		return paramOrder;
	}

	/**
     * 
     * Visual constraints that are only supposed to be created once are created
     * here. The ones that need to be updated when parameter values change
     * should inherit from <code>StandardConstraint</code> so that they are
     * added to the list of activity update listeners through the default
     * constructor of that class.
     * 
     * @param constraintClass
     * @param enabled
     */
	public static void createSingletonConstraint(Class constraintClass, boolean enabled) {

		if (VisualConstraint.class.isAssignableFrom(constraintClass) && SingletonConstraint.class.isAssignableFrom(constraintClass)) {
			if (!singletonConstraints.containsKey(constraintClass)) {
				try {

					VisualConstraint visualConstraint = (VisualConstraint) constraintClass.newInstance();
					singletonConstraints.put(constraintClass, visualConstraint);
					VisualConstraints.addConstraint((VisualConstraint) visualConstraint);
					VisualConstraints.setConstraintEnabled(constraintClass, enabled);

				} catch (Exception e) {
					e.printStackTrace();
				}
			} else {
				Logger.getLogger(ConstraintFactory.class).info("Note: Trying to create already existing singleton constraint of type " + constraintClass.getSimpleName());
			}
		} else {
			Logger.getLogger(ConstraintFactory.class).error("Wrong type of argument to createSingletonVisualConstraint: " + constraintClass);
		}

	}

	/**
     * 
     * Used when creating a sub-activity that is supposed to be performed within
     * the time frame of the parent activity. This constraint makes sure the
     * child activity cannot start before the parent activity.
     * 
     * @param a1
     *            the parent activity
     * @param a2
     *            the child activity
     * @return a time parameter ordering constraint
     */
	public static TimeParameterOrdering connectStartToStart(Activity a1, Activity a2) {
		TimeParameter a1StartTime = ActivityUtils.getStartTimeParameter(a1);
		TimeParameter a2StartTime = ActivityUtils.getStartTimeParameter(a2);
		TimeParameterOrdering missionStartConstraint = ConstraintFactory.createLaterThanConstraint(a1StartTime,
				a2StartTime,
				TimeParameterOrdering.Type.START_TO_START,
				ConstraintPolicy.MODIFY_VALUES);
		return missionStartConstraint;
	}

	/**
     * 
     * Used when creating a sub-activity that is supposed to be performed within
     * the time frame of the parent activity. This constraint ensures taht the
     * child activity cannot end after the parent activity.
     * 
     * @param parent
     *            the parent activity
     * @param child
     *            the child activity
     * @return a time parameter ordering constraint
     */
	public static TimeParameterOrdering connectEndToEnd(Activity parent, Activity child) {
		TimeParameter parentEndTime = ActivityUtils.getEndTimeParameter(parent);
		TimeParameter childEndTime = ActivityUtils.getEndTimeParameter(child);
		TimeParameterOrdering missionEndConstraint = ConstraintFactory.createLaterThanConstraint(childEndTime,
				parentEndTime,
				TimeParameterOrdering.Type.END_TO_END,
				ConstraintPolicy.MODIFY_VALUES);
		return missionEndConstraint;
	}

	/**
     * Make sure the activity is internally consistent so that start always
     * comes before end
     * 
     * @param a
     * @return a parameter order constraint
     * 
     */
	public static TimeParameterOrdering connectStartToEnd(Activity a) {
		TimeParameter start = ActivityUtils.getStartTimeParameter(a);
		TimeParameter end = ActivityUtils.getEndTimeParameter(a);
		TimeParameterOrdering internalTimeConstraint = ConstraintFactory.createLaterThanConstraint(start,
				end,
				TimeParameterOrdering.Type.START_TO_END,
				ConstraintPolicy.MODIFY_VALUES);
		return internalTimeConstraint;
	}

	/**
     * Connect two activities to form a sequence.
     * 
     * @param a1
     *            the activity that should be performed first
     * @param a2
     *            the activity that should come second
     * @return an ordering constraint that assures that activity a1 comes before a2
     */
	public static TimeParameterOrdering connectEndToStart(Activity a1, Activity a2) {

		TimeParameter endtimeA1 = ActivityUtils.getEndTimeParameter(a1);
		TimeParameter starttimeA2 = ActivityUtils.getStartTimeParameter(a2);
		TimeParameterOrdering paramOrder = ConstraintFactory.createLaterThanConstraint(endtimeA1,
				starttimeA2,
				TimeParameterOrdering.Type.END_TO_START,
				ConstraintPolicy.MODIFY_VALUES);
		
		/**
		 * "Glue" the two activities together
		 */
		paramOrder.setMaxSlack(0);
		paramOrder.setSlackAllowed(false);
		return paramOrder;
	}

}
