/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Map.Entry;

import javax.swing.JComponent;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.gui.TimeView;
import critiquer.gui.View;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;

/**
 * 
 * <p>
 * A critic that is supposed to display information on resource usage depending
 * on which resources are used at the current time in the time view
 * </p>
 * <p>
 * For each activity that is created, a constraint map is maintained that holds
 * eq constraints for the activity start and end times so that their values
 * reflect those of the mission time. The map also contains IfOnlyIf constraints
 * that forces an integer variable agentUsage to 1 if the current time in the
 * time view is within the mission time and 0 otherwise.
 * </p>
 * <p>
 * There is finally a total sum of the resource usage which is the sum of all
 * agentUsage variables for all activities.
 * </p>
 * 
 * @author olale
 * 
 */

public class ResourceUsageCritic extends StandardConstraint implements GUIComponentCritic {

	/**
     * 
     */
	private static final long		serialVersionUID = 1L;

	public static final int		  MAX_USAGE		= 1000;

	private String				   activities	   = "All activities";

	private DefaultCategoryDataset   dataset		  = new DefaultCategoryDataset();

	private JFreeChart			   chart			= ChartFactory.createBarChart("Resource usage",
															  "Resource type",
															  "Usage",
															  dataset,
															  PlotOrientation.VERTICAL,
															  true,
															  false,
															  false);

	private ChartPanel			   panel			= new ChartPanel(chart);

	private HashMap<String, Integer> usageMap		 = new HashMap<String, Integer>();

	ResourceUsageCritic() {
		usageMap.put("All", 0);
		updateDataSet();
	}

	/**
	 * The current time has changed, so modify the barchart dataset
	 * 
	 */
	private void updateDataSet() {
		for (Entry<String, Integer> entry : usageMap.entrySet()) {
			dataset.setValue(entry.getValue(), entry.getKey(), activities);
		}
	}

	/**
	 * This view is only applicable to the time view
	 * 
	 */
	@Override
	protected void initApplicableViews() {
		applicableViews.add(TimeView.class);
	}

	private boolean isActivityActive(Activity activity) {
		boolean active = false;
		Calendar currentTime = ReferenceHolder.timeView.getCurrentTime();
		TimeParameter startParam = ActivityUtils.getStartTimeParameter(activity);
		TimeParameter endParam = ActivityUtils.getEndTimeParameter(activity);
		if (startParam.hasValue() && endParam.hasValue()) {
			active = (!startParam.getValue().after(currentTime)) && (!endParam.getValue().before(currentTime));
		}
		return active;
	}

	public String getDescription() {
		return "Displays information on resource usage";
	}

	/**
	 * 
	 * We only need set the currentTime through changing the currentTime
	 * <code>eq</code> constraint since the updating is already done by the
	 * ChartPanel component
	 * 
	 */
	public void viewUpdated(View v, Graphics2D g2) {
		if (v instanceof TimeView) {
			TimeView timeView = (TimeView) v;
			ArrayList<Activity> activityList = timeView.getEvaluationActivities();
			Set<String> usageKeys = usageMap.keySet();
			for (String key : usageKeys) {
				usageMap.put(key, 0);
			}

			for (Activity activity : activityList) {
				if (isActivityActive(activity)) {
					HashSet<Agent> agents = ActivityUtils.getAgentsInActivity(activity);
					for (Agent agent : agents) {
						String agentType = agent.getClass().getSimpleName();
						if (usageMap.containsKey(agentType)) {
							usageMap.put(agentType, usageMap.get(agentType) + 1);
						} else {
							usageMap.put(agentType, 1);
						}
					}
					usageMap.put("All", usageMap.get("All") + agents.size());
				}
			}

			updateDataSet();
		}
	}

	@Override
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		// TODO Auto-generated method stub

	}

	/**
	 * Return the chart panel
	 * 
	 * @see critiquer.constraints.GUIComponentCritic#getComponent()
	 */
	public JComponent getComponent() {
		return panel;
	}
}
