/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.log4j.Logger;

import critiquer.activities.AbstractParamChangedListener;
import critiquer.activities.Activity;
import critiquer.gui.AbstractView;
import critiquer.gui.View;


/**
 * <p>
 * Abstract class for those constraints only interested in when parameter values
 * are changed. Provide dummy implementations for all other methods. To use
 * these constraints, we must make sure they are listed in the
 * ReferenceHolder.constraintClasses list. These constraints, if created by the
 * ConstraintFactory, use the default constructor in this class that adds them
 * as ActivityUpdateListeners so they do <i>NOT</i> have to register themselves
 * as activity update listeners.
 * </p>
 * 
 */
public abstract class StandardConstraint extends AbstractParamChangedListener implements
																			 SingletonConstraint,
																			 VisualConstraint {

	private boolean							active		  = false;
	/**
	 * Used to information other parties that this constraint is violated
	 */
	private boolean consistent = true;
	private Color							  color		   = Color.RED;

	protected ArrayList<Class<? extends View>> applicableViews = new ArrayList<Class<? extends View>>();
	protected static HashSet<ConstraintStatusListener> statusListeners = new HashSet<ConstraintStatusListener>();
	protected HashMap<Activity, Boolean> violationActivities = new HashMap<Activity, Boolean>();
	protected transient Logger logger;
	
	@SuppressWarnings("unchecked")
	public boolean isApplicableFor(View view) {
		boolean applicable = false;
		for (Class c : getApplicableViews()) {
			if (c.isAssignableFrom(view.getClass())) {
				applicable = true;
				break;
			}
		}
		return applicable;
	}

	protected StandardConstraint() {
		Activity.addActivityUpdateListener(this);
		logger = Logger.getLogger(getClass());
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color c) {
		color = c;
	}

	/**
     * By default, all constraints should be visible in all views
     */
	public List<Class<? extends View>> getApplicableViews() {
		if (applicableViews.isEmpty()) {
			initApplicableViews();
		}
		return applicableViews;
	}

	protected void initApplicableViews() {
		applicableViews.add(AbstractView.class);
	}

	/**
     * @see critiquer.constraints.VisualConstraint#isActive()
     */
	public boolean isActive() {
		return active;
	}

	/**
	 * 
	 * Set active status and notify status listeners
     * @see critiquer.constraints.VisualConstraint#setActive(boolean)
     */
	public void setActive(boolean active) {
		this.active = active;
		for (ConstraintStatusListener listener : statusListeners) {
			listener.activeStatusChanged(this);
		}
	}

	/**
	 * 
	 * Add to the list of update listeners
	 * 
	 * @see critiquer.constraints.VisualConstraint#addStatusListener(critiquer.constraints.ConstraintStatusListener)
	 */
	public static void addStatusListener(ConstraintStatusListener l) {
		statusListeners.add(l);
	}

	/** 
	 * 
	 * Remove from the list of update listeners
	 * @see critiquer.constraints.VisualConstraint#removeStatusListener(critiquer.constraints.ConstraintStatusListener)
	 */
	public static void removeStatusListener(ConstraintStatusListener l) {
		statusListeners.remove(l);
	}

	
	/**
	 * @return Returns the consistent.
	 */
	public boolean isConsistent() {
		return consistent;
	}

	/**
	 * Update violation status and notify status listeners
	 * @param consistent The consistent to set.
	 */
	public void setConsistent(boolean consistent) {
		this.consistent = consistent;
		for (ConstraintStatusListener listener : statusListeners) {
			listener.violationStatusChanged(this);
		}
	}

	/**
	 * Create debugging information
	 * 
	 * @see org.apache.log4j.Category#debug(java.lang.Object)
	 */
	public void debug(Object arg0) {
		logger.debug(arg0);
	}

	/**
	 * 
	 * Create non-debuggning information
	 * 
	 * @see org.apache.log4j.Category#info(java.lang.Object)
	 */
	public void info(Object arg0) {
		logger.info(arg0);
	}

	/**
	 * 
	 * Logs errors
	 * 
	 * @see org.apache.log4j.Category#error(java.lang.Object)
	 */
	public void error(Object arg0) {
		logger.error(arg0);
	}

	/**
	 * 
	 * Logs warnings
	 * 
	 * @see org.apache.log4j.Category#warn(java.lang.Object)
	 */
	public void warn(Object arg0) {
		logger.warn(arg0);
	}

	/**
	 * @return Returns the violationActivities.
	 */
	public HashMap<Activity, Boolean> getViolationActivities() {
		return violationActivities;
	}


}
