package critiquer.constraints;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.parameters.Parameter;
import critiquer.gui.View;
import critiquer.misc.Pair;
import critiquer.misc.Predicate;
import critiquer.misc.Utils;



/**
 * 
 * Thisclass should be inherited by all individual structure critics. Structure
 * critics can be checked individually for consistency through the
 * ActivityStructureCritic.
 * 
 * @author olale
 * 
 */

public abstract class StructureCritic extends StandardConstraint {

	public static Pair<Boolean,String> OK = new Pair<Boolean, String>(true,"");
	
	/**
     * 
     * Subclasses need to call this constructor in order to be included among
     * the static activity update listeners and have their applicable classes registered.
     * 
     */
	public StructureCritic() {
		super();
		addApplicableClasses();
	}
	/**
     * 
     * A structure critic can be applicable to one or more activity classes
     * 
     * @return
     */

	protected ArrayList<Class<? extends Activity>> applicableClasses = new ArrayList<Class<? extends Activity>>();

	protected HashMap<Activity, String>			errorMessages	 = new HashMap<Activity, String>();

	protected String							   description;
	
	protected transient Logger logger = Logger.getLogger(StructureCritic.class);

	
	/**
     * Is there a class among the ones in applicableClasses that is
     * assignment-compatible with the class of the argument activity?
     * 
     * @param activity
     *            The activity we want to check if this structure critic is
     *            applicable for
     * @return true iff this StructureCritic is applicable to the activity.
     */
	public boolean applicableForActivity(final Activity activity) {
		return Utils.find(applicableClasses, new Predicate<Class<? extends Activity>>() {

			public boolean check(Class<? extends Activity> c) {
				return c.isAssignableFrom(activity.getClass());
			}

		}) != null;
	}

	/**
     * 
     * This method checks if an activity is consistent with respect to some
     * structural property, such as that it should be completely specified, have
     * an agent, have a parent activity and so on. If the activity is
     * inconsistent, a message is put into the <code>errorMessages</code> map.
     * 
     * @param activity
     *            The activity to check
     * @return A pair of truth-value, message. If the check returns false as the truth value, then the message is the error message.
     */
	public abstract Pair<Boolean, String> check(Activity activity);

	protected void checkAndUpdate(Activity activity) {
		boolean applicableForActivity = applicableForActivity(activity);
		if (applicableForActivity) {
			logger.debug(getName()+" checking "+activity);
			Pair<Boolean, String> checkVal = check(activity);
			violationActivities.put(activity, checkVal.a);
			if (!checkVal.a) {
				logger.debug(activity+" inconsistent with respect to "+getName());
				errorMessages.put(activity, checkVal.b);
			} else {
				errorMessages.remove(activity);
			}
		}
		setConsistent(!violationActivities.containsValue(false));
	}
	
	public boolean isConsistent(Activity activity) {
		return violationActivities.get(activity);
	}
	
	/**
     * Describes which errors this critic will find
     * 
     * @return the description string
     */
	public abstract String getDescription();

	/**
	 * Short descriptive name
	 * 
	 * @return a string with the name
	 */
	public abstract String getName();
	
	/**
     * 
     * Returns an error message for this activity.
     * 
     * @param activity
     *            The activity that we want to check.
     * @return the error message
     */
	public String getErrorMessage(Activity activity) {
		return errorMessages.get(activity);
	}

	
	/**
     * Subclasses should override this method if they are only applicable to
     * some subclasses of Activity. This method should add all classes that
     * this critic is applicable for.
     * 
     */

	public void addApplicableClasses() {
		applicableClasses.add(Activity.class);
	}

	public void addApplicableClass(Class<? extends Activity> activityClass) {
		applicableClasses.add(activityClass);
		Collections.sort(applicableClasses, new Comparator<Class<? extends Activity>>() {

			public int compare(Class<? extends Activity> o1, Class<? extends Activity> o2) {
				int comp = 0;
				if (o1.isAssignableFrom(o2)) {
					comp=1;
				} else if (o2.isAssignableFrom(o1)) {
					comp=-1;
				}
				return comp;
			}
			
		});
	}
	
	/**
     * 
     * Checks the consistency
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityUpdated(critiquer.activities.Activity)
     */
	public void activityUpdated(Activity activity) {
		checkAndUpdate(activity);
	}

	/**
     * 
     * Checks the consistency
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityRemoved(critiquer.activities.Activity)
     */

	public void activityRemoved(Activity activity) {
		checkAndUpdate(activity);
	}

	/**
     * 
     * Checks the consistency
     * 
     * @see critiquer.activities.ActivityUpdateListener#activityCreated(critiquer.activities.Activity)
     */
	public void activityCreated(Activity activity) {
		checkAndUpdate(activity);
	}


	/**
     * 
     * Checks the consistency
     * 
     * @see critiquer.activities.ActivityUpdateListener#paramChanged(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter)
     */
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		checkAndUpdate(activity);
	}

	/**
	 * 
	 * Make sure these critics are processed before the ActivityStructureCritic
	 * 
	 * @see java.lang.Comparable#compareTo(T)
	 */
	public int compareTo(ActivityUpdateListener l) {
		int comp = 0;
		if (l.getClass()==ActivityStructureCritic.class) {
			comp=-1;
		}
		return comp;
	}

	/**
	 * @return Returns the errorMessages.
	 */
	public HashMap<Activity, String> getErrorMessages() {
		return errorMessages;
	}

	/* (non-Javadoc)
	 * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View, java.awt.Graphics2D)
	 */
	public void viewUpdated(View v, Graphics2D g2) {
		// TODO Auto-generated method stub
		
	}


	
}