/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Set;
import java.util.Map.Entry;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.IncompleteActivityException;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.gui.TimeView;
import critiquer.gui.View;



public class TimeCalculator extends StandardConstraint implements TextCritic {

	/**
     * 
     */
	private static final long		 serialVersionUID = 1L;

	private HashMap<Activity, String> errorMessages	= new HashMap<Activity, String>();

	private ConstraintPolicy		  policy		   = ConstraintPolicy.VISUALIZE;

	/**
     * If the time to complete a mission does not change for a while, we save
     * the last calculation in a hashmap here.
     */
	private HashMap<Activity, Long>   timesToComplete  = new HashMap<Activity, Long>();

	/**
     * When in aggressive mode, don't allow slack
     */
	private boolean				   aggressive	   = false;

	int							   maxSlack		 = 10;

	@Override
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		if (activity.canBeSimulated()) {
			try {
				Long timeToComplete = activity.simulate();
				errorMessages.remove(activity);

				timesToComplete.put(activity, timeToComplete);

				Long currentTime = ActivityUtils.getCurrentTimeForActivity(activity);
				boolean notEnoughTime = currentTime < timeToComplete;
				boolean tooMuchTime = currentTime > timeToComplete + maxSlack;

				if (notEnoughTime) {
					debug("Not enough time for " + activity + ": " + currentTime);
				} else if (tooMuchTime) {
					debug("Too much time for " + activity + ": " + currentTime);
				}

				if (notEnoughTime || (isAggressive() && tooMuchTime)) {
					violationActivities.put(activity, true);
					if (policy == ConstraintPolicy.MODIFY_VALUES) {
						// Ok, ensure there is enough time for the mission to
						// complete
						// even if some other event has changed the time
						// parameters
						TimeParameter startTime = ActivityUtils.getStartTimeParameter(activity);
						TimeParameter endTime = ActivityUtils.getEndTimeParameter(activity);
						if (p == endTime) {
							Calendar newTime = (Calendar) endTime.getValue().clone();
							newTime.add(Calendar.MINUTE, -1 * timeToComplete.intValue());
							startTime.setValue(newTime);

						} else {
							Calendar newTime = (Calendar) startTime.getValue().clone();
							newTime.add(Calendar.MINUTE, timeToComplete.intValue());
							endTime.setValue(newTime);
						}
					}

				} else if (violationActivities.containsKey(activity)) {
					violationActivities.put(activity, false);
				}
				setConsistent(!violationActivities.containsValue(Boolean.TRUE));
			} catch (IncompleteActivityException exception) {
				/**
                 * If we cannot calculate the time it will take for this
                 * mission, log this
                 */
				String message = "\nCannot calculate time for " + activity
								 + " since it is incompletely specified:\n\""
								 + exception.getMessage()
								 + "\"\n";
				errorMessages.put(activity, message);
				setConsistent(false);
			}

		} else {
			setConsistent(false);
			errorMessages.put(activity,
					activity + " incompletely specified, cannot simulate. Check the ActivityStructureCritic for more information");
		}

	}

	/**
     * @return Returns the aggressive.
     */
	public boolean isAggressive() {
		return aggressive;
	}

	/**
     * Set aggressiveness. If aggressive, the constraint will react on and/or
     * make sure that there is not more time allocated to the activity than what
     * the activity calculates that it will take with the current resources.
     * When not aggressive, it will only react on if there is too little time
     * allocated.
     * 
     * @param aggressive
     *            The aggressive to set.
     */
	public void setAggressive(boolean aggressive) {
		this.aggressive = aggressive;
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.constraints.SingletonConstraint#getDescription()
     */
	public String getDescription() {

		return "Calculates the time it takes for activities to complete " + "depending on the resources currently used. "
			   + "If there is less time allocated to a mission compared to what it requires, "
			   + "display a rectangle to highlight the activity";
	}

	/**
     * 
     * If our policy is to highlight this instead of forcing a value update, we
     * use this method.
     * 
     * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View,
     *      java.awt.Graphics2D)
     */
	public void viewUpdated(View v, Graphics2D g2) {
		Set<Entry<Activity, Long>> entries = timesToComplete.entrySet();
		g2.setColor(getColor());
		for (Entry<Activity, Long> entry : entries) {
			Activity activity = entry.getKey();
			Shape activityShape = v.getActivityArea(activity);
			if (violationActivities.containsKey(activity) && violationActivities.get(activity) && activityShape != null) {
				Rectangle area = activityShape.getBounds();
				Rectangle targetArea = getCalculatedArea(activity, area);
				g2.draw(area);
				g2.draw(targetArea);
				double x0 = area.getMaxX();
				double x1 = targetArea.getMaxX();
				double y = area.getCenterY();
				GeneralPath arrow = createArrowBetween(x0, y, x1, y);
				g2.draw(arrow);
			}

		}
	}

	/**
     * @param x0
     * @param y
     * @param x1
     * @param y2
     * @return
     */
	private GeneralPath createArrowBetween(double x0, double y0, double x1, double y1) {
		return createArrowBetween((float) x0, (float) y0, (float) x1, (float) y1);
	}

	/**
     * Code courtesy of <a
     * href="http://java.sun.com/products/java-media/2D/reference/faqs/index.html#Q_How_do_I_draw_a_quadratic_arrow">java.sun.com</a>
     * 
     * @param p1x
     * @param p1y
     * @param p2x
     * @param p2y
     * @return a path representing an arrow-headed line
     */
	private GeneralPath createArrowBetween(float p1x, float p1y, float p2x, float p2y) {
		GeneralPath path = new GeneralPath();
		float cx = p1x + (p2x - p1x) / 2, cy = p1y + (p2y - p1y) / 2; // Control
		// point
		// of
		// the
		// curve
		float arrSize = 5; // Size of the arrow segments

		float adjSize = (float) (arrSize / Math.sqrt(2));
		float ex = p2x - cx;
		float ey = p2y - cy;
		float abs_e = (float) Math.sqrt(ex * ex + ey * ey);
		ex /= abs_e;
		ey /= abs_e;

		// Creating quad arrow
		path.moveTo(p1x, p1y);
		path.quadTo(cx, cy, p2x, p2y);
		path.lineTo(p2x + (ey - ex) * adjSize, p2y - (ex + ey) * adjSize);
		path.moveTo(p2x, p2y);
		path.lineTo(p2x - (ey + ex) * adjSize, p2y + (ex - ey) * adjSize);
		return path;

	}

	private GeneralPath createArrowBetween(Point p0, Point p1) {
		float p1x = (float) p0.getX(), p1y = (float) p0.getY(); // P1
		float p2x = (float) p1.getX(), p2y = (float) p1.getY(); // P2
		return createArrowBetween(p1x, p1y, p2x, p2y);
	}

	/**
     * 
     * Calculate how large area the calculated time for this activitiy would
     * occupy in the timeline view.
     * 
     * To do this we must calculate:
     * 
     * <ul>
     * 
     * <li>how long time the mission currently takes </li>
     * 
     * <li>how long this time represents in distance (the <code>area</code>
     * currently associated with this <code>activity</code>) </li>
     * 
     * <li>and translate a new rectangle from the calculated lowest possible
     * time for this mission </li>
     * 
     * </ul>
     * 
     * @param activity
     * @param area
     * @return
     */
	private Rectangle getCalculatedArea(Activity activity, Rectangle area) {
		long currentActivityTime = ActivityUtils.getCurrentTimeForActivity(activity);
		double currentTimeRatio = currentActivityTime / area.getWidth();
		double calculatedAreaWidth = timesToComplete.get(activity) / currentTimeRatio;
		return new Rectangle(area.getLocation(), new Dimension((int) calculatedAreaWidth, (int) area.getHeight()));
	}

	/**
     * 
     * This can be viewable in the timeline view
     * 
     * @see critiquer.constraints.StandardConstraint#initApplicableViews()
     */
	@Override
	protected void initApplicableViews() {
		applicableViews.add(TimeView.class);
	}

	/**
     * @return Returns the policy.
     */
	public ConstraintPolicy getPolicy() {
		return policy;
	}

	/**
     * @param policy
     *            The policy to set.
     */
	public void setPolicy(ConstraintPolicy policy) {
		this.policy = policy;
	}

	/**
     * 
     * Present information on which activities that I cannot calculate the time
     * for
     * 
     * @see critiquer.constraints.TextCritic#getText()
     */
	public String getText() {
		StringBuffer s = new StringBuffer();
		Collection<String> messageSet = errorMessages.values();
		for (String message : messageSet) {
			s.append(message);
		}
		return s.toString();
	}

}
