/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Color;
import java.awt.Graphics2D;
import java.rmi.NotBoundException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.log4j.Logger;

import critiquer.activities.AbstractParamChangedListener;
import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.gui.AbstractView;
import critiquer.gui.View;



/**
 * Check/ensure that t2 doesn't come before t1
 * 
 * This is a modifying constraint for now, but we let it implement the
 * VisualConstraint interface just in case..
 */
public class TimeParameterOrdering implements VisualConstraint {

	public enum Type {
		END_TO_START("connect end to start"),
		START_TO_START("connect start to start"),
		START_TO_END("connect start to end"),
		END_TO_END("connect end to end");
		private String name;

		Type(String name) {
			this.name = name;
		}

		/**
         * @return Returns the name.
         */
		public String toString() {
			return name;
		}
	}

	/**
     * 
     */
	private static final long				serialVersionUID = 1L;

	private transient Logger						   logger		   = Logger.getLogger(getClass());

	private AbstractParamChangedListener	 t1Listener;

	private AbstractParamChangedListener	 t2Listener;

	private TimeParameter					t1;

	private TimeParameter					t2;

	private ConstraintPolicy				 policy		   = ConstraintPolicy.MODIFY_VALUES;

	private boolean						  slackAllowed	 = true;

	/**
     * Should the distance between the parameters be kept?
     */
	private Type							 type			 = Type.END_TO_START;

	/**
     * Max slack between two time parameters. Currently set to 10 minutes
     */
	private int							  maxSlack		 = 10 * 60 * 1000;

	/**
     * Minimum slack between two parameters. Currently set to ten minutes
     */
	private int							  minSlack		 = 10 * 60 * 1000;

	private Color							color;

	private ArrayList<Class<? extends View>> applicableViews;

	/*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
	@Override
	public String toString() {
		return "[" + t1.getActivity()
			   + ":"
			   + t1.getName()
			   + "]"
			   + " < "
			   + "["
			   + t2.getActivity()
			   + ":"
			   + t2.getName()
			   + "]";
	}

	/*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
	@Override
	public boolean equals(Object obj) {
		boolean eq = false;
		if (obj instanceof TimeParameterOrdering) {
			TimeParameterOrdering tp = (TimeParameterOrdering) obj;
			eq = tp.toString().equals(toString());
		}
		return eq;
	}

	private void modifyParameter(TimeParameter param, long timeInMillis) {
		Calendar newTime = new GregorianCalendar();
		newTime.setTimeInMillis(timeInMillis);
		param.setValue(newTime);
	}

	/**
     * Change the parameter which wasn't modified by the user, and possible
     * parameters that are indirectly affected if this ParameterOrdering orders
     * two different activities to each other. If two different activities are
     * affected, the parameter which is about to change should change together
     * with the other time parameter in that activity in an atomic update so
     * that there can be no intermediate states which are inconsistent.
     * 
     * @param modifiedParameter
     */
	private void adjustWithRespectTo(TimeParameter modifiedParameter) {
		try {
			/**
             * For internal consistency constraints, we need to use the atomic
             * update lock so we don't update inconsistent states
             */
			if (type == Type.START_TO_END) {
				modifiedParameter.getActivity().getAtomicUpdateLock().acquire();
			}
			long currentSlack = t2.getValue().getTimeInMillis() - t1.getValue().getTimeInMillis();

			boolean tooMuchSlack = (!isSlackAllowed()) && currentSlack > maxSlack;
			boolean tooLittleSlack = currentSlack < minSlack;
			long time = 0;
			boolean violation = isViolation();

			if (violation || tooMuchSlack || (tooLittleSlack && type == Type.START_TO_END)) {
				if (policy == ConstraintPolicy.MODIFY_VALUES) {

					/**
                     * The parameter directly affected by the change as a part
                     * of this TimeParameterOrdering
                     */
					boolean firstParameterChanged = modifiedParameter == t1;
					TimeParameter affectedParameter = (firstParameterChanged ? t2 : t1);
					/**
                     * The other time parameter of the activity that is
                     * affected.
                     */
					TimeParameter indirectlyAffectedParameter = null;
					/**
                     * Only constraints between activities need special
                     * attention
                     */
					long newTimeInMillis = modifiedParameter.getValue().getTimeInMillis();
					/**
                     * Ensure distance if there should be a certain slack
                     */
					if (tooLittleSlack && type == Type.START_TO_END) {
						newTimeInMillis += (firstParameterChanged ? minSlack : -1 * minSlack);
					}

					logger.debug(">>> START \"" + type + "\" " + t1.getFullName() + " < " + t2.getFullName());
					logger.debug(modifiedParameter.getFullName() + " = "
								 + timeToShortString(modifiedParameter.getValue()));

					if (type == Type.END_TO_START) {

						Activity activity = affectedParameter.getActivity();
						/**
                         * Calculate the current time span of the affected
                         * activity so it will be maintained
                         */
						activity.getAtomicUpdateLock().acquire();
						if (affectedParameter == ActivityUtils.getStartTimeParameter(activity)) {
							indirectlyAffectedParameter = ActivityUtils.getEndTimeParameter(activity);
							time = newTimeInMillis + indirectlyAffectedParameter.getValue().getTimeInMillis()
								   - affectedParameter.getValue().getTimeInMillis();
						} else if (affectedParameter == ActivityUtils.getEndTimeParameter(activity)) {
							indirectlyAffectedParameter = ActivityUtils.getStartTimeParameter(activity);
							time = newTimeInMillis - affectedParameter.getValue().getTimeInMillis()
								   + indirectlyAffectedParameter.getValue().getTimeInMillis();
						}

						modifyParameter(indirectlyAffectedParameter, time);

						logger.debug("\t-> " + indirectlyAffectedParameter.getFullName()
									 + " = "
									 + timeToShortString(indirectlyAffectedParameter.getValue()));

						modifyParameter(affectedParameter, newTimeInMillis);
						activity.getAtomicUpdateLock().release();
						//
						activity.signalParamChanged();
					} else {
						modifyParameter(affectedParameter, newTimeInMillis);
					}

					logger.debug("\t-> " + affectedParameter.getFullName()
								 + " = "
								 + timeToShortString(affectedParameter.getValue()));
					logger.debug("<<< END \"" + type + "\" " + t1.getFullName() + " < " + t2.getFullName());

				}
			}
			/**
             * For internal consistency constraints, we need to use the atomic
             * update lock so we don't update inconsistent states
             */
			if (type == Type.START_TO_END) {
				modifiedParameter.getActivity().getAtomicUpdateLock().release();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private String timeToShortString(Calendar time) {
		return DateFormat.getTimeInstance(DateFormat.SHORT).format(time.getTime());
	}

	private boolean isViolation() throws NotBoundException {
		return t1.getValue().after(t2.getValue());
	}

	public void setPolicy(ConstraintPolicy policy) {
		this.policy = policy;
	}

	/**
     * Remove the activity update listeners when this constraint is removed
     * 
     */
	public void disableConstraint() {
		t1.getActivity().removePrivateUpdateListener(t1Listener);
		t2.getActivity().removePrivateUpdateListener(t2Listener);
	}

	public TimeParameterOrdering(TimeParameter t1Param, TimeParameter t2Param, final Type type) {

		this.t1 = t1Param;
		this.t2 = t2Param;
		this.type = type;
		/*
         * When comparing the parameter that has changed, identity checks may
         * not be possible..
         */
		t1Listener = new AbstractParamChangedListener() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			@Override
			public <T> void paramChanged(Activity activity, Parameter<T> p) {
				if (t1 == p) {
					adjustWithRespectTo(t1);
				}
			}

			/**
             * 
             * These should be run as early as possible
             * 
             * @see critiquer.activities.AbstractParamChangedListener#compareTo(critiquer.activities.ActivityUpdateListener)
             */
			@Override
			public int compareTo(ActivityUpdateListener o) {
				return -1;
			}

		};
		t1.getActivity().addPrivateUpdateListener(t1Listener);
		t2Listener = new AbstractParamChangedListener() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			@Override
			public <T> void paramChanged(Activity activity, Parameter<T> p) {
				if (t2 == p) {
					adjustWithRespectTo(t2);
				}
			}

			/**
             * 
             * These should be run as early as possible
             * 
             * @see critiquer.activities.AbstractParamChangedListener#compareTo(critiquer.activities.ActivityUpdateListener)
             */
			@Override
			public int compareTo(ActivityUpdateListener o) {
				return -1;
			}

		};
		t2.getActivity().addPrivateUpdateListener(t2Listener);
		try {
			if (isViolation()) {
				/**
                 * Adjust the later parameter by default
                 */

				adjustWithRespectTo(t1);
			}
		} catch (NotBoundException e) {
			/**
             * Ignore this
             */
		}

	}

	public void viewUpdated(View v, Graphics2D g2) {
		if (policy == ConstraintPolicy.VISUALIZE) {
			logger.warn(this.getClass() + ".viewUpdated not implemented yet");

		}
	}

	public void enableEditFirst() {
		t1.setEditable(true);
		t2.setEditable(false);
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color c) {
		color = c;

	}

	/**
     * @return Returns the allowSlack.
     */
	public boolean isSlackAllowed() {
		return slackAllowed;
	}

	/**
     * @param allowSlack
     *            The allowSlack to set.
     */
	public void setSlackAllowed(boolean allowSlack) {
		this.slackAllowed = allowSlack;
	}

	/**
     * @return Returns the t1.
     */
	public TimeParameter getT1() {
		return t1;
	}

	/**
     * @return Returns the t2.
     */
	public TimeParameter getT2() {
		return t2;
	}

	public List<Class<? extends View>> getApplicableViews() {
		if (applicableViews == null) {
			applicableViews = new ArrayList<Class<? extends View>>();
			applicableViews.add(AbstractView.class);
		}
		return applicableViews;
	}

	public boolean isApplicableFor(View view) {
		return false;
	}

	public void setActive(boolean active) {
		// TODO Auto-generated method stub

	}

	public boolean isActive() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.constraints.VisualConstraint#addStatusListener(critiquer.constraints.ConstraintStatusListener)
     */
	public void addStatusListener(ConstraintStatusListener l) {
		// TODO Auto-generated method stub

	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.constraints.VisualConstraint#removeStatusListener(critiquer.constraints.ConstraintStatusListener)
     */
	public void removeStatusListener(ConstraintStatusListener l) {
		// TODO Auto-generated method stub

	}

	/**
     * @return Returns the type.
     */
	public Type getType() {
		return type;
	}

	/**
     * @param type
     *            The type to set.
     */
	public void setType(Type type) {
		this.type = type;
	}

	/**
     * @return Returns the maxSlack.
     */
	public int getMaxSlack() {
		return maxSlack;
	}

	/**
     * @param maxSlack
     *            The maxSlack to set.
     */
	public void setMaxSlack(int maxSlack) {
		this.maxSlack = maxSlack;
	}

	/**
     * @return Returns the minSlack.
     */
	public int getMinSlack() {
		return minSlack;
	}

	/**
     * @param minSlack
     *            The minSlack to set.
     */
	public void setMinSlack(int minSlack) {
		this.minSlack = minSlack;
	}

}
