/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.util.Set;
import java.util.Map.Entry;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.NumberSelectionParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.activities.parameters.TimeParameter;
import critiquer.gui.View;



/**
 * This class represents the constraint that activities must be completed within a certain time frame from the time they are started 
 * 
 */
public class TooLongTime extends StandardConstraint {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	
	/* (non-Javadoc)
	 * @see critiquer.activities.AbstractParamChangedListener#activityCreated(critiquer.activities.Activity)
	 */
	@Override
	public void activityCreated(Activity activity) {
		checkViolation(activity, ActivityUtils.getStartTimeParameter(activity), ActivityUtils.getEndTimeParameter(activity));
	}

	/* (non-Javadoc)
	 * @see critiquer.activities.AbstractParamChangedListener#activityUpdated(critiquer.activities.Activity)
	 */
	@Override
	public void activityUpdated(Activity activity) {
		checkViolation(activity, ActivityUtils.getStartTimeParameter(activity), ActivityUtils.getEndTimeParameter(activity));
	}

	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		TimeParameter startParam = ActivityUtils.getStartTimeParameter(activity);
		TimeParameter endParam = ActivityUtils.getEndTimeParameter(activity);
		NumberSelectionParameter maxTimeParam = ActivityUtils.getMaxTimeParameter(activity);
		if (p == startParam || p == endParam || p == maxTimeParam) {
			checkViolation(activity, startParam, endParam);
			
		}
	
	}

	private void checkViolation(Activity activity, TimeParameter startParam, TimeParameter endParam) {
		// Time currently scheduled for activity, in minutes
		
		try {

			long activityTime = ((endParam.getValue().getTimeInMillis() - startParam.getValue().getTimeInMillis()) / 60000);
			if (activityTime > activity.getMaxTime()) {
				violationActivities.put(activity, true);
			} else {
				violationActivities.put(activity, false);
			}
			
		} catch (NullPointerException e) {
			/**
			 * Quite alright, this just be due to that this is called the very first tiem
			 */
		}
		setConsistent(!violationActivities.containsValue(Boolean.TRUE));
	}

	public void viewUpdated(View v, Graphics2D g2) {
		Set<Entry<Activity, Boolean>> set = violationActivities.entrySet();
		Activity activity;
		Boolean violation;
		for (Entry<Activity, Boolean> entry : set) {
			activity = entry.getKey();
			violation = entry.getValue();
			Shape activityArea = v.getActivityArea(activity);
			/*
			 * only visualize activities that the view shows.
			 * 
			 */
			if (violation && (activityArea != null)) {
				
				Color oldColor = g2.getColor();
				g2.setColor(getColor());
				Stroke s = new BasicStroke(3,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND,0,new float[] {3},0);
				Stroke oldStroke = g2.getStroke();
				g2.setStroke(s);
				g2.draw(activityArea);
				g2.setStroke(oldStroke);
				g2.setColor(oldColor);
			}
		}
	}



	public String getDescription() {
		String nl = System.getProperty("line.separator");
		return "Draws a red line around missions that take"+nl+
		"too long time to complete according to a"+nl+
		"specified max time."+nl+nl+"The maximum allowed time depends on the type of activity";
	}

}
