package critiquer.gui;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.DateFormat;
import java.util.Calendar;

import org.apache.log4j.Logger;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.entity.CategoryItemEntity;
import org.jfree.chart.entity.ChartEntity;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.TimeParameter;



/**
 * 
 * <p>
 * Move the activities in the time line by dragging them. If you start the drag
 * close to either end of the activity (where <i>close</i> is defined as the
 * 15% of the activity rectangle closest to the ends), the duration is changed
 * instead. If you start your drag operation close to the beginning of the
 * activity, it will change the start time. If you begin the drag operation
 * close to the end, it will change the end time. Drag anywhere else and both
 * start and end are affected equally. 
 * </p>
 * 
 * @author olale
 * 
 */
final class ActivityDraggedAdapter extends MouseMotionAdapter {
	/**
     * 
     */
	private final TimeView   view;

	private final JFreeChart chart;

	/**
     * t0 is the last time selected in milliseconds by the current drag
     * operation.
     */

	private int			 t0			  = 0;

	/**
     * t1 is the current time in milliseconds selected by the current drag
     * operation.
     */
	private int			 t1			  = 0;

	
	private Activity		 draggedActivity = null;

	/**
     * How close to the ends of the activity should we begin extending the
     * activity instead of moving it?
     */
	private double		   sensitivity	 = .15;

	private Rectangle2D	  activityRectangle;

	private boolean		  moveStart;

	private boolean		  moveEnd;

	private transient Logger logger = Logger.getLogger(getClass());

	ActivityDraggedAdapter(TimeView view, JFreeChart chart) {
		super();
		this.view = view;
		this.chart = chart;
	}

	/**
     * 
     * Change the cursor so the user knows what a drag operation will do in the
     * current context
     * 
     * @see java.awt.event.MouseMotionAdapter#mouseMoved(java.awt.event.MouseEvent)
     */
	@Override
	public void mouseMoved(MouseEvent e) {
		if (shouldDrag(e.getPoint())) {
			if (moveStart && !moveEnd) {
				view.getChartPane().setCursor(new Cursor(Cursor.W_RESIZE_CURSOR));
			} else if (!moveStart && moveEnd) {
				view.getChartPane().setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
			} else if (moveStart && moveEnd) {
				view.getChartPane().setCursor(new Cursor(Cursor.HAND_CURSOR));
			}
		} else {
			view.getChartPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		if (t0 != 0 || t1 != 0) {
			t0 = t1 = 0;
			draggedActivity = null;
		}

	}

	private boolean closeToStart(Rectangle2D rect, Point point) {
		return Math.abs(point.x - rect.getX()) / rect.getWidth() < sensitivity;
	}

	private boolean closeToEnd(Rectangle2D rect, Point point) {
		return Math.abs(point.x - rect.getMaxX()) / rect.getWidth() < sensitivity;
	}

	/**
     * Change start and/or the end time of an activity as part of the drag
     * operation
     * 
     * @see java.awt.event.MouseMotionAdapter#mouseDragged(java.awt.event.MouseEvent)
     */
	@Override
	public void mouseDragged(MouseEvent e) {
		/**
         * We must be listening to events from the ChartPanel..
         */
		assert (e.getSource() == view.getChartPane());
		TimeLinePlot plot = (TimeLinePlot) chart.getPlot();
		Point mousePosition = e.getPoint();

		/**
         * Translate the mouse position so that it has the same coordinate
         * system as the activity areas
         * 
         */

		boolean shouldDrag = false;
		if (draggedActivity == null) {
			shouldDrag = shouldDrag(mousePosition);
		} else {
			shouldDrag = true;
		}
		if (shouldDrag) {
			int pendingupdates = draggedActivity.getPendingupdates();
			if (pendingupdates>0) {				
				logger.warn("There are "+ pendingupdates + " pending updates during drag!");
			}
			TimeParameter startParam = ActivityUtils.getStartTimeParameter(draggedActivity);
			TimeParameter endParam = ActivityUtils.getEndTimeParameter(draggedActivity);
			long diff = 0;
			/**
             * First time called with this activity, so initialize both t0 and
             * t1.
             */
			if (t1 == t0 && t0 == 0) {
				t0 = t1 = e.getX();
			} else {
				t1 = e.getX();
			}
			diff = plot.locationToDate(t1).getTimeInMillis()-plot.locationToDate(t0).getTimeInMillis();;
			boolean startToBeMoved = moveStart && startParam.isEditable();
			boolean endToBeMoved = moveEnd && endParam.isEditable();
			boolean moveActivity = startToBeMoved && endToBeMoved;
			if (moveActivity) {
				try {
					draggedActivity.getAtomicUpdateLock().acquire();
					moveTimeParam(startParam, diff);
					moveTimeParam(endParam, diff);
					draggedActivity.getAtomicUpdateLock().release();
					draggedActivity.signalParamChanged();
				} catch (InterruptedException exception) {
					logger.error("Mouse drag interrupted");
				}
			} else if (startToBeMoved) {
				moveTimeParam(startParam, diff);
			} else if (endToBeMoved) {
				moveTimeParam(endParam, diff);
			}
			/**
             * Compensate for moving the activity
             */
			t0 = t1;

		}

	}

	/**
     * Initialize the drag operation. This entails two things:
     * 
     * <ul>
     * 
     * <li>selecting the activity which will be dragged, if any </li>
     * 
     * <li>determining if the activity should be moved or extended in time.
     * </li>
     * 
     * </ul>
     * 
     * @param mousePosition
     * 
     * @return true if the mousePosition is within an activity area, false
     *         otherwise.
     */
	private boolean shouldDrag(Point mousePosition) {
		ChartEntity entity = view.getChartPane().getEntityForPoint(mousePosition.x, mousePosition.y);
		boolean withinActivityArea = false;
		
		if (entity != null && entity instanceof CategoryItemEntity) {
			withinActivityArea = true;
			CategoryItemEntity timeBar = (CategoryItemEntity) entity;
			
			draggedActivity = ActivityUtils.getActivityByName((Comparable) timeBar.getCategory());
			activityRectangle = ((Rectangle2D) timeBar.getArea());
			Point2D truePos = view.getChartPane().translateScreenToJava2D(mousePosition);
			mousePosition.x = (int) truePos.getX();
			mousePosition.y = (int) truePos.getY();
			moveStart = !closeToEnd(activityRectangle, mousePosition);
			moveEnd = !closeToStart(activityRectangle, mousePosition);
		}
		return withinActivityArea;
	}

	private void moveTimeParam(TimeParameter timeParam, long diff) {
		Calendar newTime = null;
		newTime = (Calendar) timeParam.getValue().clone();
		newTime.add(Calendar.MILLISECOND, (int) diff);
		//logModification(timeParam, timeParam.getValue(), newTime);
		timeParam.setValue(newTime);
	}
	private void logModification(TimeParameter modifiedParameter, Calendar oldValue, Calendar newValue) {
		DateFormat shortFormat = DateFormat.getTimeInstance(DateFormat.SHORT);
		logger.debug(modifiedParameter.getFullName() + " changed from "
						   + shortFormat.format(oldValue.getTime())
						   + " to "
						   + shortFormat.format(newValue.getTime()));
	}
	
	
}