/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.TransferHandler;

import org.apache.log4j.Logger;
import org.tigris.gef.base.Selection;
import org.tigris.gef.graph.presentation.JGraph;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.gui.graph.ActivityFigNode;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;


public class AgentTransferHandler extends TransferHandler {

	/**
	 * 
	 */
	private static final long						serialVersionUID	= 1L;

	private transient ArrayList<TransferListener>	listeners			= new ArrayList<TransferListener>();

	private transient Logger logger;

	private static AgentTransferHandler				handler				= new AgentTransferHandler();

	private AgentTransferHandler() {
		super();
		logger = Logger.getLogger(getClass());
	}

	public static AgentTransferHandler getAgentTransferHandler() {
		return handler;
	}

	public void addTransferListener(TransferListener l) {
		listeners.add(l);
	}

	/**
	 *
	 * Show the agent image if dragging an agent
	 * 
	 * @see javax.swing.TransferHandler#getVisualRepresentation(java.awt.datatransfer.Transferable)
	 */
	@Override
	public Icon getVisualRepresentation(Transferable t) {
		if (t instanceof AgentSelection) {
			AgentSelection agentSelection = (AgentSelection) t;
			return agentSelection.getPicture();
		} else {			
			return super.getVisualRepresentation(t);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.TransferHandler#canImport(javax.swing.JComponent,
	 *      java.awt.datatransfer.DataFlavor[])
	 */
	@Override
	public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
		boolean canImportFlavors = (comp instanceof TrashLabel || comp instanceof JGraph) && Arrays.asList(transferFlavors).contains(AgentSelection.agentFlavor);
		return canImportFlavors;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.TransferHandler#getSourceActions(javax.swing.JComponent)
	 */
	@Override
	public int getSourceActions(JComponent c) {
		return TransferHandler.COPY;
	}

	/**
	 * This is to be used when an agent is dragged from an agent label
	 * 
	 * @see javax.swing.TransferHandler#createTransferable(javax.swing.JComponent)
	 */
	@Override
	protected Transferable createTransferable(JComponent c) {
		Transferable t = null;
		if (c instanceof AgentLabel) {
			AgentLabel agentLabel = (AgentLabel) c;
			t = new AgentSelection(agentLabel.getAgent(), agentLabel.getAgentParam());
		}
		for (TransferListener listener : listeners) {
			listener.transferInitialized();
		}

		return t;
	}

	/**
	 * <p>
	 * Select which agents are available for a commander when a drop is
	 * performed in the graph of the resource view. Actually we have activities
	 * in the graph view, but we can only select activities that have an
	 * assigned commander and all activities with the same commander are
	 * selected if one is, so as to highlight which activities are affected
	 * </p>
	 * 
	 * @see javax.swing.TransferHandler#importData(javax.swing.JComponent,
	 *      java.awt.datatransfer.Transferable)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		boolean success = false;
		
		if (comp instanceof JGraph) {
			JGraph graph = (JGraph) comp;
			Vector<Selection> selections = graph.getEditor().getSelectionManager().selections();
			logger.debug("importData called with JGraph");
			for (Selection selection : selections) {
				if (selection.getContent() instanceof ActivityFigNode) {
					ActivityFigNode activityFigNode = (ActivityFigNode) selection.getContent();
					Activity selectedActivity = activityFigNode.getActivityNode().getActivity();
					logger.debug("Attempting to import agent to "+selectedActivity);
					if (selectedActivity.isAgentAssignable()) {
						try {
							success = ActivityUtils	.getMainAgentParameter(selectedActivity)
											.selectAgent((Agent) t.getTransferData(AgentSelection.agentFlavor));
							logger.debug("Selecting agent "+(success?"succeeded":"failed"));
						} catch (Exception e) {
							StringWriter stackTraceWriter = new StringWriter();
							e.printStackTrace(new PrintWriter(stackTraceWriter));
							logger.error(e+"\n"+stackTraceWriter.getBuffer());
						}
					} else {
						logger.debug(selectedActivity+" needs a commander");
					}
				}
			}

		} else if (comp instanceof TrashLabel) {
			
			try {
				Vector<Selection> selections = ReferenceHolder.resourceView	.getGraph()
																			.getEditor()
																			.getSelectionManager()
																			.selections();
				for (Selection selection : selections) {
					if (selection.getContent() instanceof ActivityFigNode) {
						ActivityFigNode activityFigNode = (ActivityFigNode) selection.getContent();
						Activity selectedActivity = activityFigNode.getActivityNode().getActivity();
						
						/**
                         * Remove an activity from an agent
                         */
						try {
							ActivityUtils.getMainAgentParameter(selectedActivity)
										 .deSelectAgent((Agent) t.getTransferData(AgentSelection.agentFlavor),
												 selectedActivity);
							success = true;
						} catch (UnsupportedFlavorException e) {
							logger.error(e);
						} catch (IOException e) {
							logger.error(e);
						}

						
					}
				}
			} catch (NullPointerException e) {
				logger.error(e);
			}
			
			
			
		}
		if (success) {
			for (TransferListener listener : listeners) {
				listener.transferComplete();
			}
		}
		return success;
	}
}
