/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Shape;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.tigris.gef.base.Editor;
import org.tigris.gef.event.GraphSelectionEvent;
import org.tigris.gef.event.GraphSelectionListener;
import org.tigris.gef.graph.presentation.JGraph;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.AgentParameter;
import critiquer.activities.parameters.CommanderParameter;
import critiquer.gui.graph.ActivityFigNode;
import critiquer.gui.graph.ActivityNode;
import critiquer.misc.ReferenceHolder;



/**
 * 
 * <p>
 * This view is used to allocate resources to Commanders. Here's how it works:
 * </p>
 * 
 * <ul>
 * 
 * <li>The user selects a Commander by selecting any of the activities that
 * have commanders allocated to them (marked as gray) </li>
 * 
 * <li>When an activity with a commander is selected, all activities that this
 * commander is selected to take care of are marked as selected. Therefore, we
 * mark that there is a <i>Commander</i> selected, not an individual
 * <i>Activity</i> </li>
 * 
 * <li>When an Agent is dragged from the list at the bottom to the activity
 * canvas, the selected Agent is added to the list of agents the <i>Commander</i>
 * controls, that is, it is made available to all activities that the
 * <i>Commander</i> is in charge of. </li>
 * 
 * <li>If there are sub-activities to any of those activities that have
 * commanders associated with them, they also have the agent available to them.
 * This may not be the desired behaviour, but for now that is the way it works.
 * </li>
 * 
 * </ul>
 * 
 * @author olale
 * 
 */
public class ResourceView extends AbstractView implements
/**
 * We don't need to implement ActivityUpdateListener since the graph model is
 * the same as the one used by the task view
 */
// ActivityUpdateListener,
											  TransferListener {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	private transient JPanel  resourcePanel;

	private JGraph			graph;

	protected Component	   defaultLabel;

	private boolean		   internalSelectionUpdate;

	protected Activity		selectedActivity;

	private JPanel			southPanel;

	/**
     * When we use the graph view, we use this constructor instead.
     */

	public ResourceView() {
		super(new BorderLayout());
		ReferenceHolder.resourceView = this;
		graph = new JGraph(ReferenceHolder.getGraphModel());
		// graph.getEditor().setGridHidden(true);
		Dimension graphSize = new Dimension(500, 500);
		graph.setDrawingSize(graphSize);
		graph.setPreferredSize(graphSize);
		
		defaultLabel = new JLabel("No resources to display");
		internalSelectionUpdate = false;
		/**
         * Funky way to make sure we select all those activities that have the
         * same commander, so that they all receive the same agent assignments
         */

		final GraphSelectionListener graphSelectionHandler = new GraphSelectionListener() {

			public void selectionChanged(GraphSelectionEvent gse) {
				Vector selection = gse.getSelections();
				if (!internalSelectionUpdate) {
					if (selection.size() == 1 && selection.get(0) instanceof ActivityFigNode) {
						ActivityFigNode activityFigNode = (ActivityFigNode) selection.get(0);
						selectedActivity = activityFigNode.getActivityNode().getActivity();
						updateResourcePanel(selectedActivity);
						ActivityParameter commanderParam = ActivityUtils.getParamByClassAndName(selectedActivity,
								CommanderParameter.class,
								"commander");
						if (selectedActivity.isAgentAssignable()) {
							internalSelectionUpdate = true;
							List nodes = graph.getGraphModel().getNodes();
							Editor editor = graph.getEditor();
							for (Object object : nodes) {
								if (object instanceof ActivityNode) {
									ActivityNode node = (ActivityNode) object;
									ActivityParameter commanderParam2 = ActivityUtils.getParamByClassAndName(node.getActivity(),
											CommanderParameter.class,
											"commander");
									if (node.getActivity().isAgentAssignable() && commanderParam.getValue()
																								.equals(commanderParam2.getValue())) {
										editor.getSelectionManager()
											  .addToSelection(node.presentationFor(editor.getLayerManager()
																						 .getActiveLayer()));
									}
								}
							}
							editor.damageAll();
							internalSelectionUpdate = false;
						}
					} else {
						updateResourcePanel(null);
					}

				}

			}

		};
		TaskView.addGraphSelectionListener(graph, graphSelectionHandler);

		graph.setTransferHandler(AgentTransferHandler.getAgentTransferHandler());
		AgentTransferHandler.getAgentTransferHandler().addTransferListener(this);
		Activity.addNameChangeListener(new NameChangeListener() {

			public void nameChanged() {
				graph.revalidate();
				graph.repaint();
			}

		});
		resourcePanel = new JPanel(new GridLayout(2,1));

		add(graph, BorderLayout.CENTER);
		
		southPanel = new JPanel(new BorderLayout());
		southPanel.add(resourcePanel, BorderLayout.CENTER);
		southPanel.add(critiqueComponent.getPanel(), BorderLayout.SOUTH);

		add(southPanel, BorderLayout.SOUTH);
	}

	protected void updateResourcePanel(Activity activity) {
		resourcePanel.removeAll();

		if (activity != null) {
			AgentParameter mainAgentParameter = ActivityUtils.getMainAgentParameter(activity);
			if (mainAgentParameter != null) {
				resourcePanel.add(mainAgentParameter.getAgentPanel());
				resourcePanel.add(new TrashLabel());
			}
		} else {
			resourcePanel.add(defaultLabel);
		}
		revalidate();
		repaint();
	}

	public Shape getActivityArea(Activity activity) {
		Shape activityShape = null;
		activityShape = TaskView.getActivityArea(graph, activity);
		return activityShape;
	}

	public ArrayList<Activity> getEvaluationActivities() {
		ArrayList<Activity> activities = new ArrayList<Activity>();
		Vector figs = graph.getEditor().getSelectionManager().getFigs();
		for (Object object : figs) {
			if (object instanceof ActivityFigNode) {
				ActivityFigNode activityFig = (ActivityFigNode) object;
				activities.add(activityFig.getActivityNode().getActivity());
			}
		}
		return activities;
	}

	public void transferComplete() {
		// treePane.setFreezed(false);
		revalidate();
		repaint();
	}

	public void transferInitialized() {
		// treePane.setFreezed(true);
	}

	public JPanel getResourcePanel() {
		return resourcePanel;
	}

	/**
     * @return Returns the graph.
     */
	public JGraph getGraph() {
		return graph;
	}

}
