/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;

import critiquer.activities.parameters.Location;




/**
 * This class implements the graphical representation of a map, along with
 * functionality for recognizing mouse gestures that define the location
 * parameter values
 * 
 * @author olale
 * 
 */
public class ScrollableMap extends JLabel implements Scrollable, MouseMotionListener, MouseListener {

	/**
     * 
     */
	private static final long serialVersionUID  = 1L;

	private int			   maxUnitIncrement  = 1;

	private boolean		   missingPicture	= false;

	private int			   scale;

	private Point			 t0;

	private Point			 t1;

	private Location selectedLocation = null;

	private boolean		   drawTempRectangle = false;

	private GeoView view;

	/**
     * 
     * Paint both the map and the locations, with the selected one highlighted
     * 
     * @see javax.swing.JComponent#paint(java.awt.Graphics)
     */
	@Override
	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2 = (Graphics2D)g;
		Color oldColor = g2.getColor();
		g2.setColor(Color.BLACK);

		DefaultComboBoxModel locations = view.getLocationModel();
		Location location = null;
		for (int i = 0; i < locations.getSize(); i++) {
			location = (Location)locations.getElementAt(i);
			
			drawLocation(g2, location);
		}
		Stroke stroke = g2.getStroke();
		g2.setStroke(new BasicStroke(5));
		drawLocation(g2, selectedLocation);
		g2.setStroke(stroke);
		if (drawTempRectangle) {
			g2.drawRect(t0.x, t0.y, t1.x - t0.x, t1.y - t0.y);
		}
		g2.setColor(oldColor);

	}

	private void drawLocation(Graphics2D g2, Location location) {
		if (location != null && location.getArea()!=null) {
			Rectangle rect = location.getArea();
			g2.drawRect(rect.x, rect.y, rect.width, rect.height);
		}
	}

	public ScrollableMap(ImageIcon i, int scale,GeoView view) {
		super(i);
		this.view=view;
		if (i == null) {
			missingPicture = true;
			setText("No picture found.");
			setHorizontalAlignment(CENTER);
			setOpaque(true);
			setBackground(Color.white);
		}
		maxUnitIncrement = 10;
		this.scale = scale;
		t0 = new Point();
		t1 = new Point();
		// Let the user scroll by dragging to outside the window.
		setAutoscrolls(true); // enable synthetic drag events
		addMouseMotionListener(this); // handle mouse drags
		addMouseListener(this);
	}

	// Methods required by the MouseMotionListener interface:
	public void mouseMoved(MouseEvent e) {
		Point tmp = new Point();

		if (!(t0.equals(tmp) && t1.equals(tmp)) && selectedLocation != null) {
			
			boolean setArea = JOptionPane.showConfirmDialog(this,
					"Set " + selectedLocation.toString()+" to selected area?",
					"Determine location",
					JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
			if (setArea) {
				drawTempRectangle = false;
				/**
				 * Create a rectangular area between t0 and t1 and assign it as a
				 * value to the currently selected LocationParameter
				 */
				Location l = selectedLocation;
				l.setArea(new Rectangle(t0, new Dimension(t1.x - t0.x, t1.y - t0.y)));
				
			}

			t0 = new Point();
			t1 = new Point();

		}
	}

	public void mouseDragged(MouseEvent e) {
		// The user is dragging us, so scroll!

		if (t0.equals(t1) && t0.equals(new Point())) {
			t0 = t1 = e.getPoint();
		} else {
			t1 = e.getPoint();
			drawTempRectangle = true;
			repaint();
		}

		Rectangle r = new Rectangle(e.getX(), e.getY(), 1, 1);
		scrollRectToVisible(r);
	}

	public Dimension getPreferredSize() {
		if (missingPicture) {
			return new Dimension(320, 480);
		} else {
			return super.getPreferredSize();
		}
	}

	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		// Get the current position.
		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL) {
			currentPosition = visibleRect.x;
		} else {
			currentPosition = visibleRect.y;
		}

		// Return the number of pixels between currentPosition
		// and the nearest tick mark in the indicated direction.
		if (direction < 0) {
			int newPosition = currentPosition - (currentPosition / maxUnitIncrement) * maxUnitIncrement;
			return (newPosition == 0) ? maxUnitIncrement : newPosition;
		} else {
			return ((currentPosition / maxUnitIncrement) + 1) * maxUnitIncrement - currentPosition;
		}
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL) {
			return visibleRect.width - maxUnitIncrement;
		} else {
			return visibleRect.height - maxUnitIncrement;
		}
	}

	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	public void setMaxUnitIncrement(int pixels) {
		maxUnitIncrement = pixels;
	}

	public void mouseClicked(MouseEvent e) {

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	/**
     * @return Returns the param.
     */
	public Location getSelectedLocation() {
		return selectedLocation;
	}

	/**
     * @param param
     *            The param to set.
     */
	public void setSelectedLocation(Location loc) {
		this.selectedLocation = loc;
	}
}
