/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;

import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.CategoryItemEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.labels.CategoryItemLabelGenerator;
import org.jfree.chart.labels.CategoryToolTipGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRendererState;
import org.jfree.chart.renderer.category.GanttRenderer;
import org.jfree.data.gantt.GanttCategoryDataset;
import org.jfree.data.gantt.TaskSeriesCollection;
import org.jfree.ui.RectangleEdge;

public class TimeLineRenderer extends GanttRenderer {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/*
	 * Each tasks series in the collection has to do with one agent. Tasks can occur in more than one series
	 * */
	private TaskSeriesCollection taskSeries;
	private HashMap<Comparable,Paint> colors;

	private ArrayList<TimeLineRenderingListener> listeners = new ArrayList<TimeLineRenderingListener>();
	public TimeLineRenderer(TaskSeriesCollection c,HashMap<Comparable,Paint> colorMap){
		colors=colorMap;
		taskSeries = c;
	
	}
	
	/**
	 * Use the same colors for the resources. These colors are defined by Agent.setTimeLineColor
	 * 
	 * */ 
	@Override
	public Paint getSeriesPaint(int series) {
		Comparable resourceName = taskSeries.getRowKey(series);
		return colors.get(resourceName);
		
	}
	/**
     * Draws a single task.
     *
     * @param g2  the graphics device.
     * @param state  the renderer state.
     * @param dataArea  the data plot area.
     * @param plot  the plot.
     * @param domainAxis  the domain axis.
     * @param rangeAxis  the range axis.
     * @param dataset  the data.
     * @param row  the row index (zero-based).
     * @param column  the column index (zero-based).
     */
	protected void drawTask(Graphics2D g2,
			CategoryItemRendererState state,
			Rectangle2D dataArea,
			CategoryPlot plot,
			CategoryAxis domainAxis,
			ValueAxis rangeAxis,
			GanttCategoryDataset dataset,
			int row,
			int column) {
		
		PlotOrientation orientation = plot.getOrientation();

        RectangleEdge rangeAxisLocation = plot.getRangeAxisEdge();
        
        // Y0
        Number value0 = dataset.getEndValue(row, column);
        if (value0 == null) {
            return;
        }
        double java2dValue0 = rangeAxis.valueToJava2D(
            value0.doubleValue(), dataArea, rangeAxisLocation
        );

        // Y1
        Number value1 = dataset.getStartValue(row, column);
        if (value1 == null) {
            return;
        }
        double java2dValue1 = rangeAxis.valueToJava2D(
            value1.doubleValue(), dataArea, rangeAxisLocation
        );

        if (java2dValue1 < java2dValue0) {
            double temp = java2dValue1;
            java2dValue1 = java2dValue0;
            java2dValue0 = temp;
            Number tempNum = value1;
            value1 = value0;
            value0 = tempNum;
        }

        double rectStart = calculateBarW0(
            plot, orientation, dataArea, domainAxis, state, row, column
        );
        double rectBreadth = state.getBarWidth();
        double rectLength = Math.abs(java2dValue1 - java2dValue0);
        
        Rectangle2D bar = null;
        if (orientation == PlotOrientation.HORIZONTAL) {
            bar = new Rectangle2D.Double(
                java2dValue0, rectStart, rectLength, rectBreadth
            );
        }
        else if (orientation == PlotOrientation.VERTICAL) {
            bar = new Rectangle2D.Double(
                rectStart, java2dValue1, rectBreadth, rectLength
            );
        }

        Rectangle2D completeBar = null;
        Rectangle2D incompleteBar = null;
        Number percent = dataset.getPercentComplete(row, column);
        double start = getStartPercent();
        double end = getEndPercent();
        if (percent != null) {
            double p = percent.doubleValue();
            if (plot.getOrientation() == PlotOrientation.HORIZONTAL) {
                completeBar = new Rectangle2D.Double(
                    java2dValue0, 
                    rectStart + start * rectBreadth, 
                    rectLength * p, 
                    rectBreadth * (end - start)
                );
                incompleteBar = new Rectangle2D.Double(
                    java2dValue0 + rectLength * p, 
                    rectStart + start * rectBreadth, 
                    rectLength * (1 - p), 
                    rectBreadth * (end - start)
                );
            }
            else if (plot.getOrientation() == PlotOrientation.VERTICAL) {
                completeBar = new Rectangle2D.Double(
                    rectStart + start * rectBreadth, 
                    java2dValue1 + rectLength * (1 - p), 
                    rectBreadth * (end - start), 
                    rectLength * p
                );
                incompleteBar = new Rectangle2D.Double(
                    rectStart + start * rectBreadth, 
                    java2dValue1, 
                    rectBreadth * (end - start), 
                    rectLength * (1 - p)
                );
            }
                
        }

        /*
         * BEGIN Added code
         * 
         * Add the bar to the list of bars for this task series
         * */
        
        if (dataset instanceof TaskSeriesCollection) {
			TaskSeriesCollection collection = (TaskSeriesCollection) dataset;
			Comparable key = collection.getColumnKey(column);
			notifyListeners(key,bar);
		}
        
        /*
         * END Added code
         * 
         * */
        
        Paint seriesPaint = getItemPaint(row, column);
        g2.setPaint(seriesPaint);
        g2.fill(bar);

        if (completeBar != null) {
            g2.setPaint(getCompletePaint());
            g2.fill(completeBar);
        }
        if (incompleteBar != null) {
            g2.setPaint(getIncompletePaint());
            g2.fill(incompleteBar);
        }
        
        // draw the outline...
        if (isDrawBarOutline() 
                && state.getBarWidth() > BAR_OUTLINE_WIDTH_THRESHOLD) {
            Stroke stroke = getItemOutlineStroke(row, column);
            Paint paint = getItemOutlinePaint(row, column);
            if (stroke != null && paint != null) {
                g2.setStroke(stroke);
                g2.setPaint(paint);
                g2.draw(bar);
            }
        }
        
        CategoryItemLabelGenerator generator 
            = getItemLabelGenerator(row, column);
        if (generator != null && isItemLabelVisible(row, column)) {
            drawItemLabel(
                g2, dataset, row, column, plot, generator, bar, false
            );
        }        
        
        // collect entity and tool tip information...
        if (state.getInfo() != null) {
            EntityCollection entities = state.getEntityCollection();
            if (entities != null) {
                String tip = null;
                CategoryToolTipGenerator tipster = getToolTipGenerator(
                    row, column
                );
                if (tipster != null) {
                    tip = tipster.generateToolTip(dataset, row, column);
                }
                String url = null;
                if (getItemURLGenerator(row, column) != null) {
                    url = getItemURLGenerator(row, column).generateURL(
                        dataset, row, column
                    );
                }
                CategoryItemEntity entity = new CategoryItemEntity(
                    bar, tip, url, dataset, row, 
                    dataset.getColumnKey(column), column
                );
                entities.add(entity);
            }
        }

    }

	/**
	 * @param key
	 * @param bar
	 */
	private void notifyListeners(Comparable key, Rectangle2D bar) {
		ArrayList<TimeLineRenderingListener> l = listeners;
		for (TimeLineRenderingListener listener : l) {
			listener.updateActivityArea(key, bar);
		}
	}

	public void addListener(TimeLineRenderingListener l) {
		listeners.add(l);
	}



}
