/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.gantt.Task;
import org.jfree.data.gantt.TaskSeries;
import org.jfree.data.gantt.TaskSeriesCollection;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.general.DatasetChangeListener;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.Parameter;
import critiquer.constraints.VisualConstraint;
import critiquer.constraints.VisualConstraints;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;
import critiquer.simulation.SimulationEngine;

/**
 * <p>
 * This class presents the time view. It is a bit special since the constraints
 * are notified by the TimeLinePlot and not the view itself. That's because we
 * want to interact with the timeline and therefore it's easier if the input
 * coordinate system is the same as the one used by the activity areas. The
 * constraints are therefore only given the graphics context of the chart area
 * when they display their information.
 * </p>
 * 
 * <p>
 * Interaction in this view goes as follows:
 * </p>
 * <ul>
 * 
 * <li>The user drags activities back and forth to change them.</li>
 * 
 * <li>If the user begins a drag operation close to either end of the activity,
 * it is extended in time, otherwise moved.</li>
 * 
 * </ul>
 */

public class TimeView extends JPanel implements
									View,
									DatasetChangeListener,
									TimeLineRenderingListener,
									ActivityUpdateListener {

	/**
     * 
     */
	private static final long			  serialVersionUID  = 1L;

	private ChartPanel					 chartPane;

	private TaskSeriesCollection		   tasks;

	private JList						  activityList;

	private DefaultListModel			   listModel;

	private TimeLinePlot				   plot;

	private HashMap<Comparable, Paint>	 colorMap		  = new HashMap<Comparable, Paint>();

	private JSlider						slider			= new JSlider();

	private JPanel						 southPanel;

	private JPanel						 timePanel;

	private JScrollPane					listPane;

	private SimulationEngine			   simulationEngine;

	protected ConstraintComponent		  critiqueComponent = new ConstraintComponent(this);

	private HashMap<Activity, Rectangle2D> activityAreas	 = new HashMap<Activity, Rectangle2D>();

	private Thread						 updateThread	  = null;

	public JPanel getConstraintPanel() {
		return critiqueComponent.getPanel();
	}

	public TimeView() {
		ReferenceHolder.timeView = this;
		setLayout(new BorderLayout());
		critiqueComponent.getPanel().setBorder(BorderFactory.createEtchedBorder());

		initActivityList();
		southPanel = new JPanel(new BorderLayout());
		southPanel.add(listPane, BorderLayout.CENTER);
		southPanel.add(critiqueComponent.getPanel(), BorderLayout.SOUTH);
		add(southPanel, BorderLayout.SOUTH);
		initTimePanel();
		add(timePanel, BorderLayout.CENTER);
		simulationEngine = new SimulationEngine();
	}

	private void initTimePanel() {

		slider = new JSlider();
		/**
		 * Disable the slider until there are activities to show
		 */
		slider.setEnabled(false);
		slider.addChangeListener(new ChangeListener() {

			/**
			 * When the slider is updated, run the simulation engine based on
			 * the time difference and update the graphical information
			 */
			public void stateChanged(ChangeEvent e) {
				if (slider.isEnabled()) {

					Double value = new Double(slider.getValue());
					setCurrentTime(value);
					simulationEngine.stepTo(getCurrentTime());
					repaint();

				}
			}

		});
		initColorMap();
		final JFreeChart timeChart = createTimeChart();
		chartPane = new ChartPanel(timeChart);
		chartPane.setMouseZoomable(false);
		chartPane.addMouseMotionListener(new ActivityDraggedAdapter(this, timeChart));

		timePanel = new JPanel(new BorderLayout());
		timePanel.add(chartPane, BorderLayout.CENTER);
		timePanel.add(slider, BorderLayout.NORTH);

	}

	public void updateView(Graphics2D g2) {
		for (VisualConstraint constraint : VisualConstraints.getActiveConstraints()) {
			if (constraint.isApplicableFor(this)) {
				constraint.viewUpdated(this, g2);
			}
		}
	}

	private void initActivityList() {
		listModel = new DefaultListModel();

		activityList = new JList(listModel);
		Activity.addActivityUpdateListener(this);
		activityList.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				initSimulation();
				updateTimeChart();
			}

		});
		listPane = new JScrollPane(activityList);
		listPane.setPreferredSize(new Dimension(200, 80));

	}

	private JFreeChart createTimeChart() {
		tasks = new TaskSeriesCollection();
		tasks.addChangeListener(this);
		CategoryAxis categoryAxis = new CategoryAxis("Tasks");
		DateAxis dateAxis = new DateAxis("Time");
		TimeLineRenderer renderer = new TimeLineRenderer(tasks, colorMap);
		renderer.addListener(this);
		plot = new TimeLinePlot(tasks, categoryAxis, dateAxis, renderer);

		/**
		 * This was a failed attempt at making the range axis have a fixed range
		 */
		// GregorianCalendar now = new GregorianCalendar();
		// GregorianCalendar then = new GregorianCalendar();
		// then.add(Calendar.DAY_OF_YEAR,1);
		// dateAxis.setRangeWithMargins(new
		// Range(now.getTimeInMillis(),then.getTimeInMillis()), false, false);
		// dateAxis.setAutoRange(false);
		//
		plot.setView(this);
		plot.setOrientation(PlotOrientation.HORIZONTAL);
		plot.getDataset();
		return new JFreeChart("Mission tasks", JFreeChart.DEFAULT_TITLE_FONT, plot, true);
	}

	/**
	 * <p>
	 * Each agent has it's own task series for every activity. A task series is
	 * used in the time view to list all activities over time
	 * </p>
	 * <p>
	 * If there is no specific agent assigned to this task but the task still
	 * has a duration, assign a dummy agent to a task that spans that duration.
	 * </p>
	 * 
	 * This must not be performed during an atomic activity update.
	 * 
	 */
	private ArrayList<TaskSeries> createTaskSeries(Activity activity) {
		ArrayList<TaskSeries> collection = new ArrayList<TaskSeries>();

		HashSet<Agent> agents = ActivityUtils.getAgentsInActivity(activity);
		Date start = activity.getStartTime();
		Date end = activity.getEndTime();
		if (agents.isEmpty() && end.after(start)) {
			TaskSeries series = new TaskSeries(Agent.dummyAgent.toString());
			series.add(new Task(activity.toString(), start, end));
			collection.add(series);
		} else if (!agents.isEmpty()) {
			for (Agent agent : agents) {
				TaskSeries series = new TaskSeries(agent.toString());
				series.add(new Task(activity.toString(), start, end));
				collection.add(series);
			}
		}
		return collection;
	}

	/**
	 * Create a new tasks series from the activities in the list model. Make
	 * sure there is only one task series for each resource.
	 */

	@SuppressWarnings("unchecked")
	private void updateTimeChart() {
		synchronized (tasks) {
			tasks.removeAll();
			HashMap<String, TaskSeries> resources = new HashMap<String, TaskSeries>();
			Object[] selectedActivities = activityList.getSelectedValues();
			for (int i = 0; i < selectedActivities.length; i++) {
				Activity activity = (Activity) selectedActivities[i];
				if (activity.getAtomicUpdateLock().tryAcquire()) {
					ArrayList<TaskSeries> c = createTaskSeries(activity);
					for (TaskSeries s : c) {
						if (resources.keySet().contains(s.getKey())) {
							TaskSeries seriesInCollection = resources.get(s.getKey());
							List<Task> taskList = s.getTasks();
							for (Task task : taskList) {
								seriesInCollection.add(task);
							}
						} else {
							resources.put((String) s.getKey(), s);
						}
					}
					activity.getAtomicUpdateLock().release();
				}
			}
			Collection<TaskSeries> taskCollection = resources.values();
			for (TaskSeries series : taskCollection) {
				tasks.add(series);
			}
		}
	}

	public void initSimulation() {
		updateSlider();
		simulationEngine.init(getCurrentTime());
	}

	private void initColorMap() {
		colorMap.put(Agent.dummyAgent.toString(), Agent.dummyAgent.getTimeLineColor());
		for (Agent agent : ReferenceHolder.organizationModel.getAllAgents()) {
			colorMap.put(agent.toString(), agent.getTimeLineColor());
		}
	}

	/**
	 * <p>
	 * Update the slider whenever the chart should be updated. This should only
	 * have to be done once, but since we don't have the data area available
	 * until the graph has been drawn at least once, we do it every time the
	 * chart is updated.
	 * </p>
	 * <p>
	 * In order for the slider to work correctly, it should have the bounding
	 * rectangle of the data area in the time chart as its range since the plot
	 * is able to retrieve date values from <code>double</code> values within
	 * the range of the data area.
	 * </p>
	 */
	private void updateSlider() {
		slider.setMinimum((int) getTimeChartArea().getX());
		slider.setMaximum((int) getTimeChartArea().getMaxX());
		slider.setValue((int) getTimeChartArea().getX());
		setCurrentTime(getTimeChartArea().getX());
		slider.setEnabled(true);
	}

	/**
	 * @param a
	 *            The activity to set.
	 */
	public void activityUpdated(Activity a) {
		if (a != null) {
			listModel.removeAllElements();
			for (Activity activity : ReferenceHolder.allActivities) {
				listModel.addElement(activity);
			}
			// Is this really necessary?
			updateTimeChart();
		}
	}

	/**
	 * One may select only a set of activities in the Time View for which time
	 * charts should be drawn.
	 */
	public ArrayList<Activity> getEvaluationActivities() {
		Object[] selectedObjects = activityList.getSelectedValues();
		ArrayList<Activity> selectedActivities = new ArrayList<Activity>();
		for (Object object : selectedObjects) {
			if (object instanceof Activity) {
				Activity a = (Activity) object;
				selectedActivities.add(a);
			}
		}
		return selectedActivities;
	}

	public void datasetChanged(DatasetChangeEvent event) {
		activityAreas.clear();
	}

	public void activityCreated(Activity activity) {
		if (!listModel.contains(activity)) {
			listModel.addElement(activity);
		}
	}

	public <T> void paramAdded(Activity activity, Parameter<T> p) {
		/**
		 * Nothing to do here..
		 */
	}

	/**
	 * Recalculate the time chart based on when parameters change. This need
	 * only be done when parameters affecting the time of the mission are
	 * changed, but we are not so fussy about that.
	 * 
	 * Await all pending activity updates before activating this, but do not
	 * block execution doing so. This so that we are ensured that the new time
	 * chart reflects a consistent state among the activities.
	 * 
	 */

	public <T> void paramChanged(final Activity activity, Parameter<T> p) {

		if (getEvaluationActivities().contains(activity) && updateThread == null) {
			updateThread = new Thread() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Thread#run()
				 */
				@Override
				public void run() {
					activity.acquirePendingUpdatesLock();
					initSimulation();
					updateTimeChart();
					updateThread = null;
					activity.releasePendingUpdatesLock();
				}

			};
			updateThread.start();
		}

	}

	public void activityRemoved(Activity a) {
		listModel.removeElement(a);
	}

	/**
	 * 
	 * Retrieves the current time as set by the slider
	 * 
	 * @see critiquer.gui.TimeLinePlot#getCurrentTime()
	 */
	public Calendar getCurrentTime() {
		return plot.getCurrentTime();
	}

	/**
	 * Adjust the time according to the parameter, which is converted into a
	 * date by the plot
	 * 
	 * @see critiquer.gui.TimeLinePlot#setCurrentTime(java.lang.Double)
	 */
	public void setCurrentTime(Double currentTime) {
		plot.setCurrentTime(currentTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see critiquer.gui.TimeLinePlot#getDataArea()
	 */
	public Rectangle2D getTimeChartArea() {
		return plot.getDataArea();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see critiquer.gui.TimeLinePlot#locationToDate(double)
	 */
	public Calendar locationToDate(double x) {
		return plot.locationToDate(x);
	}

	/**
	 * @return Returns the chartPane.
	 */
	public ChartPanel getChartPane() {
		return chartPane;
	}

	/**
	 * @return Returns the timePanel.
	 */
	public JPanel getTimePanel() {
		return timePanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see critiquer.gui.TimeLineRenderer#getActivityAreas()
	 */
	public HashMap<Activity, Rectangle2D> getActivityAreas() {
		return activityAreas;
	}

	public void updateActivityArea(final Comparable key, Rectangle2D bar) {
		Activity activity = ActivityUtils.getActivityByName(key);
		if (!activityAreas.containsKey(activity)) {
			activityAreas.put(activity, bar);
		} else {
			activityAreas.get(activity).add(bar);
		}

	}

	public Rectangle2D getActivityArea(Activity activity) {
		return activityAreas.get(activity);
	}

	/**
	 * 
	 * Put this one after all other update listeners so all changes have taken
	 * place when changing the time lines
	 * 
	 * @see java.lang.Comparable#compareTo(T)
	 */
	public int compareTo(ActivityUpdateListener o) {
		return 1;
	}

}
