/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.misc;

import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Transparency;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;

public class Utils {

    /**
     * Returns a point based on (x, y) but constrained to be within the bounds
     * of the given rectangle.  Borrowed from ChartPanel in JFreeChart.
     * 
     * @param x  the x-coordinate.
     * @param y  the y-coordinate.
     * @param area  the rectangle (<code>null</code> not permitted).
     * 
     * @return A point within the rectangle.
     */
    public Point getPointInRectangle(int x, int y, Rectangle2D area) {
        x = (int) Math.max(
            Math.ceil(area.getMinX()), Math.min(x, Math.floor(area.getMaxX()))
        );   
        y = (int) Math.max(
            Math.ceil(area.getMinY()), Math.min(y, Math.floor(area.getMaxY()))
        );
        return new Point(x, y);
    }
    
	public static Point getCenter(Shape shape){
		Point p = new Point();
		Rectangle rect = shape.getBounds();
		p.x=(int) rect.getCenterX();
		p.y=(int) rect.getCenterY();
		return p;
	}

	public static List<Class> getAllInterfaces(Class constraintClass) {
		List<Class> l = new ArrayList<Class>();
		Class c = constraintClass;
		while (c != null) {
			List<Class> interfaceList = Arrays.asList(c.getInterfaces());
			l.addAll(interfaceList);
			c=c.getSuperclass();
		}
		return l;
	}
	
	public static <T> Vector<T> toVector(Collection<T> c) {
		Vector<T> vector = new Vector<T>();
		for (T element : c) {
			vector.add(element);
		}
		return vector;
	}

	public static <T> Vector<T> toVector(T[] c) {
		Vector<T> vector = new Vector<T>();
		for (int i = 0; i < c.length; i++) {
			T element = c[i];			
			vector.add(element);
		}
		
		return vector;
	}

	
	public static <T> T find(Collection<T> c,Predicate<T> comp) {
		for (T elt : c) {
			if (comp.check(elt)) {
				return elt;
			}
		}
		return null;
	}
	
	public static <T> ArrayList<T> findAll(Collection<T> c,Predicate<T> comp) {
		ArrayList<T> list = new ArrayList<T>();
		for (T elt : c) {
			if (comp.check(elt)) {
				list.add(elt);
			}
		}
		return list;
	}
	
	public static <In,Out> List<Out> map(UnaryFunction<In, Out> fn, Collection<In> l) {
		List<Out> newList = new ArrayList<Out>();
		for (In element : l) {
			newList.add(fn.perform(element));
		}
		return newList;
	}
	
	public static void logException(Logger logger, Throwable e) {
		logger.error(e);
		StringWriter str = new StringWriter();
		e.printStackTrace(new PrintWriter(str));
		logger.error(str);
	}
	
	/**	 This method returns true if the specified image has transparent pixels
	 * */
    public static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
         PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }
	
    /**
     *  This method returns a buffered image with the contents of an image 
     * 
     * */
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e661 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if (hasAlpha) {
                transparency = Transparency.BITMASK;
            }
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            }
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }
	
}
