/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.resources;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.Activity.Properties;
import critiquer.activities.parameters.AgentChangedListener;
import critiquer.activities.parameters.AgentParameter;
import critiquer.gui.AgentLabel;
import critiquer.gui.ComPlan;
import critiquer.gui.StartupProgressWindow;
import critiquer.misc.Predicate;
import critiquer.misc.ReferenceHolder;
import critiquer.misc.Utils;
import critiquer.scenarios.standard.AgentParameterName;
import critiquer.simulation.AgentSimulationState;
import critiquer.simulation.DefaultAgentSimulationState;
import critiquer.simulation.SimulationEngine;



/**
 * <p>
 * Agents are a part of and perform activities. They are added and removed via
 * the Resource View.
 * </p>
 * <p>
 * In order for an agent to be considered suitable for an activity is should put
 * the appropriate entries into the <code>fitnessMap</code> class field.
 * </p>
 * <p>
 * Also, in order to calculate how long it takes for an agent to perform an
 * activity, the <code>missionContributionCalculators</code> map is used to
 * look up procedures that calculate the time it takes an agent to perform some
 * unit of work on an activity. Moreover, performing an operation may have
 * side-effects which are modelled by having a <code>HashMap</code> as an
 * argument to a <code>AgentContributionCalculator</code> that may be modified
 * by the agent while performing some action.
 * </p>
 * <p>
 * It is not intended that a single "use" of an agent solves a task, since it
 * may be impossible for an agent to complete a task on its own or it may be
 * more appropriate if the agent performs only a small part of it. As it is
 * right now, the agent reports some number on how long it takes to perform
 * (possibly only a part of) the mission and the activity is responsible for
 * accumulating this information into some overall time for the mission as a
 * whole.
 * </p>
 */

public abstract class Agent implements Serializable {

	/**
     * 
     */
	private static final long									serialVersionUID			   = 1L;

	/**
     * Placeholder agent to be used when the side-effects of using an agent is
     * desirable and the most convenient way is to have a placeholder agent
     */
	public static Agent										  dummyAgent					 = new DefaultAgent("none");
	static {
		dummyAgent.setTimeLineColor(Color.GREEN);
	}

	private Color												timeLineColor				  = Color.BLACK;

	protected static HashMap<Class, HashMap<Class, Suitability>> fitnessMap					 = new HashMap<Class, HashMap<Class, Suitability>>();

	public HashMap<Class, AgentContributionCalculator>		   missionContributionCalculators = new HashMap<Class, AgentContributionCalculator>();

	private Agent												parent						 = null;

	private ArrayList<Agent>									 children					   = new ArrayList<Agent>();

	public String												name;

	private ImageIcon											picture;

	private transient JPanel									 panel;

	private ArrayList<AgentChangedListener>					  agentChangedListeners		  = new ArrayList<AgentChangedListener>();

	/**
     * Alla <code>agentParameters</code> used by the agent must be added in
     * the constructor of the agent. Here, an agentParameter means a parameter
     * that determines how the agent works, not an activity.
     */
	private HashMap<AgentParameterName, ParameterInAgent<?>>	 agentParameters				= new HashMap<AgentParameterName, ParameterInAgent<?>>();

	protected transient AgentSimulationState					 agentSimulationState		   = new DefaultAgentSimulationState(this);

	public <T> void addParameter(ParameterInAgent<T> param) {
		agentParameters.put(param.getName(), param);

	}

	@SuppressWarnings("unchecked")
	public <C> ParameterInAgent<C> getParamByClassAndName(Class<C> c, AgentParameterName name) {
		return (ParameterInAgent<C>) agentParameters.get(name);
	}

	/**
     * Returns the name of this agent
     * 
     * @see java.lang.Object#toString()
     */
	@Override
	public String toString() {
		return name;
	}

	/**
     * Take all user-visible parameters of this agent and add them to the panel
     * 
     */
	protected void initPanel() {
		panel = new JPanel();
		panel.setLayout(new GridLayout(0, 1));
		for (ParameterInAgent agentParam : agentParameters.values()) {
			if (agentParam.getType() == ParameterInAgent.Type.EXTERNAL) {
				panel.add(agentParam.getComponent());

			}
		}
		StartupProgressWindow splashScreen = ComPlan.getPlanningTool().getSplashScreen();
		if (splashScreen.isDisplayable()) {
			splashScreen.actionDone();

		}
	}

	public ArrayList<Activity> getActivitiesForAgent() {
		return Utils.findAll(ReferenceHolder.allActivities, new Predicate<Activity>() {

			public boolean check(Activity activity) {
				return ActivityUtils.agentInActivity(Agent.this, activity);
			}

		});
	}

	public void initAgentSimulationState(SimulationEngine engine) {
		agentSimulationState.initSimulation(engine);
	}

	public AgentSimulationState getAgentSimulationState() {
		return agentSimulationState;
	}

	/**
     * 
     * Compare agents with respect to their names
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
	@Override
	public boolean equals(Object o) {
		boolean ret = false;
		if (o instanceof Agent) {
			Agent agent = (Agent) o;
			ret = toString().equals(agent.toString());
		}
		return ret;
	}

	/**
     * 
     * Use the name for hash value
     * 
     * @see java.lang.Object#hashCode()
     */
	@Override
	public int hashCode() {
		return name.hashCode();
	}

	/**
     * @param name
     */
	public Agent(String name) {
		this.name = name;
	}

	/**
     * @param parent
     *            is supposed to be an organizational unit that contains this
     *            agent, null if not applicable
     * @param name
     */
	public Agent(Agent parent, String name) {
		StartupProgressWindow splashScreen = ComPlan.getPlanningTool().getSplashScreen();
		if (splashScreen.isDisplayable()) {
			splashScreen.newAction("Creating " + name);
		}
		this.name = name;
		this.parent = parent;
		this.parent.addChild(this);
	}

	public void addChild(Agent child) {
		if (!children.contains(child)) {
			children.add(child);
		}

	}

	public AgentLabel getLabel(AgentParameter param) {
		return new AgentLabel(this, param);
	}

	/**
     * Should agents or constraints be responsible for knowing which agent is
     * suitable for which task? Right now we require agents to provide this
     * information. It could easily be lifted to a GUI instead, if that's really
     * preferable.
     */
	public static Suitability getSuitability(Agent agent, Activity activity) {
		Suitability fitness = Suitability.FIT;
		if (fitnessMap.containsKey(agent.getClass())) {
			if (fitnessMap.get(agent.getClass()).containsKey(activity.getClass())) {
				fitness = fitnessMap.get(agent.getClass()).get(activity.getClass());
			}
		}
		return fitness;
	}

	/**
     * 
     * Perform <code>activity</code> of class <code>activityClass</code> by
     * dispatching to the appropriate procedure. When we "perform" a mission, we
     * may modify some attributes that were set in the activityAttributes map.
     * For now, there is no good "standard" way of finding out which attributes
     * are present and how they should be modified, but the idea is that the
     * agents and activities in a scenario are developed in tandem with good
     * knowledge of each other. In the standard scenario, the
     * <code>StandardAgentParameterFactory</code> knows of the attributes in
     * activities in the standard scenario package.
     * 
     * These attributes reflect properties of the activity, not the agent.
     * 
     */

	@SuppressWarnings("unchecked")
	public <T extends Activity> long perform(Class<T> activityClass,
											 T activity,
											 HashMap<Properties, Integer> activityAttributes) throws UnsupportedOperationException {
		if (missionContributionCalculators.containsKey(activityClass)) {
			return missionContributionCalculators.get(activityClass).perform(activity, activityAttributes);
		} else {
			throw new UnsupportedOperationException("unsupported operation perform on " + this.getClass()
																							  .getSimpleName()
													+ " with parameter of type "
													+ activityClass.getSimpleName());
		}
	}

	/**
     * @return Returns the timeLineColor.
     */
	public Color getTimeLineColor() {
		return timeLineColor;
	}

	/**
     * @param timeLineColor
     *            The timeLineColor to set.
     */
	public void setTimeLineColor(Color timeLineColor) {
		this.timeLineColor = timeLineColor;
	}

	/**
     * @return Returns the children.
     */
	public ArrayList<Agent> getChildren() {
		return children;
	}

	/**
     * @return Returns the parent.
     */
	public Agent getParent() {
		return parent;
	}

	/**
     * @return Returns the panel.
     */
	public JPanel getPanel() {
		if (panel == null) {
			initPanel();
		}
		return panel;
	}

	public void addAgentChangedListener(AgentParameter parameter) {
		agentChangedListeners.add(parameter);
	}

	public void removeAgentChangedListener(AgentParameter parameter) {
		agentChangedListeners.remove(parameter);
	}

	/**
     * Notify AgentParameters containing this agent that it has changed some
     * property
     * 
     */
	public void notifyAgentChangedListeners() {
		for (AgentChangedListener listener : agentChangedListeners) {
			listener.agentChanged(this);
		}
	}

	/**
     * @return Returns the picture.
     */
	public ImageIcon getPicture() {
		return picture;
	}

	/**
     * @param picture
     *            The picture to set.
     */
	public void setPicture(ImageIcon picture) {
		this.picture = picture;
	}

	/**
     * Create an image baased on a filename pointing to the resource directory.
     * 
     * @param fileName
     */
	public void setPictureFromName(String fileName) {
		String fullFileName = ReferenceHolder.resourceDir + System.getProperty("file.separator") + fileName;
		setPicture(new ImageIcon(fullFileName));
	}

	/**
	 * See mapsymb.com
	 * 
	 * @return a symbol from the MapSym-NU-Land-APP6-A Unicode font table
	 * 
	 */
	public abstract Character getSymbol();
	
	/**
	 * 
	 * @return the unicode font used to render the symbol String
	 */
	public Font getSymbolFont() {
		return new Font("MapSymNULand1",Font.PLAIN,48);
	}
}
