/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.resources;

import java.io.Serializable;
import java.util.HashMap;

import critiquer.activities.Activity;
import critiquer.activities.Activity.Properties;


/**
 * <p>
 * Interface for functions defined by agents that calculate how long a mission
 * takes to complete. What do we mean by this?
 * </p>
 * 
 * <p>
 * For the missions that agents can participate in, there should be some notion
 * of what capacity they have with respect to handling the mission. This does
 * not, however, have to imply that each agent can solve a mission on their own.
 * </p>
 * 
 * <p>
 * There are only a few cases here as I see it, all under the condition that the
 * agent can actually contribute to an activity:
 * </p>
 * <ol>
 * <li>The agent may be able to do a given part of the job that in turn takes a
 * given amount of time (transporting itself to a rendez-vous point during
 * dispatch).</li>
 * 
 * <li>The agent may do a given amount of work per time unit and, if enough
 * time is given, solve the mission on its own.</li>
 * 
 * </ol>
 * 
 * <p>
 * Both these contributions can be accomodated using a single method that
 * <code>performs</code> some task and, if appropriate, modifies properties of
 * the activity to reflect the work done. The return value is then the time it
 * took the unit to perform its task. The activity may then calculate how long
 * it will take to complete based on the agents participating by using either
 * the side effects they cause or
 * </p>
 * 
 * <p>
 * "Given" amounts are calculated or specified by the agent. The human planner
 * should specify which agents to use and the job here is only to perform the
 * specified activity.
 * </p>
 * 
 */
public interface AgentContributionCalculator<T extends Activity> extends Serializable {

	/**
	 * 
	 * Perform some work by modifying appropriate attributes in the activity to
	 * indicate what is being done.
	 * 
	 * Return the time it took to perform the work.
	 * 
	 * If we consider the case where this operation would mean that we transport
	 * people from location A to location B, this would mean that each
	 * subsequent call to <code>perform</code> would transport as many people as would fit
	 * into the vehicle and drive to location B. The next call would then imply
	 * going back to location A, fetch another batch of people there and go
	 * back to B.
	 * 
	 */
	public long perform(T activity, HashMap<Properties,Integer> attributes);

}
