/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import critiquer.activities.Activity;
import critiquer.activities.ActivityConsistencyCheck;
import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.LocationParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.constraints.StandardConstraint;
import critiquer.constraints.TextCritic;
import critiquer.gui.TaskView;
import critiquer.gui.View;
import critiquer.misc.Predicate;
import critiquer.misc.Utils;



/**
 * 
 * Verify that there is a destination in the activity that should come before
 * and a static location in the after activity, or a static location in the
 * beforeActivity and a starting location in the afterActivity
 * 
 * @author olale
 * 
 */
public class CorrectOrdering extends StandardConstraint implements ActivityConsistencyCheck,TextCritic {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Activity inconsistentBeforeActivity;
	private Activity inconsistentAfterActivity;
	private String information;

	/**
     * 
     */
	public CorrectOrdering() {
		super();
		Activity.addStaticConsistencyChecker(this);
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.ActivityConsistencyCheck#paramValueCheck(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter, T)
     */
	public <T> boolean paramValueCheck(Activity a, Parameter<T> p, T newValue) {
		// TODO Auto-generated method stub
		return true;
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.ActivityConsistencyCheck#childOfCheck(critiquer.activities.Activity,
     *      critiquer.activities.Activity)
     */
	public boolean childOfCheck(Activity parentActivity, Activity Child) {
		// TODO Auto-generated method stub
		return true;
	}

	/**
     * 
     * Checks the ordering consistency
     * 
     * @see critiquer.activities.ActivityConsistencyCheck#orderingCheck(critiquer.activities.Activity,
     *      critiquer.activities.Activity)
     */
	public boolean orderingCheck(Activity beforeActivity, Activity afterActivity) {
		boolean case1 = checkCase1(beforeActivity, afterActivity);
		boolean case2 = checkCase2(beforeActivity, afterActivity);

		boolean ok = case1 || case2;
		if (!ok) {
			this.inconsistentBeforeActivity=beforeActivity;
			this.inconsistentAfterActivity=afterActivity;
			information = "There must be either a an end location or a stationary in "+beforeActivity+
			" and a stationary location or a start location in "+afterActivity;
		} else {
			this.inconsistentBeforeActivity=null;
			this.inconsistentAfterActivity=null;
		}
		setConsistent(ok);
		return ok;
	}

	private Predicate<ActivityParameter> createLocationCheck(final LocationParameter.Type type) {
		return new Predicate<ActivityParameter>() {

			public boolean check(ActivityParameter obj) {
				boolean check = false;
				if (obj instanceof LocationParameter) {
					LocationParameter locParam = (LocationParameter) obj;
					if (locParam.getType() == type) {
						check = true;
					}
				}
				return check;
			}

		};
	}
	
	/**
     * Case 1: There is a destination in the beforeActivity an a stationary
     * location in the afterActivity
     */
	private boolean checkCase1(Activity beforeActivity, Activity afterActivity) {
		return Utils.find(beforeActivity.getParams(), createLocationCheck(LocationParameter.Type.DEST_LOC)) != null && 
		Utils.find(afterActivity.getParams(), createLocationCheck(LocationParameter.Type.STATIC_LOC)) != null;
	}

	/**
     * Case 2: There is a stationary parameter in beforeActivity and a starting
     * location in afterActivity
     */
	private boolean checkCase2(Activity beforeActivity, Activity afterActivity) {
		return Utils.find(beforeActivity.getParams(), createLocationCheck(LocationParameter.Type.STATIC_LOC)) != null && 
		Utils.find(afterActivity.getParams(), createLocationCheck(LocationParameter.Type.ORIG_LOC)) != null;
	}

	/* (non-Javadoc)
	 * @see critiquer.activities.AbstractParamChangedListener#paramChanged(critiquer.activities.Activity, critiquer.activities.parameters.Parameter)
	 */
	@Override
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		// TODO Auto-generated method stub
		
	}

	/**
	 *
	 * Describe this constraint
	 * 
	 * @see critiquer.constraints.SingletonConstraint#getDescription()
	 */
	public String getDescription() {
		return "Ensure that there is a sensible ordering of activities with respect to locations";
	}

	
	
	/**
	 * 
	 * Make this view applicable to the task view
	 * 
	 * @see critiquer.constraints.StandardConstraint#initApplicableViews()
	 */
	@Override
	protected void initApplicableViews() {
		applicableViews.add(TaskView.class);
	}

	/* (non-Javadoc)
	 * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View, java.awt.Graphics2D)
	 */
	public void viewUpdated(View v, Graphics2D g2) {
		if (inconsistentBeforeActivity!=null && inconsistentAfterActivity!=null) {
			Color oldColor = g2.getColor();
			g2.setColor(getColor());
			Rectangle area1 = v.getActivityArea(inconsistentBeforeActivity).getBounds();
			Rectangle area2 = v.getActivityArea(inconsistentAfterActivity).getBounds();
			g2.drawLine((int)area1.getMaxX(), (int)area1.getCenterY(), (int)area2.getX(), (int)area2.getCenterY());
			g2.setColor(oldColor);
		}
	}

	/* (non-Javadoc)
	 * @see critiquer.constraints.TextCritic#getText()
	 */
	public String getText() {
		return information;
	}

}
