/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.util.HashSet;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.TransportationActivity;
import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.Location;
import critiquer.activities.parameters.LocationParameter;
import critiquer.constraints.StructureCritic;
import critiquer.misc.Predicate;
import critiquer.misc.Utils;
import critiquer.resources.Agent;



public class DispatchActivity extends Activity implements TransportationActivity {

	/**
     * 
     */
	private static final String DESTINATION	   = "destination";

	/**
     * 
     */
	private static final String STARTING_LOCATION = "starting location";

	/**
     * 
     */
	private static final long   serialVersionUID  = 1L;
	private static int		  numActivities	 = 0;

	private static StructureCritic agentAvailableCritic = StandardActivityFactory.createAgentAvailableCritic(DispatchActivity.class); 
	private static StructureCritic locationsDefinedCritic  = StandardActivityFactory.createTransportLocationsCritic(DispatchActivity.class);

	/**
     * @param parent
     */
	public DispatchActivity(Activity parent) {
		super("Dispatch activity", parent);
		
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.Activity#addDefaultParams()
     */
	@Override
	public void addDefaultParams() {
		super.addDefaultParams();

		LocationParameter startLocationParameter = new LocationParameter(LocationParameter.Type.ORIG_LOC,
																		 STARTING_LOCATION,
																		 this);
		addParam(startLocationParameter);
		ActivityUtils.createMissionTimeUpdater(startLocationParameter);
		LocationParameter endLocationParameter = new LocationParameter(LocationParameter.Type.DEST_LOC,
																	   DESTINATION,
																	   this);
		addParam(endLocationParameter);
		ActivityUtils.createMissionTimeUpdater(endLocationParameter);
	}
	
	/**
	 * 
     * Return the maximum time taken by any agent to complete this activity
     * 
     * @see critiquer.activities.Activity#calculateTimeToComplete(java.util.HashSet)
     */
	@Override
	public long simulate() {

		long timeToComplete = 0;
		// time in minutes
		timeToComplete = 0;
		HashSet<Agent> agentsInActivity = ActivityUtils.getAgentsInActivity(this);
		for (Agent agent : agentsInActivity) {
				timeToComplete = Math.max(timeToComplete, agent.perform(DispatchActivity.class, this, null));
		}
		return timeToComplete;

	}

	/**
     * 
     * Add location parameter dependency
     * 
     * @see critiquer.activities.Activity#order(critiquer.activities.Activity)
     */
	@Override
	public void order(Activity laterActivity) {
		super.order(laterActivity);
		LocationParameter stationaryLocationParam = (LocationParameter) Utils.find(laterActivity.getParams(),
				new Predicate<ActivityParameter>() {

					public boolean check(ActivityParameter param) {
						boolean check = false;
						if (param instanceof LocationParameter) {
							LocationParameter locParam = (LocationParameter) param;
							if (locParam.getType() == LocationParameter.Type.STATIC_LOC) {
								check = true;
							}
						}
						return check;
					}

				});
		LocationParameter destinationParam = ActivityUtils.getParamByClassAndName(this,
				LocationParameter.class,
				DESTINATION);
		/**
         * If the location parameter of this activity has been set, use that
         * value. Otherwise, use the value of the stationary location parameter
         * of the activity that follows.
         */
		if (destinationParam.hasValue()) {
			stationaryLocationParam.setValue(destinationParam.getValue());
		} else {
			destinationParam.setValue(stationaryLocationParam.getValue());
		}

	}

	/**
     * 
     * When removing ordering constraints, create a new stationary location for
     * the activity that is decuopled from this one.
     * 
     * @see critiquer.activities.Activity#removeOrder(critiquer.activities.Activity)
     */
	@Override
	public void removeOrder(Activity laterActivity) {
		super.removeOrder(laterActivity);
		LocationParameter stationaryLocationParam = (LocationParameter) Utils.find(laterActivity.getParams(),
				new Predicate<ActivityParameter>() {

					public boolean check(ActivityParameter param) {
						boolean check = false;
						if (param instanceof LocationParameter) {
							LocationParameter locParam = (LocationParameter) param;
							if (locParam.getType() == LocationParameter.Type.STATIC_LOC) {
								check = true;
							}
						}
						return check;
					}

				});
		stationaryLocationParam.setValue(new Location(stationaryLocationParam));
	}

	@Override
	public int getNumActivities() {
		return DispatchActivity.numActivities++;
	}

	/**
     * Return value of the starting location parameter
     * 
     * @see critiquer.activities.TransportationActivity#getStartLocation()
     */
	public Location getStartLocation() {
		return ActivityUtils.getParamByClassAndName(this, LocationParameter.class, STARTING_LOCATION).getValue();
	}

	/**
     * 
     * Return the value of the destination location parameter
     * 
     * @see critiquer.activities.TransportationActivity#getEndLocation()
     */
	public Location getEndLocation() {
		return ActivityUtils.getParamByClassAndName(this, LocationParameter.class, DESTINATION).getValue();
	}

	/**
	 * 
	 * Check that locations and agents are defined
	 * 
	 * @see critiquer.activities.Activity#canBeSimulated()
	 */
	@Override
	public boolean canBeSimulated() {
		return agentAvailableCritic.isConsistent(this) && locationsDefinedCritic.isConsistent(this);
	}

}
