/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.LocationParameter;
import critiquer.activities.parameters.NumberSelectionParameter;
import critiquer.constraints.StructureCritic;
import critiquer.misc.Predicate;
import critiquer.misc.Utils;
import critiquer.resources.Agent;



public class FirstAidActivity extends Activity {

	/**
	 * 
	 */
	private static final String NUMBER_INJURED = "number injured";

	/**
     * 
     */
	private static final String ACCIDENT_LOCATION = "Accident location";

	public enum Properties implements Activity.Properties {
		PATIENTS("patients");
		
		private String name;
		public String toString() {
			return name;
		}
		
		Properties(String name) {
			this.name=name;
		}
	}
	
	/**
     * 
     */
	private static final long   serialVersionUID  = 1L;

	private static int		  numActivities;

	private static StructureCritic noInjuredCritic = StandardActivityFactory.createNoInjuredCritic(FirstAidActivity.class);

	private static StructureCritic agentCritic = StandardActivityFactory.createAgentAvailableCritic(FirstAidActivity.class);

	public FirstAidActivity(Activity parent) {
		super("First aid activity", parent);
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.Activity#addDefaultParams()
     */
	@Override
	public void addDefaultParams() {
		super.addDefaultParams();
		Vector<Integer> numbers = new Vector<Integer>();
		for (int i = 0; i < 10; i++) {
			numbers.add(i);
		}
		final NumberSelectionParameter injuredSelector = new NumberSelectionParameter(this,
																					  NUMBER_INJURED,
																					  numbers,
																					  0);
		addPrivateUpdateListener(ActivityUtils.createMissionTimeUpdater(injuredSelector));
		addParam(injuredSelector);
		addParam(new LocationParameter(LocationParameter.Type.STATIC_LOC, ACCIDENT_LOCATION, this));
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.activities.Activity#calculateTimeToComplete(java.util.HashSet)
     */
	@Override
	public long simulate() {

		// time in minutes
		long totalTime = 0;
		long timeForRound = 0;
		HashMap<Activity.Properties, Integer> attributes = new HashMap<Activity.Properties, Integer>();
		
		attributes.put(Properties.PATIENTS, getInjured());			
		HashSet<Agent> agents = ActivityUtils.getAgentsInActivity(getRootActivity());
		while (attributes.get(Properties.PATIENTS) > 0) {
			timeForRound = 0;
			for (Agent agent : agents) {
				timeForRound = Math.max(timeForRound, agent.perform(FirstAidActivity.class, this, attributes));
				if (attributes.get(Properties.PATIENTS) > 0) {
					break;
				}
			}
			totalTime += timeForRound;
		}

		return totalTime;
	}

	/**
     * @return Returns the injured.
     */
	public int getInjured() {
		NumberSelectionParameter numberSelector = ActivityUtils.getParamByClassAndName(this,
				NumberSelectionParameter.class,
				NUMBER_INJURED);
		int returnValue = 0;
		returnValue = numberSelector.getValue();
		return returnValue;
	}

	@Override
	public int getNumActivities() {
		return FirstAidActivity.numActivities++;
	}

	/**
     * 
     * Add location parameter dependency
     * 
     * @see critiquer.activities.Activity#order(critiquer.activities.Activity)
     */
	@Override
	public void order(Activity laterActivity) {
		super.order(laterActivity);
		LocationParameter startLocationParam = (LocationParameter) Utils.find(laterActivity.getParams(),
				new Predicate<ActivityParameter>() {

					public boolean check(ActivityParameter param) {
						boolean check = false;
						if (param instanceof LocationParameter) {
							LocationParameter locParam = (LocationParameter) param;
							if (locParam.getType() == LocationParameter.Type.ORIG_LOC) {
								check = true;
							}
						}
						return check;
					}

				});
		// Just a precaution, should maybe be a junit test
		assert (startLocationParam != null);
		LocationParameter accidentLocation = ActivityUtils.getParamByClassAndName(this,
				LocationParameter.class,
				ACCIDENT_LOCATION);
		if (startLocationParam.hasValue()) {			
			accidentLocation.setValue(startLocationParam.getValue());
		} else {
			startLocationParam.setValue(accidentLocation.getValue());
		}

	}

	/* (non-Javadoc)
	 * @see critiquer.activities.Activity#canBeSimulated()
	 */
	@Override
	public boolean canBeSimulated() {
		return noInjuredCritic.isConsistent(this) && agentCritic.isConsistent(this);
	}


}
