/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.IncompleteActivityException;
import critiquer.activities.TransportationActivity;
import critiquer.activities.Activity.Properties;
import critiquer.activities.parameters.Location;
import critiquer.activities.parameters.LocationParameter;
import critiquer.activities.parameters.NumberSelectionParameter;
import critiquer.constraints.StructureCritic;
import critiquer.misc.Pair;
import critiquer.resources.Agent;



public class RecoverActivity extends Activity implements TransportationActivity {

	/**
	 * 
	 */
	private static final String NUMBER_INJURED = "number injured";
	/**
	 * 
	 */
	private static final String PATIENTS = "patients";
	
	public enum Properties implements Activity.Properties {
		
		PATIENTS("patients");
		
		
		private String name = "";
		public String toString() {
			return name;
		}
		Properties(String name) {
			this.name=name;
		}
	}
	/**
	 * 
	 */
	private static final String STARTING_LOCATION = "starting location";
	/**
	 * 
	 */
	private static final String MEDICAL_FACILITY_LOCATION = "medical facility location";
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	private static int numActivities;
	private static StructureCritic noInjuredCritic = StandardActivityFactory.createNoInjuredCritic(RecoverActivity.class);
	private static StructureCritic agentCritic = StandardActivityFactory.createAgentAvailableCritic(RecoverActivity.class);
	private static StructureCritic locationsCritic = StandardActivityFactory.createTransportLocationsCritic(RecoverActivity.class);

	
	public RecoverActivity(Activity parent) {
		super("Recover",parent);
	}

	/* (non-Javadoc)
	 * @see critiquer.activities.Activity#addDefaultParams()
	 */
	@Override
	public void addDefaultParams() {
		super.addDefaultParams();
		Vector<Integer> numbers = new Vector<Integer>();
		for (int i = 0; i < 10; i++) {
			numbers.add(i);
		}
		NumberSelectionParameter injuredSelector = new NumberSelectionParameter(this,NUMBER_INJURED,numbers,2);
		addParam(injuredSelector);
		ActivityUtils.createMissionTimeUpdater(injuredSelector);
		
		LocationParameter startLocationParameter = new LocationParameter(LocationParameter.Type.ORIG_LOC,STARTING_LOCATION,this);
		addParam(startLocationParameter);
		ActivityUtils.createMissionTimeUpdater(startLocationParameter);
		
		LocationParameter endLocationParameter = new LocationParameter(LocationParameter.Type.DEST_LOC,MEDICAL_FACILITY_LOCATION,this);
		addParam(endLocationParameter);
		ActivityUtils.createMissionTimeUpdater(endLocationParameter);
	
	}

	@Override
	public long simulate() throws IncompleteActivityException {
		// time in minutes
		long totalTime=0;
		long timeForRound=0;
		HashMap<Activity.Properties, Integer> attributes = new HashMap<Activity.Properties,Integer>();
		attributes.put(Properties.PATIENTS, getInjured());
		
		HashSet<Agent> agents = ActivityUtils.getAgentsInActivity(this);
		while (attributes.get(Properties.PATIENTS) > 0) {
			// TODO: Change this to reflect that units need to go back to
			// retrieve patients once more
			timeForRound = 0;
			
			for (Agent agent : agents) {
				timeForRound = Math.max(timeForRound, agent.perform(RecoverActivity.class, this, attributes));
				if (attributes.get(Properties.PATIENTS) <= 0) {
					break;
				}
			}
			totalTime += timeForRound;
			
		}
		
		
		return totalTime;
		
	}

	/**
	 * @return Returns the injured.
	 */
	public int getInjured() {
		NumberSelectionParameter numberSelector = ActivityUtils.getParamByClassAndName(this, NumberSelectionParameter.class, NUMBER_INJURED);
		int returnValue = 0;
		returnValue = numberSelector.getValue();
		return returnValue;
	}

	@Override
	public int getNumActivities() {
		return RecoverActivity.numActivities++;
	}

	/* (non-Javadoc)
	 * @see critiquer.activities.TransportationActivity#getStart()
	 */
	public Location getStartLocation() {
		return ActivityUtils.getParamByClassAndName(this, LocationParameter.class, STARTING_LOCATION).getValue();
	}

	/* (non-Javadoc)
	 * @see critiquer.activities.TransportationActivity#getEnd()
	 */
	public Location getEndLocation() {
		return ActivityUtils.getParamByClassAndName(this, LocationParameter.class, MEDICAL_FACILITY_LOCATION).getValue();
	}

	

	/* (non-Javadoc)
	 * @see critiquer.activities.Activity#canBeSimulated()
	 */
	@Override
	public boolean canBeSimulated() {
		return noInjuredCritic.isConsistent(this) && agentCritic.isConsistent(this) && locationsCritic.isConsistent(this);
	}

}
