/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityFactory;
import critiquer.activities.ActivityUtils;
import critiquer.activities.ToplevelActivity;
import critiquer.activities.TransportationActivity;
import critiquer.activities.parameters.Location;
import critiquer.constraints.ActivityStructureCritic;
import critiquer.constraints.ConstraintFactory;
import critiquer.constraints.StructureCritic;
import critiquer.misc.Pair;
import critiquer.resources.Agent;



public class StandardActivityFactory implements ActivityFactory {

	private transient static Logger logger = Logger.getLogger(StandardActivityFactory.class);

	private interface ActivityCreator {

		Activity createActivity();

	}

	private HashMap<String, ActivityCreator> activityTemplates;

	public ArrayList<Class<? extends Activity>> getActivityClasses() {
		ArrayList<Class<? extends Activity>> activityClasses = new ArrayList<Class<? extends Activity>>();
		activityClasses.add(ToplevelActivity.class);
		activityClasses.add(DefaultActivity.class);
		activityClasses.add(DispatchActivity.class);
		activityClasses.add(FirstAidActivity.class);
		activityClasses.add(RecoverActivity.class);
		activityClasses.add(RoadBlockActivity.class);
		return activityClasses;
	}

	public StandardActivityFactory() {
		activityTemplates = new HashMap<String, ActivityCreator>();

		activityTemplates.put("Rescue", new ActivityCreator() {

			public Activity createActivity() {
				return createRescueActivity();
			}

		});
		activityTemplates.put("Block roads", new ActivityCreator() {

			public Activity createActivity() {
				return createRoadBlockActivity();
			}

		});
	}

	public Activity selectActivityType(JFrame parentFrame) {
		String message = "Select activity type";
		String selectedKey = (String) JOptionPane.showInputDialog(parentFrame,
				message,
				message,
				JOptionPane.PLAIN_MESSAGE,
				null,
				activityTemplates.keySet().toArray(),
				null);
		if (selectedKey != null) {
			return activityTemplates.get(selectedKey).createActivity();
		} else {
			return null;
		}
	}

	private Activity createRoadBlockActivity() {
		Activity roadBlockActivity = new DefaultActivity(null);
		new DispatchActivity(roadBlockActivity);
		new RoadBlockActivity(roadBlockActivity);
		return roadBlockActivity;
	}

	private Activity createRescueActivity() {
		Activity rescueActivity = new DefaultActivity(null);
		new DispatchActivity(rescueActivity);
		new FirstAidActivity(rescueActivity);
		new RecoverActivity(rescueActivity);
		return rescueActivity;
	}

	/**
     * Create an activity using the constructor which takes only a parent
     * activity and pass <code>null</code> as reference to the parent.
     * 
     */
	@SuppressWarnings("unchecked")
	public <T extends Activity> T createActivity(Class<T> activityClass) {
		T activity = null;
		if (Activity.class.isAssignableFrom(activityClass)) {
			try {
				Constructor constructor = activityClass.getConstructor(new Class[] { Activity.class });
				activity = (T) constructor.newInstance(new Object[] { null });
				ConstraintFactory.connectStartToEnd(activity);
			} catch (Exception e) {
				logger.error("Constructor for " + activityClass.getSimpleName()
							 + " threw exception "
							 + e.getCause().getMessage());
				StringWriter stackWriter = new StringWriter();
				e.printStackTrace(new PrintWriter(stackWriter));
				logger.error("Stack trace of cause: " + stackWriter.getBuffer());
			}
		} else {
			logger.error("Trying to initialize non-activity class: " + activityClass.getSimpleName());
		}
		return activity;
	}

	/**
     * 
     * Add a new structure critic to the ActivityStructureCritic if it doesn't
     * already exist there, in which case we simply add the activityClass as one
     * of the applicable classes for that critic.
     * 
     * @param newCritic
     * @param activityClass
     * @param criticName
     * @return
     */
	private static StructureCritic getStructureCritic(StructureCritic newCritic,
													  Class<? extends Activity> activityClass,
													  String criticName) {

		ActivityStructureCritic singleton = ActivityStructureCritic.getSingleton();
		StructureCritic critic = singleton.getStructureCritic(criticName);
		if (critic == null) {
			singleton.addStructureCritic(newCritic);
			logger.debug("adding new structure critic \""+newCritic.getName()+"\"");
			return newCritic;
		} else {
			critic.addApplicableClass(activityClass);
			logger.debug("Adding applicable class "+activityClass.getSimpleName()+" to \""+critic.getName()+"\"");
			return critic;
		}

	}

	static StructureCritic createAgentAvailableCritic(final Class<? extends Activity> activityClass) {
		final String noAgentCriticName = "agent checker";
		StructureCritic noAgentCritic = new StructureCritic() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			/**
             * This structure critic should only be applicable to
             * <code>activityClass</code>
             * 
             * @see critiquer.constraints.StructureCritic#addApplicableClasses()
             */
			@Override
			public void addApplicableClasses() {
				applicableClasses.add(activityClass);
			}

			@Override
			public String getDescription() {
				return "checks to see that there are agents assigned to the activity";
			}

			@Override
			public String getName() {
				return noAgentCriticName;
			}

			@Override
			public Pair<Boolean, String> check(Activity activity) {

				Pair<Boolean, String> checkVal = new Pair<Boolean, String>(true, "");
				HashSet<Agent> agentsInActivity = ActivityUtils.getAgentsInActivity(activity);
				if (agentsInActivity.isEmpty()) {
					checkVal.a = false;
					checkVal.b += activity + " must have at least one agent assigned";
				}
				return checkVal;
			}

		};
		return getStructureCritic(noAgentCritic, activityClass, noAgentCriticName);

	}

	/**
     * @param class1
     * @return
     */
	public static StructureCritic createTransportLocationsCritic(final Class<? extends Activity> activityClass) {
		final String criticName = "transport activity critic";
		StructureCritic transportCritic = new StructureCritic() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			/**
             * Only applicable for the type of activity specified as parameter
             * to createTransportLocationsCritic
             * 
             * @see critiquer.constraints.StructureCritic#addApplicableClasses()
             */
			@Override
			public void addApplicableClasses() {
				applicableClasses.add(activityClass);
			}

			@Override
			public String getDescription() {
				return "ensures that there are start and end locations defined";
			}

			@Override
			public String getName() {
				return criticName;
			}

			@Override
			public Pair<Boolean, String> check(Activity activity) {
				Pair<Boolean, String> checkVal = new Pair<Boolean, String>(true, "");
				if (activity instanceof TransportationActivity) {
					TransportationActivity transportActivity = (TransportationActivity) activity;
					Location startLoc = transportActivity.getStartLocation();
					Location endLoc = transportActivity.getEndLocation();
					if (startLoc == null || startLoc.getArea() == null) {
						checkVal.a = false;
						checkVal.b += "start location of " + activity + " is not defined\n";
					}
					if (endLoc == null || endLoc.getArea() == null) {
						checkVal.a = false;
						checkVal.b += "end location of " + activity + " is not defined\n";
					}

				} else {
					checkVal.a = false;
					checkVal.b += activity + " is not a transportation activity\n";
				}
				return checkVal;
			}

		};
		return getStructureCritic(transportCritic, activityClass, criticName);
	}

	public static StructureCritic createNoInjuredCritic(final Class<? extends Activity> activityClass) {
		final String noInjuredCritic = "no injured-critic";
		StructureCritic injuredCritic = new StructureCritic() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			/*
             * (non-Javadoc)
             * 
             * @see critiquer.constraints.StructureCritic#addApplicableClasses()
             */
			@Override
			public void addApplicableClasses() {
				applicableClasses.add(activityClass);
			}

			@Override
			public Pair<Boolean, String> check(Activity activity) {
				Pair<Boolean, String> checkVal = new Pair<Boolean, String>(true, "");
				try {
					activity.getClass().getMethod("getInjured", new Class[] {}).invoke(activity, new Object[] {});
				} catch (NullPointerException e) {
					checkVal.a = false;
					checkVal.b = activity + " must specify an how many patients are injured";
				} catch (IllegalArgumentException exception) {
					// TODO Auto-generated catch block
					exception.printStackTrace();
				} catch (SecurityException exception) {
					// TODO Auto-generated catch block
					exception.printStackTrace();
				} catch (IllegalAccessException exception) {
					// TODO Auto-generated catch block
					exception.printStackTrace();
				} catch (InvocationTargetException exception) {
					// TODO Auto-generated catch block
					exception.printStackTrace();
				} catch (NoSuchMethodException exception) {
					// TODO Auto-generated catch block
					exception.printStackTrace();
				}
				return checkVal;
			}

			@Override
			public String getDescription() {
				return "ensure that there is a parameter for how many injured patients there are";
			}

			@Override
			public String getName() {
				return noInjuredCritic;
			}

		};
		return getStructureCritic(injuredCritic, activityClass, noInjuredCritic);
	}

}
