/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.util.HashMap;
import java.util.Vector;

import critiquer.activities.TransportationActivity;
import critiquer.activities.Activity.Properties;
import critiquer.misc.Utils;
import critiquer.resources.Agent;
import critiquer.resources.AgentContributionCalculator;
import critiquer.resources.ParameterInAgent;
import critiquer.resources.SingleSelectionParameter;


/**
 * 
 * <p>
 * This class contains static factory methods for constructing parameters that
 * are part of <i>agents</i>, not activities
 * </p>
 */
public class StandardAgentParameterFactory {

	private static Vector<Long> createTimeRange() {
		Vector<Long> timeRange = new Vector<Long>();
		for (long i = 0; i < 200; i += 10) {
			timeRange.add(i);
		}

		return timeRange;
	}

	private static Vector<Integer> createCapacityRange() {
		Vector<Integer> capacityRange = new Vector<Integer>();
		for (int i = 0; i < 10; i++) {
			capacityRange.add(i);
		}
		return capacityRange;
	}

	public static void createFuelParameters(Agent agent, int fuelTankCapacity, int fuelConsumption) {
		agent.addParameter(new SingleSelectionParameter<Integer>(AgentParameterName.FUEL_CAPACITY,
																 agent,
																 Utils.toVector(new Integer[] { 10,
																		 30,
																		 50,
																		 70,
																		 90,
																		 110,
																		 130 }),
																 fuelTankCapacity));
		/**
         * Fuel consumption in litres per 100 kilometers
         */
		agent.addParameter(new SingleSelectionParameter<Integer>(AgentParameterName.FUEL_CONSUMPTION,
																 agent,
																 Utils.toVector(new Integer[] { 1,
																		 2,
																		 3,
																		 4,
																		 5,
																		 6,
																		 7,
																		 8,
																		 9,
																		 10,
																		 15,
																		 20,
																		 40,
																		 80 }),
																 new Integer(fuelConsumption)));
	}

	/**
     * 
     * Calculate the time it takes for an agent to perform a transportation
     * activity, along with possible side-effects. This method is meant to be used when calculating the effects of a single, isolated activity.
     * 
     * @param a
     * @param activity
     * @return the time in minutes it took to perform the transport
     */
	private static long performTransport(final Agent a,
										 TransportationActivity activity,
										 HashMap<Properties, Integer> activityAttributes) {
		
		/**
         * speed in kilometers per hour. Translate to meters per minute instead,
         * since we should return a value in minutes for this unit.
         */
		long speed = a.getParamByClassAndName(Long.class, AgentParameterName.SPEED).getValue() * 1000 / 60;
		double distance = 0;
		distance = activity.getStartLocation().distanceTo(activity.getEndLocation());

		return (long) (distance / speed);
	}

	/**
     * The speed parameter dictates how long time it will take to dispatch units
     * as well as to go between locations later.
     */
	public static void createSpeedParameter(final Agent a, long initialSpeed) {

		a.addParameter(new SingleSelectionParameter<Long>(AgentParameterName.SPEED, a, createTimeRange(), initialSpeed));

		/**
         * Calculate time for dispatch mission in minutes
         */
		a.missionContributionCalculators.put(DispatchActivity.class,
				new AgentContributionCalculator<DispatchActivity>() {

					/**
                     * 
                     */
					private static final long serialVersionUID = 1L;

					public long perform(DispatchActivity activity, HashMap<Properties, Integer> attributes) {
						return performTransport(a, activity, attributes);
					}

				});

	}

	/**
     * <p>
     * Creates parameters for taking care of patients, indicating both how many
     * this agent can cater at a time as well as how long it takes to
     * administer first aid to such a group
     * </p>
     */
	public static void createFirstAidParameters(final Agent a, long initialTime, int initialCapacity) {

		a.addParameter(new SingleSelectionParameter<Long>(AgentParameterName.FIRST_AID_TIME,
														  a,
														  createTimeRange(),
														  initialTime));

		a.addParameter(new SingleSelectionParameter<Integer>(AgentParameterName.FIRST_AID_CAPACITY,
															 a,
															 createCapacityRange(),
															 initialCapacity));

		a.missionContributionCalculators.put(FirstAidActivity.class,
				new AgentContributionCalculator<FirstAidActivity>() {

					/**
                     * 
                     */
					private static final long serialVersionUID = 1L;

					public long perform(FirstAidActivity activity, HashMap<Properties, Integer> attributes) {
						long time = a.getParamByClassAndName(Long.class, AgentParameterName.FIRST_AID_TIME).getValue();
						int patientsBefore = attributes.get(FirstAidActivity.Properties.PATIENTS);

						if (patientsBefore > 0) {
							int emergencyCapacity = a.getParamByClassAndName(Integer.class,
									AgentParameterName.FIRST_AID_CAPACITY).getValue();
							int patientsAfter = Math.max(0, patientsBefore - emergencyCapacity);
							attributes.put(FirstAidActivity.Properties.PATIENTS, patientsAfter);
							return time;
						} else {
							return 0;
						}

					}

				});

	}

	/**
     * <p>
     * Creates a parameter that describes the transportation capacity of the
     * agent.
     * </p>
     * <p>
     * This requires a speed parameter to be present and will create one if it
     * doesn't exist.
     * </p>
     * <p>
     * Also, we hook up a time calculator for RecoverActivities here.
     * </p>
     */
	public static void createTransportParameter(final Agent a, int initialCapacity) {

		ParameterInAgent<Long> speedParameter = a.getParamByClassAndName(Long.class, AgentParameterName.SPEED);
		if (speedParameter == null) {
			createSpeedParameter(a, 100);
		}
		a.addParameter(new SingleSelectionParameter<Integer>(AgentParameterName.TRANSPORT_CAPACITY,
															 a,
															 createCapacityRange(),
															 initialCapacity));

		a.missionContributionCalculators.put(RecoverActivity.class, new AgentContributionCalculator<RecoverActivity>() {

			/**
             * 
             */
			private static final long serialVersionUID = 1L;

			public long perform(RecoverActivity activity, HashMap<Properties, Integer> attributes) {
				int patientsBefore = attributes.get(RecoverActivity.Properties.PATIENTS);
				int transportCapacity = a.getParamByClassAndName(Integer.class, AgentParameterName.TRANSPORT_CAPACITY)
										 .getValue();
				int patientsAfter = Math.max(0, patientsBefore - transportCapacity);
				attributes.put(RecoverActivity.Properties.PATIENTS, patientsAfter);
				return performTransport(a, activity, attributes);
			}

		});
	}

	public static void createRoadBlockTimeParameter(final Agent a, long initialTime) {
		a.addParameter(new SingleSelectionParameter<Long>(AgentParameterName.ROADBLOCK_TIME,
														  a,
														  createTimeRange(),
														  initialTime));
		a.missionContributionCalculators.put(RoadBlockActivity.class,
				new AgentContributionCalculator<RoadBlockActivity>() {

					/**
                     * 
                     */
					private static final long serialVersionUID = 1L;


					public long perform(RoadBlockActivity activity, HashMap<Properties, Integer> attributes) {
						return a.getParamByClassAndName(Long.class, AgentParameterName.ROADBLOCK_TIME).getValue();			
					}

				});
	}

}
