/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.simulation;

import java.awt.Graphics2D;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.TransportationActivity;
import critiquer.gui.GeoView;
import critiquer.resources.Agent;
import critiquer.scenarios.standard.AgentParameterName;


/**
 * 
 * 
 * @author olale
 * 
 */
public class DefaultAgentSimulationState implements AgentSimulationState {

	/**
     * 
     */
	public static final String	  FUEL_LEVEL		 = "fuel level";

	/**
     * 
     */
	private static final long	   serialVersionUID   = 1L;

	private HashMap<String, Object> state			  = new HashMap<String, Object>();

	private Agent				   agent;

	private SimulationEngine		engine;

	/**
     * This is the list of all activities this agent participates in, sorted in
     * temporal order
     */
	private ArrayList<Activity>	 activitiesForAgent = new ArrayList<Activity>();

	private transient Logger				  logger			 = Logger.getLogger(getClass());

	/**
     * 
     */
	public DefaultAgentSimulationState(Agent agent) {
		this.agent = agent;
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.resources.AgentSimulationState#setPropertyMap(java.util.HashMap)
     */

	public void setProperty(String key, Object value) {
		state.put(key, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.resources.AgentSimulationState#getProperty(java.lang.String,
     *      java.lang.Class)
     */
	@SuppressWarnings("unchecked")
	public Object getProperty(String name) {
		Object returnValue = null;
		if (state.containsKey(name)) {
			return state.get(name);
		}
		return returnValue;

	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.resources.AgentSimulationState#renderState(critiquer.simulation.GeoView,
     *      java.awt.Graphics2D)
     */
	public void renderState(GeoView view, Graphics2D g2) {
		// TODO Auto-generated method stub

	}

	/**
     * @see critiquer.simulation.AgentSimulationState#getAgent()
     */
	public Agent getAgent() {

		return agent;
	}

	/**
     * 
     * Called every time the simulation engine starts a simulation.
     * 
     * TODO: What is the semantics when simulating activities that have parents
     * and children? Should we simply assume that there is some sensible
     * implementation of this provided by the agents?
     * 
     * @see critiquer.simulation.SimulationElement#initSimulation()
     */
	public void initSimulation(SimulationEngine engine) {
		this.engine = engine;
		setProperties();
		activitiesForAgent = agent.getActivitiesForAgent();

		sortActivities(activitiesForAgent);
	}

	private void sortActivities(List<Activity> activities) {
		Collections.sort(activities, new Comparator<Activity>() {

			public int compare(Activity a1, Activity a2) {
				Calendar t1 = ActivityUtils.getEndTimeParameter(a1).getValue();
				Calendar t2 = ActivityUtils.getEndTimeParameter(a2).getValue();
				return t1.compareTo(t2);
			}

		});
	}

	/**
     * 
     * Step forward or backwards in time to the absolute time <code>time</code>
     * 
     * @param time
     *            the destination time
     */
	public void stepTo(Calendar time) {
		setProperties();
		ArrayList<Activity> activities = activitiesForAgent;
		for (Activity activity : activities) {
			Calendar endTime = ActivityUtils.getEndTimeParameter(activity).getValue();
			if (time.after(endTime)) {
				consumeActivity(activity);
			} else {
				logger.debug(activity + " comes after "
							 + DateFormat.getTimeInstance(DateFormat.SHORT).format(time.getTime())
							 + ", aborting");
				break;
			}
		}
	}

	/**
     * 
     * Move forward in the simulation with the agent
     * 
     * @see critiquer.simulation.SimulationElement#stepForward(int)
     */
	public void stepForward(int step) {
		Calendar nextTime = (Calendar) engine.getCurrentTime().clone();
		nextTime.add(Calendar.MINUTE, step);
		stepTo(nextTime);

	}

	/**
     * Set the properties that define this state.
     * 
     */
	protected void setProperties() {
		setProperty(FUEL_LEVEL, agent.getParamByClassAndName(Integer.class, AgentParameterName.FUEL_CAPACITY)
									 .getValue());
		// setProperty("time since rest", 0);
	}

	/**
     * When <code>activity</code> has finished, calculate how properties of
     * the agent are modified
     * 
     * @param activity
     */
	public void consumeActivity(Activity activity) {
		// TODO: There really should be a method of dispatching to the proper activity consumer here
		
		if (activity.canBeSimulated()) {
			if (activity instanceof TransportationActivity) {
				TransportationActivity transportActivity = (TransportationActivity) activity;
				// distance in 100 kilometers
				double distance = transportActivity.getStartLocation().distanceTo(transportActivity.getEndLocation()) / 100000;
				// fuel consumption in liters per 100 kilometers
				Integer fuelConsumption = getAgent().getParamByClassAndName(Integer.class,
						AgentParameterName.FUEL_CONSUMPTION).getValue();
				Integer fuelLevel = (Integer) getProperty(FUEL_LEVEL);

				int newFuelLevel = (int) (fuelLevel - fuelConsumption * distance);
				logger.debug("New fuel level of " + getAgent() + " after "+activity+": " + newFuelLevel);
				setProperty(FUEL_LEVEL, new Integer(newFuelLevel));
			}
			
		}
	}

	/**
     * @see critiquer.simulation.SimulationElement#stepBackward(int)
     */
	public void stepBackward(int step) {
		Calendar previousTime = (Calendar) engine.getCurrentTime().clone();
		previousTime.add(Calendar.MINUTE, step);
		stepTo(previousTime);
	}

}
