/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.simulation;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;

import critiquer.activities.parameters.Commander;
import critiquer.constraints.SimulationCritic;
import critiquer.constraints.VisualConstraint;
import critiquer.constraints.VisualConstraints;
import critiquer.misc.ReferenceHolder;
import critiquer.resources.Agent;



/**
 * 
 * This class represents an object that updates the state information of agents
 * that are part of the current simulation, as well as active constraints
 * 
 * @author olale
 * 
 */
public class SimulationEngine {

	private ArrayList<SimulationCritic> simulationCritics = new ArrayList<SimulationCritic>();
	private ArrayList<AgentSimulationState> agentSimulationStates = new ArrayList<AgentSimulationState>();
	private Calendar currentTime;

	public void init(Calendar startTime) {
		this.currentTime=startTime;
		/**
		 * Populate the simulation elements with critics and agents
		 */
		agentSimulationStates.clear();
		simulationCritics.clear();
		for (Commander commander : ReferenceHolder.commanders.getCommanders().values()) {
			HashSet<Agent> agents = commander.getSubordinates();
			for (Agent agent : agents) {
				agent.initAgentSimulationState(this);
				AgentSimulationState agentSimulationState = agent.getAgentSimulationState();
				agentSimulationStates.add(agentSimulationState);
			}
		}
		/**
		 * Add simulation critics as well
		 */
		for (VisualConstraint constraint : VisualConstraints.getActiveConstraints()) {
			if (constraint instanceof SimulationCritic) {
				SimulationCritic simulationCritic = (SimulationCritic) constraint;
				simulationCritic.initSimulation(this);
				simulationCritics.add(simulationCritic);
			}
		}
	}
	/**
	 * 
	 * Tell all simulation elements the simulation is advancing
	 * 
	 * @param step in minutes
	 */
	public void stepForward(int step) {
		for (SimulationElement simulationElement : agentSimulationStates) {
			simulationElement.stepForward(step);
		}
		for (SimulationElement simulationElement : simulationCritics) {
			simulationElement.stepForward(step);
		}
		
		currentTime.add(Calendar.MINUTE, step);
	}
	
	/**
	 * 
	 * Tell all simulation elements the simulation is advancing
	 * 
	 * @param step in minutes
	 */
	public void stepTo(Calendar time) {
		for (SimulationElement simulationElement : agentSimulationStates) {
			simulationElement.stepTo(time);
		}
		for (SimulationElement simulationElement : simulationCritics) {
			simulationElement.stepTo(time);
		}
		currentTime=time;
	}
	
	/**
	 * Reverse the simulation
	 * @param step in minutes. Negative figure.
	 */
	public void stepBackwards(int step) {
		for (SimulationElement simulationElement : agentSimulationStates) {
			simulationElement.stepForward(step);
		}
		for (SimulationElement simulationElement : simulationCritics) {
			simulationElement.stepBackward(step);
		}
		currentTime.add(Calendar.MINUTE, step);
	}
	
	/**
	 * @return Returns the currentTime.
	 */
	public Calendar getCurrentTime() {
		return currentTime;
	}
	/**
	 * @param currentTime The currentTime to set.
	 */
	public void setCurrentTime(Calendar currentTime) {
		this.currentTime = currentTime;
	}
	/**
	 * @return Returns the agentSimulationStates.
	 */
	public ArrayList<AgentSimulationState> getAgentSimulationStates() {
		return agentSimulationStates;
	}
	
	
}